/* $Id: cmain.cpp,v 1.44 2002/04/30 15:22:40 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <string>
#include <csignal>

#include "SDL.h"
#include "SDL_net.h"

#include "libArianneclient.h"
#include "Parser/Parser.h"
#include "Log/Logger.h"
#include "Conversion/Conversion.h"
#include "Conversion/StringTool.h"
#include "4_Object/Subtypes.h"

#include "InitPath.h"

std::string g_UserName;
std::string g_Password;
std::string g_RSAPrivateKeyLocation;
bool g_UseRSA=false;

/** The Arianne Ini file name */
#define INI_FILE "Client.xml"

void Initialization()
  {
  InitPath(INI_FILE);
  SDL_Init(SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE);
  }

void Exit()
  {
  SDL_Quit();
  }

bool ReadParams(int argc,char **argv)
  {
  if(argc==1) 
    {
    glog_App.Debug("SYNTAX: arianne-client -u <username> [-p <password>]|[-k <RSAKeyFile>] [-h]",Log::fatal_error);
    return false;
    }

  // scan the commandline for parameters
  for(int i=1; i<argc; ++i)
    {
    if(std::string(argv[i]) == std::string("-h"))
      {
      glog_App.Debug("SYNTAX: arianne-client -u <username> [-p <password>]|[-k <RSAKeyFile>] [-h]",Log::notice);
      return false;
      }
    else if(std::string(argv[i]) == std::string("-u"))
      {
      ++i;
      if (i != argc)
          g_UserName = argv[i];
      }
    else if(std::string(argv[i]) == std::string("-p"))
      {
	  if (g_Password != "" || g_RSAPrivateKeyLocation != "")
		{
		glog_App.Debug("SYNTAX: arianne-client -u <username> [-p <password>]|[-k <RSAKeyFile>] [-h]",Log::fatal_error);
		return false;
		}

      ++i;
      if (i != argc)
          g_Password = argv[i];
      }
    else if(std::string(argv[i]) == std::string("-k"))
      {
	  if (g_Password != "" || g_RSAPrivateKeyLocation != "")
		{
		glog_App.Debug("SYNTAX: arianne-client -u <username> [-p <password>]|[-k <RSAKeyFile>] [-h]",Log::fatal_error);
		return false;
		}

	  ++i;
          if (i != argc)
	  {
              g_RSAPrivateKeyLocation = argv[i];
              g_UseRSA=true;
	  }
      }
    }
  
  if (g_UserName == "" || (g_Password == "" && g_RSAPrivateKeyLocation == "")) 
	{
	glog_App.Debug("SYNTAX: arianne-client -u <username> [-p <password>]|[-k <RSAKeyFile>] [-h]",Log::fatal_error);
	return false;
	}

  return true;
  }

int g_actionsToRecieve=9;

void ProcessAction(std::list<Action> const& action)
  {
  for(std::list<Action>::const_iterator i=action.begin();i!=action.end();++i)
    {
    if((*i).HasAttrib("status"))
      {
      std::cout << "COMPLETE Action: " << (*i).GetType() << " from " << (*i).GetSource() << " has status " << (*i).GetAttrib("status") << std::endl;
      g_actionsToRecieve--;
      continue;
      }

    if((*i).GetType()=="move")
      {
      std::cout << "Action: " << (*i).GetType() << " from " << (*i).GetSource();
      std::cout << " with Content \"" << ConvertCode2Spaces((*i).GetAttrib("destination")) << "\"" << std::endl;
      }

    if((*i).GetType()=="chat")
      {
      std::cout << "Action: " << (*i).GetType() << " from " << (*i).GetSource();
      std::cout << " with Content \"" << ConvertCode2Spaces((*i).GetAttrib("content")) << "\"" << std::endl;
      }

    if((*i).GetType()=="get")
      {
      std::cout << "Action: " << (*i).GetType() << " from " << (*i).GetSource() << std::endl;
      }

    if((*i).GetType()=="put")
      {
      std::cout << "Action: " << (*i).GetType() << " from " << (*i).GetSource() << std::endl;
      }
    }
  }

int main(int argc, char **argv)
  {
  if(!ReadParams(argc,argv))
    return -1;

  Initialization();
  glog_App.Debug("Initializing Client");
  signal(SIGSEGV,SIG_DFL);


  if(argc<2)
    {
    glog_App.Debug("SYNTAX: arianne-server -u <username> [-p <password>]|[-k <RSAKeyFile>] [-h]",Log::fatal_error);
    glog_App.Debug("Shutdowning down Client");
    Exit();
    return -1;
    }
 

  std::ifstream aInifile((g_etcdir+INI_FILE).c_str());
  if(!aInifile)
    {
    glog_App.Debug(g_etcdir+INI_FILE+" not available: Check instalation");
    glog_App.Debug("Shutdowning down Client");
    Exit();
    return -1;
    }

  PersistentString pvHostName (g_etcdir+INI_FILE, "ServerOptions", "HostName");
  PersistentInt pvPortNumber (g_etcdir+INI_FILE, "ServerOptions", "PortNumber");
  PersistentString pvRendererType (g_etcdir+INI_FILE, "Renderer", "Type", "libArianneTextRenderer");

  glog_App.Debug(std::string("Connecting to ")+std::string(pvHostName)+std::string(":")+IntToString((Uint32)pvPortNumber));

  // Let's define the variables that integrate with the *user interface*
  bool user_request_available=false;
  bool logout_request=false;
  bool send_action=false;
  Uint32 StartTime=SDL_GetTicks();

  //Let's define also our world structure.
  std::list<Object> world;


  ArianneClient client;

  // Creating here the client login information.
  ArianneClient::LoginInfo login;
  login.Host=pvHostName;
  login.Port=pvPortNumber;
  login.Name=g_UserName;
  if(g_UseRSA)
    {
    login.Type=ArianneClient::rsakey;
    }
  else
    {
    login.Type=ArianneClient::password;
    }

  login.Password=g_Password;
  login.RSAKeyLocation=g_RSAPrivateKeyLocation;

  // Gather info about kind of computer of statistical porpouses.
  ArianneClient::Info info;
  // TODO: Determine this on compile time.
#if defined(_WIN32)
  info.OS="Win32";
  info.CPU="i386";
#elif defined(__linux__)
  info.OS="Linux";
  info.CPU="i386";
#elif defined(__BEOS__)
  info.OS="BeOS";
  info.CPU="i386";
#elif defined(macintosh)
  info.OS="MacOS";
  info.CPU="PPC";
#else
  info.OS="Other";
#endif

  try
    { 
    // We proceed to login.
    client.Login(login);

    // and send info to the server.
    client.SendSystemInfo(info);

    while(!logout_request)
      {
      // We delay a bit
      SDL_Delay(500);

      // We collect messages from the Net and process them.
      client.PumpMessages();

      // if we have some input odf the user avaliable, we send it to the server.  
      if(user_request_available)
        {
        // TODO: Build list of actions.        
        client.SendAction(std::list<Action>());
        }

      if(client.isAvailablePerception())
        {
        Uint32 timestamp;
        std::list<Object> modified, deleted;
        client.RecievePerception(timestamp,modified,deleted);
        std::cout << "- Perception has " << modified.size() << " modified objects and " << deleted.size() << " deleted objects." << std::endl;

        // We recieve the Perception and go and update world.
        std::list<Object>::const_iterator i;

        for(i=deleted.begin();i!=deleted.end();++i)
          {
          //Delete object of the world.
          world.remove(*i);
          }

        for(i=modified.begin();i!=modified.end();++i)
          {
          //Add object to the world
          bool found=false;

          for(std::list<Object>::iterator j=world.begin();j!=world.end();++j)
            {
            if((*j).GetAttrib("id")==(*i).GetAttrib("id"))
              {
              // Replace with the updated object.
              *j=*i;
              found=true;
              break;
              }
            }

          if(!found)
            {
            world.push_back(*i);
            }          
	      }

        std::cout << "- Now World has " << world.size() << " object on it" << std::endl;
        }

      if(client.isAvailableAction())
        {
        std::list<std::list<Action> > action;
        client.RecieveAction(action);
	  
        for(std::list<std::list<Action> >::iterator i=action.begin();i!=action.end();++i)
          {
          //Apply action effects to objects.
          //No RP effects but SFX, GFX or AI for example.
          ProcessAction(*i);
          }
        }

      /* GUI Imitation Code.
         This part of the code is to mimic the user input.
      */

      // The User sent a action message.
      if((SDL_GetTicks()-StartTime> 30000) && (!send_action))
        {
        std::list<Action> actions;

        Action tmp1;
        tmp1.PutAttrib("type","chat");
        Object::ID source=client.GetController().front();
        tmp1.PutAttrib("source",source);
        tmp1.PutAttrib("content","Hi, This is the first Arianne ALPHA 2 Action Message");
        actions.push_back(tmp1);

        Action tmp2;
        tmp2.PutAttrib("type","move");
        tmp2.PutAttrib("source",source);
        tmp2.PutAttrib("destination","167 141 0");
        tmp2.PutAttrib("speed","10");
        actions.push_back(tmp2);

        Action tmp3;
        tmp3.PutAttrib("type","get");
        tmp3.PutAttrib("source",source);
        tmp3.PutAttrib("slot","r_hand");
        tmp3.PutAttrib("objectID","5");
        actions.push_back(tmp3);

        Action tmp4;
        tmp4.PutAttrib("type","move");
        tmp4.PutAttrib("source",source);
        tmp4.PutAttrib("destination","35 89 0");
        tmp4.PutAttrib("speed","15");
        actions.push_back(tmp4);

        Action tmp5;
        tmp5.PutAttrib("type","put");
        tmp5.PutAttrib("source",source);
        tmp5.PutAttrib("slot","r_hand");
        actions.push_back(tmp5);

        Action tmp6;
        tmp6.PutAttrib("type","chat");
        tmp6.PutAttrib("source",source);
        tmp6.PutAttrib("content","I get the object again to the Original location.");
        actions.push_back(tmp6);

        Action tmp7;
        tmp7.PutAttrib("type","get");
        tmp7.PutAttrib("source",source);
        tmp7.PutAttrib("slot","l_hand");
        tmp7.PutAttrib("objectID","5");
        actions.push_back(tmp7);

        Action tmp8;
        tmp8.PutAttrib("type","move");
        tmp8.PutAttrib("source",source);
        tmp8.PutAttrib("destination","167 141 0");
        tmp8.PutAttrib("speed","5");
        actions.push_back(tmp8);

        Action tmp9;
        tmp9.PutAttrib("type","put");
        tmp9.PutAttrib("source",source);
        tmp9.PutAttrib("slot","l_hand");
        actions.push_back(tmp9);

        client.SendAction(actions);
        send_action=true;
        }

      // The User requested to logout...
      //if(SDL_GetTicks()-StartTime> 90000)
      if(g_actionsToRecieve==0)
        {
        logout_request=true;
        }

	  }
 
    ArianneClient::Stats stats;

    stats.ClientID="Arianne Test Text Client";
    stats.ClientVersion="0.0.0";
    stats.Frames="";

    client.Logout(stats);
    }
  catch(std::exception const& e)
    {
    std::cerr << "Fatal error: " << e.what() << std::endl;
    }
  
  glog_App.Debug("Shutdowning down Client");
  Exit();

  exit(0);
  }

