/* $Id: GenericMessage.cpp,v 1.16 2002/03/07 20:37:38 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
    This an abstraction of a common Message.

    All the messages are translated to this generic messages.
***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "GenericMessage.h"
#include "DataTypes/Serializer.h"
#include "Conversion/Bzip2Compress.h"

GenericMessage::GenericMessage()
  {
  ID=0;
  TimeToken=0;
  Type=Flags=0;
  }

GenericMessage::GenericMessage(GenericMessage const& gm):
  ID(gm.ID),
  TimeToken(gm.TimeToken),
  Type(gm.Type),
  Flags(gm.Flags),
  Data(gm.Data)
  {
  }

GenericMessage::~GenericMessage()
  {
  }


GenericMessage& GenericMessage::operator=(GenericMessage const& gm)
  {
  if(&gm == this)
  {
     return *this;
  }
  ID=gm.ID;
  TimeToken=gm.TimeToken;
  Data=gm.Data;
  Type=gm.Type;
  Flags=gm.Flags;

  return *this;
  }


void GenericMessage::SetID(Uint32 ID)
  {
  this->ID=ID;
  }

Uint32 GenericMessage::GetID() const
  {
  return ID;
  }

Uint32 GenericMessage::GetTimeToken() const
  {
  return TimeToken;
  }


void GenericMessage::read(Serializer &s)
  {
  s >> Type;
  s >> Flags;

  Data.clear();
  s >> Data;

  if(Flags&BZIP2_COMPRESSED==BZIP2_COMPRESSED)
    {
    Bzip2Compress compress;
    Data=compress.Decode(Data);
    }
  }

void GenericMessage::write(Serializer &s) const
  {
  s << Type;

  /* We set a threshold value to use or not compressor. */
  if(Data.size()>128)
    {
    Bzip2Compress compress;

    Uint8 modFlags=(Flags|BZIP2_COMPRESSED);
    s << modFlags;
    s << compress.Encode(Data);
    }
  else
    {
    s << Flags;
    s << Data;
    }
  }

GenericMessage *GenericMessage::clone() const
{
  return new GenericMessage(*this);
}

