/* $Id: GenericMessage.h,v 1.17 2002/03/08 12:21:23 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ARIANNE_GenericMessage_H
#define ARIANNE_GenericMessage_H

#include <vector>

#include "Types.h"

class Serializer;

/* A few defines for compression handling. */
#define NO_COMPRESSED    0
#define BZIP2_COMPRESSED 1
#define ZLIB_COMPRESSED  2

/** This class stores a generic message type. */
struct GenericMessage
  {
  /** This identify the source of the message, that is always an Connection. */
  Uint32 ID;
  /** This variable defined when the message was created. */
  Uint32 TimeToken;

  /** This store the type of message */
  Uint8 Type;	
  /** This store additional info of message */
  Uint8 Flags;
  /** This contains the data in a portable way. */
  std::vector<Uint8> Data;

  GenericMessage();
  GenericMessage(GenericMessage const&);
  virtual ~GenericMessage();

  GenericMessage& operator=(GenericMessage const&);

  /** This set the player that owns the message. */
  void SetID(Uint32 ID);
  /** This returns the player that owns the message */
  Uint32 GetID() const;
  /** This returns the time at what message was recieved. */
  Uint32 GetTimeToken() const;

  void read(Serializer &s);
  void write(Serializer &s) const;

  /** Create a copy of the message, allocation memory and returns a pointer. */
  virtual GenericMessage *clone() const;
  };

#endif
