/* $Id: Message.h,v 1.16 2002/03/08 12:21:23 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ARIANNE_MESSAGE
#define ARIANNE_MESSAGE

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Types.h"
#include "Exceptions.h"

class Serializer;

#define ARIANNE_INVALID		    0

/* Admin group */
#define ARIANNE_SEND_VERSION	1
#define ARIANNE_NLOGIN	    	2
#define ARIANNE_LOGOUT		    3
#define ARIANNE_SYSTEM_INFO	    4

/* Game group */ 
#define ARIANNE_ACTION	    	5
#define ARIANNE_PERCEPTION	    6

/* Internal Engine feature */
#define ARIANNE_POISON			255

/* Sub Login Flags */
#define ARIANNE_SEND_LOGIN	    1
#define ARIANNE_METHOD_PASS  	2
#ifdef HAVE_CRYPTO
	#define ARIANNE_METHOD_RSA	3
#endif
#define ARIANNE_SEND_PASS	    4
#ifdef HAVE_CRYPTO
	#define ARIANNE_SEND_W2		5
	#define ARIANNE_SEND_M		6
	#define ARIANNE_SEND_S_M_	7
#endif
#define ARIANNE_SEND_CONFIRM	8

/** The abstract class for Messages */
class Message
  {
  protected:
  /** Type of the Message */
  Uint8 sType;

  /** Identificator of the connection from where the message was recieved/send */
  Uint32 ID;

  public:
  Message() {};
  Message(Message const& b): sType(b.sType),ID(b.ID) {};
  virtual ~Message() {};

  /** This is the type of the message */
  Uint8 GetType() {return sType;};

  /** This is the connection number that own the message */
  void SetID(Uint32 ID) { this->ID=ID; }
  /** This returns the connection number */
  Uint32 GetID() { return ID; }

  virtual void read(Serializer &s)=0;
  virtual void write(Serializer &s) const=0;

  virtual Message *clone() const =0;

  class ClassNotSerializableException: public Exception
    {
    public:
	ClassNotSerializableException(std::string const &where = "UNKNOWN"): Exception()
	  {
	  message = where + " - Class is not Serializable.";
	  }  
    };
  };

  
#endif
