/* $Id: Attributes.cpp,v 1.12 2002/04/26 13:55:16 doomster Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "Attributes.h"
#include "DataTypes/Serializer.h"
#include "Conversion/StringTool.h"

using namespace std;

Attributes::Attributes():
  m_timestamp(0)
  {
  }

Attributes::~Attributes()
  {
  }

Attributes::Attributes(Attributes const& rhs):m_timestamp(rhs.m_timestamp),m_attribs(rhs.m_attribs)
  {
  }

Attributes& Attributes::operator=(Attributes const& rhs)
  {
  if(this != &rhs)
  {
    m_attribs = rhs.m_attribs;
    // TODO: is this right ? What is the policy on the use of the timestamp ?
    m_timestamp = rhs.m_timestamp;
  }
  return *this;
  }

bool Attributes::operator==(Attributes const& b) const
  {
  return (m_attribs == b.m_attribs);
  }

bool Attributes::equal_withTimeStamp(Attributes const& b) const
  {
  if(m_timestamp == b.m_timestamp)
    {
    return true;
    }
  else
    {
    return Attributes::operator==(b);
    }
  }

bool Attributes::HasAttrib(std::string const& Attrib) const
  {
  return (m_attribs.find(Attrib)!=m_attribs.end());
  }
  
std::list<std::string> Attributes::ListAttribs() const
  {
  std::list<std::string> retval;
  std::map<std::string, std::string>::const_iterator iter;
  
  for(iter = m_attribs.begin(); iter != m_attribs.end(); iter++)
    {
    retval.push_back((*iter).first);
    }

  return(retval);
  }

void Attributes::PutAttrib(string const& key, string const& value)
  {
  m_timestamp++;

  pair<std::map<string,string>::iterator,bool> result=m_attribs.insert(pair<string,string>(key,value));
  if(result.second==false)
    {
    std::map<string,string>::iterator it=m_attribs.find(key);
    if(it!=m_attribs.end())
      {
      (*it).second=value;
      }
    else
      {
      throw AttributeNotFoundException();
      }
    }
  }

void Attributes::DelAttrib(std::string const& key)
  {
  m_timestamp++;

  m_attribs.erase(key);
  }


string Attributes::GetAttrib(string const& key) const
  {
  std::map<string,string>::const_iterator it=m_attribs.find(key);
  
  if(it!=m_attribs.end())
    {
    string tmp=(*it).second;
    return tmp;
    }

  throw std::exception();
  }

bool Attributes::empty() const
  {
  return m_attribs.empty();
  }


void Attributes::read(Serializer &s)
  {
  m_timestamp++;
  m_attribs.clear();
  
  Uint32 size;
  s >> size;

  for(int i=0;i<(int)size;i++)
    {
    string first,second;
    s >> first;
    s >> second;

    PutAttrib(first,second);
    }
  }

void Attributes::clear()
  {
  m_attribs.clear();
  }


void Attributes::write(Serializer &s) const
  {
  s << m_attribs.size();
  for(std::map<string,string>::const_iterator it=m_attribs.begin();it!=m_attribs.end();++it)
    {
    s << (*it).first << (*it).second;
    }
  }

void Attributes::read(std::istream &s)
  {
  m_timestamp++;
  m_attribs.clear();

  Uint32 size;
  s >> size;
  for(int i=0;i<(int)size;i++)
    {
    string type, value;
    s >> type >> value;

    PutAttrib(ConvertCode2Spaces(type),ConvertCode2Spaces(value));
    }
  }
 
void Attributes::write(std::ostream &s) const
  {
  s << m_attribs.size() << ' ';
  for(std::map<string,string>::const_iterator it=m_attribs.begin();it!=m_attribs.end();++it)
    {
    s << ConvertSpaces2Code((*it).first) << ' ' << ConvertSpaces2Code((*it).second) << ' ';
    }
  }

string& Attributes::operator[](const string& key)
  {
  m_timestamp++;
  return(m_attribs[key]);
  }
