/* $Id: Attributes.h,v 1.12 2002/04/26 13:55:16 doomster Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef Arianne_Attributes_H
#define Arianne_Attributes_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Types.h"
#include "DataTypes/Serializer.h"

#include <iostream>
#include <map>
#include <list>
#include <string>

//TODO: Change exceptions thrown by the methods for a Class Exception.
/** TODO: in order to trace changes in a set of Attributes, one has to remember
 the timestamp for each of them. Better approach would be to store the time
 of the last modification, preferably with something like SDL_GetTicks() or the
 the World's turn-counter. Then, you could simply compare with the time of the
 last check and identify everything newer as modified. */
/** TODO: in the assignment-operator, the timestamp is also simply copied,
leading to pretty unpredictable results. This behaviour is at least inconsistent
because methods like read and write are also a kind of assignment but they don't
care at all about the timestamp ... */

/** This is a generic Attributes class.*/
class Attributes
  {
  /** This member will allow us to know when the attrib was modified.
  Current (2002-02-27) use is to just increment this member whenever
  something is changed. */
  Uint32 m_timestamp;

  protected:
  /** The attributes the Attributes has. */
  std::map<std::string,std::string> m_attribs;

  public:
  Attributes();
  Attributes(Attributes const& b);
  virtual ~Attributes();
  virtual Attributes& operator=(Attributes const& b);

  /** this compares the contained attributes for equality */
  virtual bool operator==(Attributes const& b) const;
  /** in addition to the attributes, this also compares the timstamps */
  virtual bool equal_withTimeStamp(Attributes const& b) const;

  virtual void read(Serializer &s);
  virtual void write(Serializer &s) const;

  virtual void read(std::istream &s);
  virtual void write(std::ostream &s) const;

  /** Returns true if the Attributes has the given attribute. */
  bool HasAttrib(std::string const& Attrib) const;
  /** Returns the list of attributes in this instance */
  std::list<std::string> ListAttribs() const;
  /** Add the attribute to the Attributes replacing it if it already exists. */
  void PutAttrib(std::string const& key, std::string const& value);
  /** Delete the attribute if it exists. */
  void DelAttrib(std::string const& key);
  /** Get the value of an attribute. Throws a std::exception if the
  required attribute doesn't exist */
  std::string GetAttrib(std::string const& key) const;
  /** Returns a reference to an attribute. Inserts a new attribute if
  the given key was not found. */
  std::string& operator[](std::string const& key);
  
  /** This method clears all the attributes. */
  void clear();

  /** Returns true if the Attributes is empty. */
  virtual bool empty() const;

  class AttributeNotFoundException: public Exception
    {
    public:
      AttributeNotFoundException(std::string const& where = "UNKNOWN"): Exception()
      {
          message = where + " - Error: Attribute not found";
      }
    }; 
  };

inline std::ostream& operator<<(std::ostream& out, Attributes const& t)
  {
  t.write(out);
  return out;
  }

inline std::istream& operator>>(std::istream& in, Attributes& t)
  {
  t.read(in);
  return in;
  }

inline Serializer& operator<<(Serializer& s, std::list<Attributes> const& l)
  {
  s << (Uint32)l.size();

  std::list<Attributes>::const_iterator last = l.end();
  for(std::list<Attributes>::const_iterator it = l.begin(); it!=last; ++it)
    {
    s << *it;
    }
  return s;
  }

inline Serializer& operator>>(Serializer& s, std::list<Attributes>& l)
  {
  Uint32 size;
  s >> size;
  l.clear();

  while(size--)
    {
    Attributes tmp;
    s >> tmp;
    l.push_back(tmp);
    }

  return s;
  }


#endif
