/* $Id: Object.cpp,v 1.22 2002/04/05 14:00:43 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "Object.h"
#include "DataTypes/Serializer.h"
#include "Conversion/StringTool.h"

using namespace std;

const Object::ID Object::Invalid="";

Object::Object(): Attributes()
  {
  }

Object::~Object()
  {
  for(std::map<std::string,Object *>::iterator it=m_slots.begin();it!=m_slots.end();++it)
    {
    delete ((*it).second);
    (*it).second=0;
    }
  }

Object::Object(Object const& rhs):Attributes(rhs)
  {
  for(std::map<std::string,Object *>::const_iterator it=rhs.m_slots.begin();it!=rhs.m_slots.end();++it)
    {
    if((*it).second)
      {
      m_slots.insert(pair<string,Object*> ((*it).first, new Object(*((*it).second))));
      }
    else
      {
      m_slots.insert(pair<string,Object*> ((*it).first, 0));
      }
     
    }
  }

void Object::AddObject2Slot(std::string slot,Object const& object)
  {
  map<string,Object *>::iterator found=m_slots.find(slot);
  if(found!=m_slots.end())
    {
    if((*found).second==0)
      {
      (*found).second=new Object(object);
      return;
      }
    throw ObjectSlotUsedException();
    }

  throw ObjectNoSlotException();
  }

Object Object::RemoveObject2Slot(std::string slot)
  {
  Object tmp;

  map<string,Object *>::iterator found=m_slots.find(slot);
  if(found!=m_slots.end())
    {
    if((*found).second)
      {
      tmp=*(*found).second;

      delete (*found).second;
      (*found).second=0;
    
      return tmp;
      }
    else
      {
      throw ObjectSlotNoObjectException();
      }
    }

  throw ObjectNoSlotException();
  }

bool Object::isUsedSlot(std::string slot)
  {
  map<string,Object *>::iterator found=m_slots.find(slot);
  if(found!=m_slots.end())
    {
    if((*found).second==0)
      {
      return false;
      }
    else
      {
      return true;
      }
    }    

  throw ObjectNoSlotException();
  }

Object& Object::operator=(Object const& b)
  {
  Attributes::operator=(b);

  /* We empty the slot array, we need to free memory. */
  for(std::map<std::string,Object *>::iterator it_del=m_slots.begin();it_del!=m_slots.end();++it_del)
    {
    delete ((*it_del).second);
    (*it_del).second=0;
    }
  m_slots.clear();

  for(std::map<std::string,Object *>::const_iterator it=b.m_slots.begin();it!=b.m_slots.end();++it)
    {
    if((*it).second)
      {
      m_slots.insert(pair<string,Object*> ((*it).first, new Object(*((*it).second))));
      }
    else
      {
      m_slots.insert(pair<string,Object*> ((*it).first, 0));
      }     
    }

  return *this;
  }

bool Object::operator==(Object const& b) const
  {
  //TODO: Need to consider too slots.
  bool result=Attributes::operator==(b);

  return result;
  }


bool Object::empty() const
  {
  //TODO: Need to consider too slots.
  bool result=Attributes::empty();

  return result;
  }


void Object::read(Serializer &s)
  {
  //TODO: Need to consider too slots.
  Attributes::read(s);
  }

void Object::write(Serializer &s) const
  {
  //TODO: Need to consider too slots.
  Attributes::write(s);
  }

void Object::read(std::istream &s)
  {
  Attributes::read(s);

  string tmp;
  s >> tmp;

  /* We empty the slot array, we need to free memory. */
  for(std::map<std::string,Object *>::iterator it=m_slots.begin();it!=m_slots.end();++it)
    {
    delete ((*it).second);
    (*it).second=0;
    }
  m_slots.clear();

  int num_slots=atoi(tmp.c_str());
  for(Uint8 i=0;i<num_slots;i++)
    {
    string slot_name;
    Object object;

    /* WARNING: Again, note that the algorithm is recursive. */
    s >> slot_name >> object;
    if(object.empty())
      {
      /* If Object is empty we show it. */
      m_slots.insert(pair<string,Object *>(slot_name,0));
      }
    else
      {
      m_slots.insert(pair<string,Object *>(slot_name,new Object(object)));
      }
    }
  }
 
void Object::write(std::ostream &s) const
  {
  Attributes::write(s);

  s << m_slots.size() << " ";
  for(std::map<std::string,Object *>::const_iterator it=m_slots.begin();it!=m_slots.end();++it)
    {
    /* The slot can be empty, so we show that event. */
    if((*it).second)
      {
      /* WARNING: Note that this algorithm is recursive.
         We should verify that it is not possible that an object include itself. */
      s << (*it).first << " " << (*(*it).second);
      }
    else
      {
      s << (*it).first << " " << Object();
      }
    }
  }
