/* $Id: Object.h,v 1.24 2002/04/05 23:38:54 mutable Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef Arianne_Object_H
#define Arianne_Object_H
	
#include "Attributes.h"

//TODO: Slots needs to be added to the read/write methods.

/** This is a generic object class.*/
class Object: public Attributes
  {
  friend class ObjectClassManager;

  // TODO: I am using a pointer because I get and error about 
  // 'second' uses undefined class 'Object'
  std::map<std::string,Object *> m_slots;

  public:
  Object();
  Object(Object const& b);
  Object(Attributes const &attribs): Attributes(attribs) { }
  ~Object();
  Object& operator=(Object const& b);

  /* Specific methods for accessing the Slots */
  void AddObject2Slot(std::string slot,Object const& Object);
  Object RemoveObject2Slot(std::string slot);
  bool isUsedSlot(std::string slot);

  bool operator==(Object const& b) const;

  void read(Serializer &s);
  void write(Serializer &s) const;

  void read(std::istream &s);
  void write(std::ostream &s) const;

  /** Returns true if the object is empty. */
  bool empty() const;

  /** The type definition for identifiers. */
  typedef std::string ID;
  static const Object::ID Invalid;

  class ObjectNotFoundException: public Exception
    {
    public:
	  ObjectNotFoundException(std::string const& where = "UNKNOWN"): Exception()
	  {
	      message = where + " - Error: Object don't found";
	  }
    }; 
    
  class ObjectSlotException: public Exception
    {
    protected:
	ObjectSlotException(std::string const& where = "UNKNOWN"): Exception()
      {
      }
    };

  class ObjectSlotUsedException: public ObjectSlotException
    {
    public:
	ObjectSlotUsedException(std::string const& where = "UNKNOWN"): ObjectSlotException()
      {
      message = where + " - Error: Slots is being used. Can't add object.";
      }
    };

  class ObjectNoSlotException: public ObjectSlotException
    {
    public:
	ObjectNoSlotException(std::string const& where = "UNKNOWN"): ObjectSlotException()
      {
      message = where + " - Error: Slots does not exist.";
      }
    };

  class ObjectSlotNoObjectException: public ObjectSlotException
    {
    public:
	ObjectSlotNoObjectException(std::string const& where = "UNKNOWN"): ObjectSlotException()
      {
      message = where + " - Error: Slots does not own any object.";
      }
    };
  };

inline std::ostream& operator<<(std::ostream& out, Object const& t)
  {
  t.write(out);
  return out;
  }

inline std::istream& operator>>(std::istream& in, Object& t)
  {
  t.read(in);
  return in;
  }

inline Serializer& operator<<(Serializer& s, std::list<Object> const& l)
  {
  s << (Uint32)l.size();

  std::list<Object>::const_iterator last = l.end();
  for(std::list<Object>::const_iterator it = l.begin(); it!=last; ++it)
    {
    s << (Object)(*it);
    }
  return s;
  }

inline Serializer& operator>>(Serializer& s, std::list<Object>& l)
  {
  Uint32 size;
  s >> size;
  l.clear();

  while(size--)
    {
    Object tmp;
    s >> tmp;
    l.push_back(tmp);
    }

  return s;
  }


#endif
