/* $Id: ObjectClassManager.cpp,v 1.7 2002/04/08 23:25:45 doomster Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "ObjectClassManager.h"

#include "Log/Logger.h"
#include "../basic/InitPath.h"
// TODO one of these should be redundant
#include <fstream>
#include <cstdio>

using namespace std;

ObjectClassManager::ObjectClassManager()
  {
  }

ObjectClassManager::~ObjectClassManager()
  {
  }

/** This load all the templates included in the config file */  
void ObjectClassManager::Load(std::string const& conf)
  {
#if 1
  std::ifstream mainfile(conf.c_str());
  if(!mainfile)
    {
    std::string msg = "ObjectClassManager::Load('" + conf + "'): failed to open file\n";
    glog_Layer4.Debug(msg, Log::error);
    throw ObjectClassFileNotFoundException();  
    }

  while(mainfile)
    {
    std::string line;
    std::getline( mainfile, line);
    // skip empty line and those that begin with a '#'
    if(line.empty())
      continue;
    if('#' == line[0])
      continue;

    //TODO: Convert to valid PATH
    std::string filename = g_sharedir + line;

    ObjectClassInfo tmp;
    try{
      tmp.Load(filename);
    }catch(...){
      std::string msg = "ObjectClassManager::Load(): failed to load ObjectClassInfo '" + filename + "'.\n";
      glog_Layer4.Debug(msg, Log::error);
      throw;
    }

    pair<std::map<ObjectClassInfo::Type,ObjectClassInfo>::iterator,bool> found=m_Classes.insert(pair<ObjectClassInfo::Type,ObjectClassInfo>(tmp.ClassType,tmp));
    //     pair<std::map<ClassInfo>::iterator, bool> found;
    //    found = m_Classes.insert(ClassInfo(tmp.ClassType, tmp));
          
    if(!found.second)
      {
      // This element has been inserted twice...
      m_Classes[tmp.ClassType]=tmp;
      std::string msg = "ObjectClassManager::Load(): WARNING: ELEMENT '" + filename + "' inserted twice. Check Files.\n";
      glog_Layer4.Debug( msg, Log::warning);
      }
    }
  if(!mainfile.eof())
  {
    std::string msg = "ObjectClassManager::Load('" + conf + "'): failure parsing file\n";
    glog_Layer4.Debug(msg, Log::warning);
  }
#else
  FILE *Handle=fopen(conf.c_str(),"rb");

  if(Handle)
    {
    while(!feof(Handle))
      {
      char buffer[256];
      
      if(fscanf(Handle,"%s",buffer)!=EOF)
        {
        string file_name=buffer;
        if(!file_name.empty())
          {
          ObjectClassInfo tmp;
          //TODO: Convert to valid PATH
          tmp.Load(file_name);

          pair<std::map<ObjectClassInfo::Type,ObjectClassInfo>::iterator,bool> found=m_Classes.insert(pair<ObjectClassInfo::Type,ObjectClassInfo>(tmp.ClassType,tmp));
          
          if(!found.second)
            {
            // This element has been inserted twice...
            m_Classes[tmp.ClassType]=tmp;
            glog_Layer4.Debug("ELEMENT inserted twice. Check Files.",Log::warning);
            }
          }
        }
      }


    fclose(Handle);
    return;
    }

  throw ObjectClassFileNotFoundException();  
#endif
  }

/** This returns a copy of the Object Class information. */
ObjectClassInfo ObjectClassManager::Get(ObjectClassInfo::Type const& obj) const
  {
  std::map<ObjectClassInfo::Type,ObjectClassInfo>::const_iterator found=m_Classes.find(obj);
  if(found!=m_Classes.end())
    {
    return (*found).second;
    }

  throw ObjectClassEntryNotFoundException();
  }

Object ObjectClassManager::Create(string object_type) const
  {
  ObjectClassInfo objectclass=Get(object_type);
  Object tmp;
  
  tmp.PutAttrib("type",objectclass.ClassType);
  tmp.PutAttrib("volume",objectclass.ClassVolume);
  tmp.PutAttrib("weight",objectclass.ClassWeight);

  std::map<string,std::pair<string,std::list<string> > >::const_iterator it;
  for(it=objectclass.ClassAttributes.begin();it!=objectclass.ClassAttributes.end();++it)
    {
    // TODO: Add random value to modify the stats.
    tmp.PutAttrib((*it).first,(*it).second.first);
    }

  std::list<string>::const_iterator it_1;
  for(it_1=objectclass.ClassSlots.begin();it_1!=objectclass.ClassSlots.end();++it_1)
    {
    tmp.m_slots.insert(pair<string,Object*> (*it_1,0));
    }
  
  return tmp;
  }
