/* $Id: ObjectLoad.cpp,v 1.9 2002/04/09 12:06:24 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "ObjectLoad.h"
#include "XML/tinyxml.h"
#include "Conversion/StringTool.h"
#include "InitPath.h"
#include "Log/Logger.h"

#include <iostream>
#include <algorithm>

using namespace std;

void IterateXMLTree(TiXmlNode *child,ObjectClassInfo &dest, std::list<string>& Enviroment);


void ObjectClassInfo::Load(string filename) 
  {
  //FIXME-MAC: fix this...
  #ifdef MACOSX
    string name = g_sharedir + filename;
    filename = name;
  #endif

  TiXmlDocument doc(filename);
  if(!doc.LoadFile())
    {
    glog_Common.Debug(doc.ErrorDesc(),Log::fatal_error);
    throw ObjectLoadFileNotFoundException(filename);
    }

  TiXmlDocument *parent=&doc;

  std::list<string> Enviroment;
  IterateXMLTree(parent,*this,Enviroment);  
  }

/* ARIANNE Object Load specific */
void AssignVariableValue(ObjectClassInfo &dest,string const& value, std::list<string> const& Enviroment)
  {
  std::list<string> VarGeneral;
  VarGeneral.push_back("Object");
  VarGeneral.push_back("General");

  if(find_list(Enviroment.begin(),Enviroment.end(),VarGeneral))
    {
    if(find_string(Enviroment.begin(),Enviroment.end(),"Type"))
      {      
      dest.ClassType=value;
      return;
      }
    if(find_string(Enviroment.begin(),Enviroment.end(),"Parent"))
      {
      dest.ClassParent=value;
      return;
      }
    if(find_string(Enviroment.begin(),Enviroment.end(),"Description"))
      {
      dest.ClassDescription=value;
      return;
      }
    if(find_string(Enviroment.begin(),Enviroment.end(),"Class"))
      {
      dest.ClassIsPassive=(value=="passive");
      return;
      }
    }

  std::list<string> VarCommon;
  VarCommon.push_back("Object");
  VarCommon.push_back("Common");
  if(find_list(Enviroment.begin(),Enviroment.end(),VarCommon))
    {
    if(find_string(Enviroment.begin(),Enviroment.end(),"Volume"))
      {      
      dest.ClassVolume=value;
      return;
      }
    if(find_string(Enviroment.begin(),Enviroment.end(),"Weight"))
      {
      dest.ClassWeight=value;
      return;
      }

    std::list<string> VarSlot;
    VarSlot.push_back("Slots");
    if(find_list(Enviroment.begin(),Enviroment.end(),VarSlot))
      {
      if(find_string(Enviroment.begin(),Enviroment.end(),"Type"))
        {      
        dest.ClassSlots.push_back(value);
        return;
        }
      }
    
    std::list<string> VarForSlot;
    VarForSlot.push_back("ForSlots");
    if(find_list(Enviroment.begin(),Enviroment.end(),VarForSlot))
      {
      if(find_string(Enviroment.begin(),Enviroment.end(),"Type"))
        {      
        dest.ClassForSlots.push_back(value);
        return;
        }
      }

    std::list<string> VarPropierties;
    VarPropierties.push_back("Properties");
    if(find_list(Enviroment.begin(),Enviroment.end(),VarPropierties))
      {
      std::list<string> VarOnAction;
      VarOnAction.push_back("onAction");
      if(find_list(Enviroment.begin(),Enviroment.end(),VarOnAction))
        {
        if(find_string(Enviroment.begin(),Enviroment.end(),"Execute"))
          {      
          dest.ClassEventOnAction.insert(pair<string,string>(Enviroment.back(),value));
          return;
          }
        }

      std::list<string> VarOnDestruction;
      VarOnDestruction.push_back("onDestruction");
      if(find_list(Enviroment.begin(),Enviroment.end(),VarOnDestruction))
        {
        if(find_string(Enviroment.begin(),Enviroment.end(),"Create"))
          {      
          dest.ClassEventOnDestruction.push_back(pair<string,int>(Enviroment.back(),atoi(value.c_str())));
          return;
          }
        }
      }

    std::list<string> VarAttributes;
    VarAttributes.push_back("Attributes");
    VarAttributes.push_back("Entry");
    if(find_list(Enviroment.begin(),Enviroment.end(),VarAttributes))
      {
      if(find_string(Enviroment.begin(),Enviroment.end(),"Default"))
        {
        list<string>::const_iterator found=find(Enviroment.begin(),Enviroment.end(),"Entry");
        if(found!=Enviroment.end())
          {
          found++; // Attribute name.
          string att_name=*found;
          map<string,pair<string,list<string> > >::iterator found_att=dest.ClassAttributes.find(att_name);
          if(found_att!=dest.ClassAttributes.end())
            {
            dest.ClassAttributes[att_name].first=value;
            }
          else
            {
            dest.ClassAttributes.insert(pair<string,pair<string,list<string> > >(att_name,pair<string,list<string> >(value,list<string>())));
            }
          }
        return;
        }

      if(find_string(Enviroment.begin(),Enviroment.end(),"Values"))
        {
        list<string>::const_iterator found=find(Enviroment.begin(),Enviroment.end(),"Entry");
        if(found!=Enviroment.end())
          {
          found++; // Attribute name.
          string att_name=*found;
          map<string,pair<string,list<string> > >::iterator found_att=dest.ClassAttributes.find(att_name);
          if(found_att!=dest.ClassAttributes.end())
            {
            dest.ClassAttributes[att_name].second.push_back(value);
            }
          else
            {
            list<string> values;
            values.push_back(value);
            // There was no default field, so we set it to *any*
            dest.ClassAttributes.insert(pair<string,pair<string,list<string> > >(att_name,pair<string,list<string> >("*any*",values)));
            }
          }
        return;
        }
      }

    std::list<string> VarContainer;
    VarContainer.push_back("Container");
    if(find_list(Enviroment.begin(),Enviroment.end(),VarContainer))
      {
      if(find_string(Enviroment.begin(),Enviroment.end(),"Type"))
        {      
        dest.ClassContainerType.second.push_back(value);
        return;
        }
      }

    std::list<string> VarContainable;
    VarContainable.push_back("Containable");
    if(find_list(Enviroment.begin(),Enviroment.end(),VarContainable))
      {
      if(find_string(Enviroment.begin(),Enviroment.end(),"Volume"))
        {      
        dest.ClassContainerType.first=value;
        return;
        }
      }
    }

  // Seems that this XML file is not an Arianne one... :-/

  cout << "Error Enviroment:" << std::endl;
  for(std::list<string>::const_iterator it_env=Enviroment.begin();it_env!=Enviroment.end();++it_env)
    {
    cout << *it_env << " ";
    }
  cout << value << std::endl;

  throw ObjectClassInfo::ObjectLoadParseException();
  }

/* ARIANNE Object Load Specific */
std::list<string> LookForAttributes(std::list<string> const& Enviroment)
  {
  std::list<string> Env1;
  Env1.push_back("Properties");
  Env1.push_back("Create");

  if(find_list(Enviroment.begin(),Enviroment.end(),Env1))
    {
    std::list<string> Tags;
    Tags.push_back("type");

    return Tags;
    }

  std::list<string> Env1_1;
  Env1_1.push_back("Properties");
  Env1_1.push_back("Execute");

  if(find_list(Enviroment.begin(),Enviroment.end(),Env1_1))
    {
    std::list<string> Tags;
    Tags.push_back("type");

    return Tags;
    }

  std::list<string> Env3;
  Env3.push_back("Attributes");
  Env3.push_back("Entry");

  if(find_list(Enviroment.begin(),Enviroment.end(),Env3))
    {
    std::list<string> Tags;
    Tags.push_back("name");
    Tags.push_back("default");

    return Tags;
    }

  return std::list<string>();
  }

void IterateXMLTree(TiXmlNode *parent,ObjectClassInfo &dest,std::list<string>& Enviroment)
  {
  TiXmlNode *child;

  for(child=parent->IterateChildren(0);child;child=parent->IterateChildren(child))
    {
    switch(child->Type())
      {
      case TiXmlNode::ELEMENT:
        {
        Enviroment.push_back(child->Value());
        int i=1;

        /* Here we should look for Attributes. */
        std::list<string> listAttributes=LookForAttributes(Enviroment);
        for(std::list<string>::iterator it=listAttributes.begin();it!=listAttributes.end();++it)
          {
          const string *tmp=child->ToElement()->Attribute(*it);
          if(tmp)
            {
            // We add the attribute to the Enviroment.
            Enviroment.push_back(*tmp);            
            i++;
            }
          }

        IterateXMLTree(child,dest,Enviroment);

        for(int j=0;j<i;j++)
          {
          Enviroment.pop_back();
          }

        break;
        }
      case TiXmlNode::TEXT:
        {
        AssignVariableValue(dest,child->Value(),Enviroment);
        break;
        }
      }
    }
  }


std::ostream& operator <<(std::ostream& out, ObjectClassInfo const& dest)
  {
  out << "Class Type         : " << dest.ClassType << std::endl;
  out << "Parent Class Type  : " << dest.ClassParent << std::endl;
  out << "Class Description  : " << dest.ClassDescription << std::endl;
  out << "Class MetaType     : " << ((dest.ClassIsPassive==true)?"passive":"active") << std::endl;
  out << "Class Volume       : " << dest.ClassVolume << std::endl;
  out << "Class Weight       : " << dest.ClassWeight << std::endl;
  out << "Class Slots        : ";
  for(std::list<std::string>::const_iterator it=dest.ClassSlots.begin();it!=dest.ClassSlots.end();++it)
    {
    out<< *it << " ";
    }
  out << std::endl;

  out << "Class For Slots    : ";
  for(std::list<std::string>::const_iterator it_1=dest.ClassForSlots.begin();it_1!=dest.ClassForSlots.end();++it_1)
    {
    out<< *it_1 << " ";
    }
  out << std::endl;

  out << "Event OnDestruction: " << std::endl;
  for(std::list<std::pair<std::string,int> >::const_iterator it_2=dest.ClassEventOnDestruction.begin();it_2!=dest.ClassEventOnDestruction.end();++it_2)
    {
    out <<"  (" << (*it_2).first << "," << (*it_2).second << ") " << std::endl;
    }
  out << std::endl;

  out << "Event OnAction     : " << std::endl;
  for(std::map<string,string>::const_iterator it_3=dest.ClassEventOnAction.begin();it_3!=dest.ClassEventOnAction.end();++it_3)
    {
    out<< "  (" << (*it_3).first << " -> " << (*it_3).second << ") " << std::endl;
    }
  out << std::endl;

  std::map<string,std::pair<string,list<string> > >::const_iterator it_rp;

  out << "Attributes   : " << std::endl;
  for(it_rp=dest.ClassAttributes.begin();it_rp!=dest.ClassAttributes.end();++it_rp)
    {
    out << "  (" << (*it_rp).first << " -> " << (*it_rp).second.first << ")  \t[";
    for(std::list<std::string>::const_iterator it=(*it_rp).second.second.begin();it!=(*it_rp).second.second.end();++it)
      {
      out << *it << " ";
      }
    
    out  << "]" << std::endl;
    }
  out << std::endl;

  out << "Containable Volume : " << dest.ClassContainerType.first << std::endl;
  out << "Container Types    : ";
  for(std::list<std::string>::const_iterator it_4=dest.ClassContainerType.second.begin();it_4!=dest.ClassContainerType.second.end();++it_4)
    {
    out << *it_4 << " ";
    }
  out << std::endl;

  return out;
  }
