/* $Id: ObjectLoad.h,v 1.5 2002/04/05 14:00:43 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __object_loader_h_
#define __object_loader_h_


#include <string>
#include <utility>
#include <list>
#include <map>
#include <iostream>

#include "Exceptions.h"

struct ObjectClassInfo 
  {
  /* We define a few types for help us */
  typedef std::string Type;
  typedef std::string Description;
  typedef std::string SlotType;
  typedef std::string Attribute;
  typedef std::string Value;
  typedef std::string Action;
  typedef std::string Script;
  typedef std::string Weight;
  typedef std::string Volume;

  typedef int Amount;
  

  /** The Type of the class of object. */
  Type ClassType;
  /** The Type of the parent of this class of object.
      If there is no parent, then this field is "none". */
  Type ClassParent;
  /** The file that contains the description of this class of object */
  Description ClassDescription;
  /** Is the class of object passive? */
  bool ClassIsPassive;

  /** The size of the smallest cylinder that totally contain the class of object. */
  Volume ClassVolume;
  /** The weight of the class of object. */
  Weight ClassWeight;

  /** The slots the class of object has */
  std::list<SlotType> ClassSlots;
  /** The slots where the class of object can be used. 
      If there is no restriction, this field contains "any". */
  std::list<SlotType> ClassForSlots;

  /* TODO: It is not generic enough. */
  /** Objects and amount of them that are created on Object of that class Destruction */
  std::list<std::pair<Type,Amount> > ClassEventOnDestruction;
  /** Scripts that are executed when the class of object recieve an action. */
  std::map<Action,Script> ClassEventOnAction;
  
  /** Attributes that the class of object has. It is the attribute, its default 
      value and a list of valid values, that could be empty for no verification. */
  std::map<Attribute,std::pair<Value,std::list<Value> > > ClassAttributes;
  
  /** The size of the cylinder that represent the volume the object can contain
      and the type of container that the object is. */
  std::pair<Volume,std::list<Type> > ClassContainerType;


  void Load(std::string filename);

  class ObjectLoadFileNotFoundException: public Exception
    {
    public:
	  ObjectLoadFileNotFoundException(std::string const& File,std::string const& where = "UNKNOWN"): Exception()
	  { 
	      message = where + " - Error: Object Class File \""+File+"\" not found";
	  }
    }; 

  class ObjectLoadParseException: public Exception
    {
    public:
	  ObjectLoadParseException(std::string const& where = "UNKNOWN"): Exception()
	  {
	      message = where + " - Error: XML file is not an Arianne one.";
	  }
    }; 
  };


std::ostream& operator <<(std::ostream& out, ObjectClassInfo const& dest);

#endif

