/* $Id: Piece.cpp,v 1.26 2002/04/26 18:15:58 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "Piece.h"

#include <vector>
#include <stdio.h>

#include "Subtypes.h"
#include "Conversion/StringTool.h"

using namespace std;


Piece::Piece(Uint32 xi, Uint32 yi, Uint32 xe, Uint32 ye):
  x_init(xi),
  y_init(yi),
  x_end(xe),
  y_end(ye)
  {
  }

Piece::~Piece()
  {
  }


bool Piece::Load(std::istream &s,Object::ID &str)
  {
  int MaxUsedID = atoi(str.c_str());
  Uint32 size;

  s >> size;

  for(int i=0;i<(int)size;++i)
    {
    Object tmp;	

	tmp.read(s);      
    m_Objects[tmp.GetAttrib("id")] = tmp;
      
    // Convert the id to int
    Object::ID id_str = tmp.GetAttrib("id");
    int tmp_id = atoi(id_str.c_str());

    if(MaxUsedID<tmp_id)
	  {
 	  MaxUsedID=tmp_id;
  	  }
    }
  
  char buffer[16];
  sprintf(buffer,"%i",MaxUsedID);
  str=buffer;

  return true;
  }

bool Piece::Save(std::ostream &s)
  {
  s << m_Objects.size() << std::endl;

  std::map<std::string,Object>::iterator last=m_Objects.end();
  for(std::map<std::string,Object>::iterator i=m_Objects.begin();i!=last; ++i)
    {
	s << (*i).second << std::endl;
	}

  return true;
  }


bool Piece::isInside(Object const& object)
  {
  string pos=object.GetAttrib("position");
  vector<string> split(Split(pos));
    
  float x(atof(split[0].c_str()));
  float y(atof(split[1].c_str()));

  if(x >= x_init &&
     x <  x_end  &&
     y >= y_init &&
     y <  y_end)
    {
    return true;
    }

  return false;
  }


Object Piece::Get(Object::ID const& id) const
  {
  std::map<Object::ID,Object>::const_iterator i=m_Objects.find(id);
  if(i==m_Objects.end())  
    {
    throw Object::ObjectNotFoundException(__FILE_AND_LINE__);
    }
  else
    {
    return (*i).second;
    }
  }

void Piece::Add(Object const& object)
  {
  // TODO: add an assert(object.HasAttrib("id"));
  std::map<string, Object>::iterator i=m_Objects.find(object.GetAttrib("id"));
  if(i==m_Objects.end())
    {
    pair<string, Object> p(object.GetAttrib("id"),object);
    m_Objects.insert(p);
    }
  else
    {
    (*i).second=object;
    }
  }

void Piece::Del(Object::ID const& id)
  {
  m_Objects.erase(id);
  }

void Piece::Del(Object const&object)
  {
  Del(object.GetAttrib("id"));
  }


/** Returns objects in the given cylinder. */
std::list<Object> Piece::isInRange(string const& Center,Uint32 Distance)
  {
  std::list<Object> list;

  Position cent(Center);
 
  std::map<string,Object>::iterator last = m_Objects.end();
  for(std::map<string,Object>::iterator i=m_Objects.begin();i!=last; ++i)
    {
    string pos = (*i).second.GetAttrib("position");

    Position tmp_pos(pos);      

    Uint32 tmp=(Uint32)(tmp_pos.Distance(cent));
    // TODO Consider Volume in the computation.
    if (tmp < Distance)
      {
      list.push_back((*i).second);
      }
    }  

  return list;
  }

void Piece::Edit(Object::ID const &id, string& Attrib, string& NewVal)
  {
  std::map<string, Object>::iterator i=m_Objects.find(id);
  if(i==m_Objects.end())  
    {
    throw Object::ObjectNotFoundException(__FILE_AND_LINE__);
    }
  else
    {
    (*i).second.PutAttrib(Attrib, NewVal);
    }
  }
