/* $Id: Mutex.h,v 1.14 2002/03/08 12:21:24 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef MUTEX_H
#define MUTEX_H

#include "SDL_thread.h"
#include "Exceptions.h"

/** This is Mutex abstraction using SDL*/
class Mutex
  {
  SDL_mutex* m_SDLMutex;
  
  Mutex(const Mutex& b);
  Mutex& operator=(Mutex& b);

  public: 
  Mutex(bool locked = false);
  virtual ~Mutex();
  
  /** This blocks the caller if the Mutex is already locked. */
  void Lock();
  /** This unlock the mutex. */
  void Unlock();


  class MutexException: public Exception
  {
      protected:
	  MutexException(): Exception() {}
  };

  class MutexLockException: public MutexException
  {
      public:
	  MutexLockException(std::string const &where = "UNKNOWN"): MutexException()
	  {
	      message = where + " - Error locking mutex.";
	  }
  };

  class MutexUnlockException: public MutexException
  {
      public:
	  MutexUnlockException(std::string const &where = "UNKNOWN"): MutexException()
	  {
	      message = where + " - Error unlocking mutex.";
	  }
  };

  class MutexAllocException: public MutexException
  {
      public:
	  MutexAllocException(std::string const &where = "UNKNOWN"): MutexException()
	  {
	      message = where + " - Error allocating Mutex";
	  }
  };
};

#endif
