/* $Id: Queue.h,v 1.15 2002/03/08 12:21:24 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ARIANNE_Queue_H
#define ARIANNE_Queue_H

#include "DataTypes/AList.h"
#include "Concurrent/Semaphore.h"
#include "Concurrent/Mutex.h"
#include "Concurrent/Templock.h"

//TODO: This class use pointers. We should start to consider a pointerless approach.

/** This is a Queue that blocks on Get when there is no data. */
template <class BaseType>
class Queue
  {
  protected:
  AList<BaseType> Messages;
  Semaphore Sem;
  Mutex mutex;

  Queue(const Queue& b);
  Queue& operator=(Queue& b);

  public:
  Queue():
    Sem(0)
    {
    }

  virtual ~Queue()
    {
    Messages.Clear();
    }
 
  /** This clears all the objects on the queue */
  void Clear()
    {
    Messages.Clear();
    }

  /** This adds an object to the list and notifies blocked processed. */
  void Add(BaseType *GenMes)
    {
    assert(GenMes);

    TempLock tmpMut(mutex);
    Messages.Add(GenMes);
    Sem.Post();
    }

  /** This returns an object from the queue or blocks if it is empty */
  BaseType* Get()
    {
    Sem.Wait();
    TempLock tmpMut(mutex);
    return Messages.Remove(0);
    }

  /** This returns true if the queue isn't empty. */
  bool isAvailable() const
    {
    TempLock tmpMut(mutex);
    if(Messages.Size()) 
      {
      return true;
      }

    return false;
    }
  };

#endif
