/* $Id: Semaphore.h,v 1.14 2002/03/08 12:21:24 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef Semaphore_H
#define Semaphore_H

#include "SDL_thread.h"
#include "Exceptions.h"

/** This is a semaphore class using SDL */
class Semaphore
  {
  SDL_sem* m_SDLSemaphore;

  Semaphore(const Semaphore& b);
  Semaphore& operator=(Semaphore& b);
  
  public: 
  Semaphore(Uint8 count=1);
  virtual ~Semaphore();
  
  /** The V operation of disktra semaphores */
  void Wait();
  /** The P operation of disktra semaphores */
  void Post();

  class SemaphoreException: public Exception
  {
      protected:
	  SemaphoreException(): Exception() {}
  };

  class SemaphoreWaitException: public SemaphoreException
  {
      public:
	  SemaphoreWaitException(std::string const &where = "UNKNOWN"): SemaphoreException()
	  {
	      message = where + " - Error waiting on Semaphore.";
	  }
  };

  class SemaphorePostException: public SemaphoreException
  {
      public:
	  SemaphorePostException(std::string const &where = "UNKNOWN"): SemaphoreException()
	  {
	      message = where + " - Error posting on Semaphore.";
	  }
  };

  class SemaphoreAllocException: public SemaphoreException
  {
      public:
	  SemaphoreAllocException(std::string const &where = "UNKNOWN"): SemaphoreException()
	  {
	      message = where + " - Error allocating Semaphore";
	  }
  };
  
  };


#endif
