/***************************************************************************
                          thread.cpp  -  description
                             -------------------
    begin                : Tue Jul 4 2000
    copyright            : (C) 2000 by Ulrich Eckhardt
    email                : Doomster@knuut.de
    modified by          : Masahiro Minami
                         : Miguel Angel Blanch Lardin
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "Thread.h"
#include "Templock.h"
#include "Exceptions.h"
#include "Log/Logger.h"
#include <iostream>
/** This is the global thread-function that is entered by every SDL_thread. It is
always given the address of a Thread-object as argument and simply calls the
objects threadFunction().
It has to be declared with C-linkage since it is acessed by a C-library. */

#ifdef macintosh
pascal void* commonThreadFunction( void* data )
{
	if( Thread* thread = reinterpret_cast<Thread*>( data ))
	{
		thread->threadFunction();
	}
	
	return nil;					// Returns nil; is this right thing?
}

# ifdef TARGET_API_MAC_CARBON
ThreadEntryUPP	entryUPP = NewThreadEntryUPP( commonThreadFunction );
# endif

#else
extern "C" int commonThreadFunction(void* data)
{
    if(Thread* thread = reinterpret_cast<Thread*>(data))
    {
        int id = SDL_ThreadID();
        try
        {
            // set the threadID
            thread->m_ID = id;
            return thread->threadFunction();
        }
        catch(std::exception &e)
        {
        //TODO: We should use the exception information, don't you think?
        glog_Common.Debug(std::string("Cought exception in Thread ") + thread->threadName()+": "+e.what(),Log::unmanaged_exception);
        throw;
        }
        catch(...)
        {
        glog_Common.Debug(std::string("Cought exception in Thread ") + thread->threadName(),Log::unmanaged_exception);
        throw;
        }
    }
    else
    {
        throw Thread::ThreadInvalidData(__FILE_AND_LINE__);
    }

    return -1;
}
#endif

Thread::Thread(std::string threadn):
  m_ThreadName(threadn),
#ifdef macintosh
#else
  m_SDLThread(0),
#endif
  m_ID(0)
{
}

Thread::~Thread()
{
 	#ifdef macintosh
 	if(m_ID)
 		KillThread();
 	#else
 	if(m_SDLThread != 0)
 		KillThread();
 	#endif // macintosh
}

void Thread::Start()
{
#ifdef macintosh
#  ifdef TARGET_API_MAC_CARBON
    OSErr	theErr = NewThread( kCooperativeThread, entryUPP, reinterpret_cast<void*>(this),
  							0, 0, nil, &m_ID );
#  else
    OSErr	theErr = NewThread( kCooperativeThread, commonThreadFunction, reinterpret_cast<void*>(this),
  							0, 0, nil, &m_ID );
#  endif
#else
    m_SDLThread = SDL_CreateThread(&commonThreadFunction, reinterpret_cast<void*>(this));
#endif
}

/** Wait for a thread to finish. The return code for the thread function
is placed in the area pointed to by 'status', if 'status' is not NULL.*/
void Thread::WaitThread(int *status/*= 0*/)
{
#ifdef macintosh
#else
	SDL_WaitThread(m_SDLThread, status);
	m_SDLThread = 0;
#endif
}

/** Forcefully kill a thread without worrying about its state */
void Thread::KillThread()
{
//	TempLock lock(m_mutex);
//	cerr << "killThread: ID=" << threadID() << endl;
	
#ifdef macintosh
	::DisposeThread( m_ID, nil, true );
#else	
	SDL_KillThread(m_SDLThread);
	m_SDLThread = 0;
#endif
}
	                           	
LoopThread::LoopThread(std::string threadn):
  Thread(threadn),
  m_suspend(false),
  m_sync(false),
  m_suspended(false),
  m_terminate(false),
  m_running(false)
{
}
LoopThread::~LoopThread()
{
}

void LoopThread::Start()
{
    TempLock lock(m_sync);

    if(m_suspended)
    {
        m_suspended = false;
        m_suspend.Unlock();
    }
    else if(!m_running)
    {
        Thread::Start();
    }
}
/** this method suspends the thread after the current loop()-call
returns */
void LoopThread::Suspend()
{
	TempLock lock(m_sync);
	
	if(! m_suspended)
	{
		m_suspended = true;
		m_suspend.Lock();
	}
}

/** this method terminates the thread after the current loop()-call
returns */
int LoopThread::Stop()
{
	m_terminate = true;
	// start the thread if not already running
	//Start();
	return 0;
}

int LoopThread::threadFunction()
{
	// wait until the mutex is unlocked and then start the loop
	m_suspend.Lock();
	m_suspend.Unlock();

    m_running = true;
	while(! m_terminate)
	{
		loop();
		
#ifdef macintosh
		// On MacOS, we are not using Mutex architecture;
		// Instead, we use ThreadManager functions
		// TODO: but this doesn't pause the thread if someone called suspend()? uli-2001-04-28
		::YieldToAnyThread();
#else
		// lock the mutex temporarily. This will wait here if someone called
		// suspend() in the meantime
		m_suspend.Lock();
		m_suspend.Unlock();
#endif
	}
	m_running = false;

    glog_Common.Debug(threadName()+" stopped running.");
	return 0;
}

/** Wait for a thread to finish. The return code for the thread function
is placed in the area pointed to by 'status', if 'status' is not NULL.*/
void LoopThread::Wait(int *status/*= 0*/)
{
Thread::WaitThread(status);
}


/** returns true if the Thread is running */
bool LoopThread::isRunning()
{
    return m_running;
}

