/* $Id: Conversion.h,v 1.17 2002/04/17 19:40:15 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2001 by Benjamin Lerman
    email                : quisar@quisar.ambre.net
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/* A few functions very useful to convert strings from differents formats. *
 ***************************************************************************/

#ifndef ARIANNE_conversion_h_
#define ARIANNE_conversion_h_

#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <algorithm>

#include "Types.h"
#include "Exceptions.h"

class ConversionException : public Exception
{
  public:
    ConversionException(std::string const &where = "UNKNOWN",
                        std::string const &msg = "Conversion Error"):
    Exception()
    {
      message = where + msg;
    }
};

inline void VectorToString(std::vector<unsigned char> const &vect, std::string &str)
{
  str.resize(vect.size());
  std::copy( vect.begin(), vect.end(), str.begin());
}

inline void StringToVector(std::string const &str, std::vector<unsigned char> &vect)
{
  vect.resize(str.size());
  std::copy( str.begin(), str.end(), vect.begin());
}

inline void CharToVector(const unsigned char *str, unsigned int size,  std::vector<unsigned char> &vect)
{
  vect.resize(size);
  std::copy( str, str+size, vect.begin());
}

inline void VectorToChar(std::vector<unsigned char> const &vect, unsigned char *str)
{
  std::copy( vect.begin(), vect.end(), str);
}

inline void CharToString(const unsigned char *str, unsigned int size, std::string &vect)
{
  vect.resize(size);
  std::copy( str, str+size, vect.begin());
}

inline void StringToChar(std::string const &vect, unsigned char *str)
{
  std::copy( vect.begin(), vect.end(), str);
}
/** convert each byte in bvect to its textual representation which is stored in vect */
inline void BinaryVectorToHex(std::vector<unsigned char> const &bvect, std::vector<unsigned char> &vect)
{
 unsigned int v, t;
 std::vector<unsigned char>::const_iterator i;
 std::vector<unsigned char>::const_iterator last;
 std::vector<unsigned char>::iterator j;

 vect.resize(2*bvect.size());
 for(i=bvect.begin(), j=vect.begin(), last = bvect.end(); i != last; ++i)
 {
  v = (unsigned int) *i;
  // upper nibble
  t = v / 16;
  if(t < 10)
    *j = ((unsigned char) (((unsigned int) '0') + t));
  else
    *j = ((unsigned char) (((unsigned int) 'a') + t -10));
  ++j;

  // lower nibble
  t = v % 16;
  if(t < 10)
   *j = ((unsigned char) (((unsigned int) '0') + t));
  else
   *j = ((unsigned char) (((unsigned int) 'a') + t -10));
  ++j;
 }
}

inline void BinaryVectorToHexString(std::vector<unsigned char> const &bvect, std::string &str)
{
 std::vector<unsigned char> vect;
 BinaryVectorToHex(bvect, vect);
 VectorToString(vect, str);
}

inline void BinaryStringToHexString(std::string const &bstr, std::string &str)
{
  std::vector<unsigned char> bvect;
  StringToVector(bstr, bvect);
  BinaryVectorToHexString(bvect, str);
}

/** convert a series of hex-character from str to their binary
representation which is stored in bstr
TODO: convert the bstr-argument to vector, strings shouldn't contain zeroes
*/
inline void HexStringToBinaryString(std::string const &str, std::string &bstr)
{
  // small sanity-check
  if(str.size() % 2 != 0)
  {
    throw ConversionException(__FILE_AND_LINE__,
                              "Source string has an odd number of characters");
    return;
  }

  std::string::const_iterator i, last;
  std::string::iterator j;

  unsigned char c;
  bstr.resize(str.size() / 2);
  for(i=str.begin(), j=bstr.begin(), last=str.end(); i != last; ++j)
  {
    c = 0;
    // high-nibble
    if((*i >= '0') && (*i <= '9'))
      c += (*i - '0');
    else if((*i >= 'a') && (*i <= 'f'))
      c += (*i - 'a' + 10);
    else if((*i >= 'A') && (*i <= 'F'))
      c += (*i - 'A' + 10);
    else
    {
      throw ConversionException(__FILE_AND_LINE__,
                                "Source string a non-hex character");
      return;
    }
    ++i;

    c *= 16;
    // low-nibble
    if((*i >= '0') && (*i <= '9'))
      c += (*i - '0');
    else if((*i >= 'a') && (*i <= 'f'))
      c += (*i - 'a' + 10);
    else if((*i >= 'A') && (*i <= 'F'))
      c += (*i - 'A' + 10);
    else
    {
      throw ConversionException(__FILE_AND_LINE__,
                                "Source string a non-hex character");
      return;
    }
    ++i;

    *j = c;
  }
}

template<typename T>
    inline std::string IntToString(T const& i)
    {
        std::ostringstream str;
        str << (Uint32) i;
        return str.str();
    }

inline Uint32 StringToInt(std::string const& mystr)
{
    return atoi(mystr.c_str());
}

template<typename T>
    inline void IntToNetworkVector(T i, std::vector<Uint8> &r)
    {
	r.clear();
        while(i)
	{
	    r.insert(r.begin(),i&255);
	    i >>= 8;
	}
    }

template<typename T>
    inline void NetworkVectorToInt(std::vector<Uint8> const &r, T &i)
    {
	i=0;
	std::vector<Uint8>::const_iterator last = r.end();
	for(std::vector<Uint8>::const_iterator it=r.begin(); it != last; ++it)
	{
	    i <<= 8;
	    i += *it;
	}
    }

#endif
