/* $Id: RsaKey.h,v 1.14 2002/02/24 14:27:26 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2001 by Benjamin Lerman
    email                : quisar@quisar.ambre.net
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
 * This class implements the RSA algorithm. It also allows to create Keys  *
 * and to save it to disk...                                               *
 ***************************************************************************/

#ifndef ARIANNE_crypto_rsakey_h_
#define ARIANNE_crypto_rsakey_h_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>
#include <vector>

#ifdef HAVE_OPENSSL
#include <openssl/rsa.h>
#else
#include <rsa.h>
#endif

class RSAKey
{

 public:
	RSAKey(std::string const& path);
	RSAKey(std::string const& n, std::string const& e);
       	RSAKey();
	~RSAKey();
	bool IsValid() const;
	bool SaveKey(std::string const& ) const;
	bool SavePublicKey(std::string const& path) const;
	bool GetPublicKey(std::string& n, std::string& e);
	bool LoadKey(std::string const& );
	void GenerateKey();
	void GenerateKey(const int nbbits);
	void GenerateKey(const int nbbits, const int ve);
	bool SignMessage(std::vector<unsigned char> const& message, std::vector<unsigned char> &signature) const;
	bool SignMessage(std::string const& message, std::vector<unsigned char> &signature) const;
	bool VerifySignature(std::vector<unsigned char> const& message, std::vector<unsigned char> const &signature) const;
	bool VerifySignature(std::string const& message, std::vector<unsigned char> const &signature) const;

 private:
	RSA *key;
	const static std::string n;
	const static std::string e;
	const static std::string d;
	const static std::string p;
	const static std::string q;
	const static std::string dmp1;
	const static std::string dmq1;
	const static std::string iqmp;
};

#endif
