/* $Id: Sha1.cpp,v 1.14 2002/04/17 20:25:33 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2001 by Benjamin Lerman
    email                : quisar@quisar.ambre.net
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_CRYPTO

#ifdef HAVE_OPENSSL
#include <openssl/sha.h>
#else
#include <sha.h>
#endif

#include <string>
#include <vector>

#include "Conversion/Conversion.h"
#include "Crypto/Sha1.h"
#include "Crypto/Rand.h"
#include "Log/Logger.h"
#include "Exceptions.h"

void SHA1::GetHash(std::vector<unsigned char> const &m, std::vector<unsigned char> &hash)
{
 SHA_CTX c;
 unsigned char md[20];

 SHA1_Init(&c);
 SHA1_Update(&c, &m[0], m.size());
 SHA1_Final(md, &c);
 CharToVector(md, 20, hash);
}

void SHA1::GetHash(std::string const &m, std::vector<unsigned char> &hash)
{
 std::vector<unsigned char> tmp;
 StringToVector(m ,tmp);
 GetHash(tmp, hash);
}

void SHA1::cryptPW(RAND& rand, std::string const& pass, std::string& res)
{
  std::vector<unsigned char>seed;
  std::vector<unsigned char>hash;
  std::string tmp;
  std::string binaryres;
 
  rand.GetRandomBytes(seed, 10);
  VectorToString(seed, tmp);
  tmp += pass;
  GetHash(tmp, hash);
  VectorToString(seed, binaryres);
  VectorToString(hash, tmp);
  binaryres += tmp;

  try
  {
    BinaryStringToHexString(binaryres, res);
  }
  catch (Exception &e)
  {
    glog_Common.Debug(__FILE_AND_LINE__, e.what(), Log::error);
    throw;
  }
}

bool SHA1::verifyPW(std::string const& pass, std::string const& pwhash)
{
  std::string seed;
  std::string binaryseed;
  std::string hash;
  std::vector<unsigned char> newhash;

  if (pwhash.size() != 60)
    return false;

  seed = pwhash.substr(0, 20);
  hash = pwhash.substr(20, 40);

  try
  {
    HexStringToBinaryString(seed, binaryseed);
  }
  catch (Exception &e)
  {
    glog_Common.Debug(__FILE_AND_LINE__, e.what(), Log::error);
    throw;
  }
 
  binaryseed += pass;
  GetHash(binaryseed, newhash);

  try
  {
    BinaryVectorToHexString(newhash, seed);
  }
  catch (Exception &e)
  {
    glog_Common.Debug(__FILE_AND_LINE__, e.what(), Log::error);
    throw;
  }

  return((bool) (seed == hash));
}

#endif

