/* $Id: Serializer.cpp,v 1.14 2002/01/24 23:14:41 arianne_rpg Exp $ */
/***************************************************************************
                          Serializer.cpp  -  description
                             -------------------
    begin                : Sun Mar 18 2001
    copyright            : (C) 2001 by Ulrich Eckhardt
    email                : Doomster@knuut.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "Serializer.h"
#include <algorithm>

Serializer::Serializer():
  m_index(0)
  {
  }

Serializer::Serializer(std::vector<Uint8> const& data):
  m_data(data),
  m_index(0)
  {
  }

Serializer::~Serializer()
  {
  }

void Serializer::push(Uint8 c)
  {
  m_data.push_back(c);
  }

Uint8 Serializer::pop()
  {
  if(m_index >=  m_data.size())
    {
    throw EOFException(__FILE_AND_LINE__);
	}
  else
    {
	return m_data[m_index++];
	}
  }

void Serializer::tovector(std::vector<Uint8> &data) const
  {
  Sint32 size=m_data.size()-m_index;
  if(size<0) size=0;
  data.resize(size);

  if(size) std::copy(m_data.begin() + m_index, m_data.end(), data.begin());
  }


Serializer& operator<<(Serializer& s, Uint8 const& t)
  {
  s.push(t);
  return s;
  }

Serializer& operator<<(Serializer& s, Uint16 const& t)
  {
  s.push((t>>8)&255);
  s.push(t&255);
  return s;
  }

Serializer& operator<<(Serializer& s, Uint32 const& t)
  {
  s.push((t>>24)&255);
  s.push((t>>16)&255);
  s.push((t>>8)&255);
  s.push(t&255);
  return s;
  }

Serializer& operator>>(Serializer& s, Uint8& t)
  {
  t=s.pop();
  return s;
  }

Serializer& operator>>(Serializer& s, Uint16& t)
  {
  t=s.pop()<<8;
  t+=s.pop();
  return s;
  }

Serializer& operator>>(Serializer& s, Uint32& t)
  {
  t=s.pop()<<24;
  t+=s.pop()<<16;
  t+=s.pop()<<8;
  t+=s.pop();
  return s;
  }

Serializer& operator<<(Serializer& s, bool const& b)
  {
  return s << Uint8(b);
  }

Serializer& operator>>(Serializer& s, bool& b)
  {
  Uint8 tmp;
  s >> tmp;
  b = (tmp!=0);
  return s;
  }

Serializer& operator<<(Serializer& s, std::string const& l)
{
    s << (Uint32)l.size();
    std::string::const_iterator last = l.end();
    for(std::string::const_iterator it = l.begin(); it!=last; ++it)
        s.push(*it);

    return s;
}

Serializer& operator<<(Serializer& s, std::string& l)
{
    s << (Uint32)l.size();
    std::string::iterator last = l.end();
    for(std::string::iterator it = l.begin(); it!=last; ++it)
        s.push(*it);

    return s;
}

Serializer& operator>>(Serializer& s, std::string& l)
{
    Uint32 size;
    s >> size;
    l.resize(size);
    std::string::iterator last = l.end();
    for(std::string::iterator it = l.begin(); it!=last; ++it)
        *it = s.pop();
    
    return s;
}
