/* $Id: Log.cpp,v 1.20 2002/04/28 08:16:32 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Log.h"
#include "Path/FilePath.h"
#include "Concurrent/Templock.h"

// 010115 masahiro minami elsur@aaa.letter.co.jp

#include <ctime>
#include <iomanip>
#include <iostream>

using std::clog;
using std::endl;
using std::ofstream;

Log glog_Common("Common");
Log glog_Layer0("Layer0");
Log glog_Layer1("Layer1");
Log glog_Layer2("Layer2");
Log glog_Layer3("Layer3");
Log glog_Layer4("Layer4");
Log glog_Layer5("Layer5");
Log glog_App   ("App   ");

/* The static members */
std::ofstream Log::out;
bool Log::ToScreen = true;
Mutex Log::screenMutex;

using namespace std;


const char *ErrorMessagesText[]=
{
  "notice",
  "managed exception",
  "warning",
  "error",
  "unmanaged exception",
  "fatal error"
};


Log::Log(const std::string &name):
    Name(name)
{
}

void Log::toFile(const std::string &Path, bool screen)
{
  /** close the old file, reset the streamstate and open the new file*/
  out.close();
  out.clear();
  out.open(getValidFilePath(Path).c_str());

  ToScreen = screen;
}

Log::~Log()
{
}

void Log::Debug(const std::string &Location, const std::string &Text, Level const &level)
{
  Debug(Location+std::string(": ")+Text,level);
}

void Log::Debug(const std::string &Text, Level const &level)
{
  TempLock tmpMut(mutex);

  time_t t = time(0);
  struct tm *localt = localtime(&t);
  
  if (out)
  {
    out << setfill('0')
     << setw(2) << localt->tm_hour << ":"
     << setw(2) << localt->tm_min << ":"
     << setw(2) << localt->tm_sec << "|"
     << Name << "|" << ErrorMessagesText[level] << "-> " << Text << endl;
  }
  if (ToScreen)
  {
    TempLock tmpsMut(screenMutex);

    clog << setfill('0')
     << setw(2) << localt->tm_hour << ":"
     << setw(2) << localt->tm_min << ":"
     << setw(2) << localt->tm_sec << "|"
     << Name << "|" << ErrorMessagesText[level] << "-> " << Text << endl;
  }
}

