/* $Id: Log.h,v 1.18 2002/04/26 13:13:15 doomster Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __ARIANNE_RPG_Log_h
#define __ARIANNE_RPG_Log_h

#include <fstream>
#include <string>
#include "Concurrent/Mutex.h"

/** logging-helper class
Use the two Debug() overloads to log some debug-messages. See 'Logger.h' for a
set of predefined logging-targets.
You can control the target with the toFile-method.
*/
class Log
  {
  /** whether logging-messages should also go to screen (aka std::clog) */
  static bool ToScreen;
  /** target-file for logging */
  static std::ofstream out;
  /** mutex to prevent mutliple threads from scrambling each other's log-messages */
  static Mutex screenMutex;

  std::string Name;
  Mutex mutex;

  // not copyable
  Log(Log const& b);
  // not assignable
  Log& operator=(Log const& b);


  public:
  /** classification for the debug-message */
  enum Level
    {
    notice=0,
    managed_exception,
    warning,
    error,
    unmanaged_exception,
    fatal_error
    };

  /** construct a logging-target. The given name is prefixed to all output
  that is sent there. */
  Log(const std::string &Name);
  ~Log();

  /** This shows 'Text' with the associated error level */
  void Debug(const std::string &Text, Level const& level=notice);
  /** This shows 'Location' and 'Text' with the associated error level */
  void Debug(const std::string &Location, const std::string &Text, Level const& level=notice);

  /* (re)direct logging to the file 'Path' and optionally to the screen */
  static void toFile(const std::string &Path, bool ToScreen=false);

  };

#endif

