/* $Id: Manager.h,v 1.17 2002/03/30 12:21:23 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ARIANNE_Manager_H
#define ARIANNE_Manager_H

#include "Exceptions.h"
#include "Concurrent/Queue.h"
#include "Conversion/Conversion.h"

#include "2_Command/Command.h"

/** This is the abstract Manager class. */
class Manager
  {
  // not copyable
  Manager(Manager const&);
  // not assignable
  Manager& operator=(Manager const&);

  friend class CommandManager;

  protected:
  /** The Queue of Commands to be executed. */
  Queue<Command> Commands;
  /** A pointer to where the generated Commands should be send. */
  CommandManager *CommManager;

  public:
  Manager();
  virtual ~Manager();

  /** set the target CommandManager where to send messages */
  void Attach(CommandManager* m)
    {
    CommManager = m;
    }

  /** Add a command to the Command Queue */
  void AddCommand(Command *Comm);
  /** Get the first available command or block until available */
  Command *GetCommand();
  /** Returns true if there are commands available. */
  bool isAvailableCommand() const;

  class ManagerUnCommandException: public Exception
    {
    public:
    ManagerUnCommandException(std::string const& where = "UNKNOWN"): Exception()
      {
      message = where + " - Error recieving command: Unknown type.";
      }

    ManagerUnCommandException(std::string const& where, Uint32 type): Exception()
      {
      message = where + " - Error recieving command: Unknown type:" + IntToString(type);
      }
    };
 };

#endif
