/* $Id: Connection.cpp,v 1.13 2001/12/23 17:28:40 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Connection.h"
#include "Exceptions.h"
#include "Define.h"
#include <cassert>
#include <iostream>
#include "DataTypes/Serializer.h"

Connection::Connection(): BlockSend(false), BlockReceived(false)
  {
  Port=0;
  DataSent=DataRecv=0;
  }

Connection::Connection(const char *Host, Uint16 Port): BlockSend(false), BlockReceived(false)
  {
  if(Host)
    this->Host = Host;
  this->Port=Port;
  DataSent=DataRecv=0;
  }

Connection::~Connection()
  {
  DataSent=DataRecv=0;
  }
   
void Connection::SetHost(const char *Host, Uint16 Port)
  {
  if(Host)
	this->Host=Host;

  this->Port=Port;
  }

Connection& Connection::operator <<(GenericMessage &gm)
  {
  Serializer mes;

  mes << gm;

  std::vector<Uint8> mesv;
  mes.tovector(mesv);
  {
    TempLock lock(BlockSend);
    DataToSend.insert(DataToSend.end(), mesv.begin(), mesv.end());
  }
  return *this;
  }

Connection& Connection::operator >>(GenericMessage &gm)
  {
  Serializer s;
  Uint32 size=0;

  {
    TempLock lock(BlockReceived);
    size=DataReceived[0];
    size = DataReceived[2]<<24;
    size +=DataReceived[3]<<16;
    size +=DataReceived[4]<<8;
    size +=DataReceived[5];

    if(size > DataReceived.size() - 6)
      throw ConnectionNoMessageException(__FILE_AND_LINE__);

    std::vector<Uint8>::iterator i;
    for(i=DataReceived.begin(); i!= DataReceived.begin()+size+6; ++i)
      s.push(*i);

    DataReceived.erase(DataReceived.begin(), DataReceived.begin()+size+6);
  }

  s >> gm;

  return *this;
  }

void Connection::GetStatistics(Uint32 *DataSent,Uint32 *DataRecv)
  {
  *DataSent=this->DataSent;
  *DataRecv=this->DataRecv;
  }

void Connection::SetStatistics(Uint32 DataSent,Uint32 DataRecv)
  {
  this->DataSent=DataSent;
  this->DataRecv=DataRecv;
  }


ConnectionSet::ConnectionSet(Uint32 NumConnections)
  {
  assert(NumConnections>0);

  socketset=SDLNet_AllocSocketSet(NumConnections);

  if(!socketset)
    {
	throw ConnectionSetOpenException(__FILE_AND_LINE__);
	}
  }

ConnectionSet::~ConnectionSet()
  {
  SDLNet_FreeSocketSet(socketset);
  }

void ConnectionSet::Wait(Uint32 time)
  {
  int tmp=SDLNet_CheckSockets(socketset,time);

  if(tmp==-1)
    {
	throw ConnectionSetSelectException(__FILE_AND_LINE__);
	}
  }


void Connection::SendPrompt()
{
  BlockingSend((const unsigned char *)ARIANNE_PROMPT.c_str(), ARIANNE_PROMPT.size(), 1);
}

bool Connection::RecvPrompt()
{
  std::vector<char> buf;
  buf.resize(ARIANNE_PROMPT.size() + 1);
  buf[ARIANNE_PROMPT.size()] = '\0';
  BlockingRecv((unsigned char *) &buf[0], ARIANNE_PROMPT.size(), 1);
  bool result=((std::string(&buf[0])) == ARIANNE_PROMPT);

  return result;
}

void Connection::SendBuffer()
{
  TempLock lock(BlockSend);
  if(DataToSend.size() == 0)
    return;
  Sint32 size;
  size = Send(&DataToSend[0], DataToSend.size(), 1);
  DataToSend.erase(DataToSend.begin(), DataToSend.begin() + size);
}

void Connection::ReceiveBuffer()
{
  TempLock lock(BlockReceived);
  if(! isData())
    return;
  std::vector<Uint8>buftmp;
  Sint32 size;
  buftmp.resize(2000);
  size = Recv(&buftmp[0], 2000, 1);
  DataReceived.insert(DataReceived.end(), buftmp.begin(), buftmp.begin() + size);
}

bool Connection::isMessage()
{
    Uint32 size=0;
    TempLock lock(BlockReceived);
    if(DataReceived.size() < 6)
	return false;
    size = DataReceived[2]<<24;
    size +=DataReceived[3]<<16;
    size +=DataReceived[4]<<8;
    size +=DataReceived[5];

    return (size <= DataReceived.size() - 6);
}

bool ConnectionSet::isMessage(Connection *conn)
{
  assert(conn);
  return conn->isMessage();
}
