/* $Id: TCPConnection.cpp,v 1.18 2002/04/02 23:48:10 mutable Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "TCPConnection.h"
#include "Exceptions.h"

#include <cassert>
#include <sstream>
#include <iostream>

using namespace std;


Sint32 TCPConnection::BlockingSend(const Uint8 *Data,Uint32 size_elem,Uint32 n_elems)
  {
  assert(TCP);
  assert(Data);

  Sint32 tmp=0;
  Uint32 Remaining=size_elem*n_elems;

  while(Remaining>0)
    {
    tmp=SDLNet_TCP_Send(TCP,(char *)Data,Remaining);

    if(tmp==0 || tmp<0)
      {
	  throw ConnectionWriteException(__FILE_AND_LINE__);
	  }

	Data=&Data[tmp];
	Remaining-=tmp;
	}

  DataSent+=size_elem*n_elems;

  return size_elem*n_elems;
  }

Sint32 TCPConnection::Send(const Uint8 *Data,Uint32 size_elem,Uint32 n_elems)
{
  assert(TCP);
  assert(Data);

  Sint32 tmp=0;
  tmp = SDLNet_TCP_Send(TCP,(char *)Data,size_elem*n_elems);
  if(tmp==0 || tmp<0)
    throw ConnectionWriteException(__FILE_AND_LINE__);

  DataSent+=size_elem*n_elems;

  return tmp;
}

Sint32 TCPConnection::BlockingRecv(Uint8 *Data,Uint32 size_elem,Uint32 n_elems)
  {
  assert(TCP);
  assert(Data);

  Sint32 tmp=0;
  Uint32 Remaining=size_elem*n_elems;

  while(Remaining>0)
    {
    tmp=SDLNet_TCP_Recv(TCP,(char *)Data,Remaining);

    if(tmp==0 || tmp<0)
      {
      throw ConnectionReadException(__FILE_AND_LINE__);
      }

    Data=&Data[tmp];
    Remaining-=tmp;
    }

  DataRecv+=size_elem*n_elems;

  return size_elem*n_elems;
  }

Sint32 TCPConnection::Recv(Uint8 *Data,Uint32 size_elem,Uint32 n_elems)
{
  assert(TCP);
  assert(Data);

  Sint32 tmp=0;
  tmp = SDLNet_TCP_Recv(TCP,(char *)Data,size_elem*n_elems);
  if(tmp==0 || tmp<0)
    throw ConnectionReadException(__FILE_AND_LINE__);

  DataRecv+=size_elem*n_elems;

  return tmp;
}

TCPConnection::TCPConnection():Connection(),
  TCP(0) 
  {
  }

TCPConnection::TCPConnection(char *Host, Uint16 Port):Connection(Host,Port),
  TCP(0)
  {
  }

TCPConnection::~TCPConnection()
  {
  }


std::string TCPConnection::GetIPAddress()
  {
  std::ostringstream sout;
  // Quick fix-- TODO: fix this correctly
  #ifndef MACOSX
  sout << (IP.host % 256) << "."
       << ((IP.host / 256) % 256) << "."
       << ((IP.host / 256 / 256) % 256) << "."
       << ((IP.host / 256 / 256 / 256) % 256);
  #else
  sout << ((IP.host / 256 / 256 / 256) % 256) << "."
       << ((IP.host / 256 / 256) % 256) << "."
       << ((IP.host / 256) % 256) << "."
       << (IP.host % 256);
  #endif
  
  return sout.str();
  }

void TCPConnection::Open()
  {
  // SDL doesn't honor const, therefore the const_cast
  SDLNet_ResolveHost(&IP,const_cast<char*>(Host.c_str()),Port);
  TCP=SDLNet_TCP_Open(&IP);

  if(!TCP)
    {
	throw ConnectionOpenException(__FILE_AND_LINE__);
	}
  }

void TCPConnection::Close()
  {
      try {
	  SendBuffer();
      }
      catch(ConnectionWriteException& e)
      {
      }
      SDLNet_TCP_Close(TCP);
  }

Connection *TCPConnection::Accept()
  {
  TCPConnection *newConnection=new TCPConnection;
  newConnection->TCP=SDLNet_TCP_Accept(TCP);

  if(!newConnection->TCP)
    {
    delete newConnection;
    newConnection=0;
    throw ConnectionAcceptException(__FILE_AND_LINE__);
    }

  newConnection->IP = *SDLNet_TCP_GetPeerAddress(newConnection->TCP);
  return newConnection;
  }

bool TCPConnection::isData()
  {
  if(SDLNet_SocketReady(TCP))
    {
    return true;
    }
  
  return false;
  }

TCPConnectionSet::TCPConnectionSet(Uint32 NumConnections): ConnectionSet(NumConnections)
  {
  }

TCPConnectionSet::~TCPConnectionSet()
  {
  }

void TCPConnectionSet::AddConnection(Connection *conn)
  {
  if(SDLNet_TCP_AddSocket(socketset,((TCPConnection*)conn)->TCP)<0)
    {
	throw ConnectionSetAddException(__FILE_AND_LINE__);
	}
  }

void TCPConnectionSet::RemoveConnection(Connection *conn)
  {
  if(SDLNet_TCP_DelSocket(socketset,((TCPConnection*)conn)->TCP)<0)
    {
	throw ConnectionSetRemoveException(__FILE_AND_LINE__);
	}
  }

bool TCPConnectionSet::isData(Connection *conn)
  {
  if(SDLNet_SocketReady(((TCPConnection*)conn)->TCP))
    {
    return true;
    }

  return false;
  }


