/* $Id: Parser.cpp,v 1.18 2002/04/05 14:00:44 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by 
    email                : 
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Parser.h"
#include "Conversion/Conversion.h"

#include <fstream>
#include <string>

#include "XML/tinyxml.h"
#include "Conversion/StringTool.h"

using namespace std;

void IterateXMLTree(TiXmlNode *parent,string const &section,string const& key, string &buffer, std::list<string>& Enviroment)
  {
  for(TiXmlNode *child=parent->IterateChildren(0);child;child=parent->IterateChildren(child))
    {
    switch(child->Type())
      {
      case TiXmlNode::ELEMENT:
        {
        Enviroment.push_back(child->Value());

        IterateXMLTree(child,section,key,buffer,Enviroment);

        Enviroment.pop_back();
        break;
        }
      case TiXmlNode::TEXT:
        {
        list<string> tmp;
        tmp.push_back(section);
        tmp.push_back(key);
        if(find_list(Enviroment.begin(),Enviroment.end(),tmp))
          {
          buffer=child->Value();
          return;
          }

        break;
        }
      }
    }
  }


/*  Retrive a character string from the ini file in the buffer.
    If key==NULL it returns a double NUL terminated list of keys.
    The default value must never be NULL.
    Section and key names are case insensitive.
    Returns the number of characters placed in the buffer
*/
int PersistentVar::GetIniFileStr(std::string const& section,std::string const& key,std::string const& defval,std::string &buffer,std::string const& inifile)
  {
  /* We want it with XML parser. */
  TiXmlDocument doc(inifile);
  if(!doc.LoadFile())
    {
    cout << doc.ErrorDesc() << endl;
    throw ParserFileNotFoundException();
    }

  std::list<string> Enviroment;
  IterateXMLTree(&doc,section,key,buffer,Enviroment);

  // We did not find the value, lets use the default one.
  if(buffer.empty())
    buffer = defval;

  return buffer.size();
  }

PersistentString::PersistentString(std::string const &fname,std::string const &block,std::string const &key,std::string const &defvalue)
{
  filename=fname;
  blockname=block;
  keyname=key;
  GetIniFileStr(blockname,keyname,defvalue,value,filename);
}

std::string const& PersistentString::operator= (std::string const& newvalue)
  {
  if(newvalue!=value)
    {
	value=newvalue;
	dirty=true;
    }
  return value;
  }

PersistentInt::PersistentInt(std::string const& fname,std::string const& block,std::string const &key,int defvalue)
  {
  filename=fname;
  blockname=block;
  keyname=key;
  std::string buf;
  GetIniFileStr(blockname,keyname,"",buf,filename);
  value=defvalue;
  if(buf != "")
    {
	value = StringToInt(buf);
	  }
  }

int PersistentInt::operator= (int newvalue)
  {
  dirty=(newvalue != value);
  value=newvalue;
  return value;
  }
