/* $Id: NetworkClientManager.cpp,v 1.6 2002/04/26 18:03:32 doomster Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "0_Network/NetworkClientManager.h"
#include "Log/Logger.h"
#include "Concurrent/Templock.h"

NetworkClientManager::NetworkClientManager():
  NetworkManager(),
  DoRead(true),
  connSet(1)
  {
  Continue=true;
  }

NetworkClientManager::~NetworkClientManager()
  {
  }


void NetworkClientManager::Open(std::string const& Host,Uint16 Port)
  {
  pconn.conn=new TCPConnection();
  pconn.conn->SetHost(Host.c_str(),Port);

  try
    {
    pconn.conn->Open();

    connSet.AddConnection(pconn.conn);
    }
  catch(Connection::ConnectionOpenException &e)
    {
    glog_Layer0.Debug(__FILE_AND_LINE__,e.what(),Log::fatal_error);
    throw;
    }
  catch(ConnectionSet::ConnectionSetAddException &ne)
    {
    glog_Layer0.Debug(__FILE_AND_LINE__,ne.what(),Log::fatal_error);
    throw;
    }

  pconn.Used=true;
  pconn.LastMessageTime=SDL_GetTicks();

  Continue=true;
  try
    {
    if(!pconn.conn->RecvPrompt())
    {
      glog_Layer0.Debug("Didn't received the write Prompt...");
      ClosePlayerConnection(&pconn);
      throw Connection::ConnectionReadException(__FILE_AND_LINE__);
    }
    }
  catch(Connection::ConnectionWriteException &e) 
    {
    glog_Layer0.Debug(__FILE_AND_LINE__,e.what());

    /* There was an error writting on Connection, so we kill it */
    ClosePlayerConnection(&pconn);
    throw;
    }
  }

void NetworkClientManager::Close()
  {
  if(!Continue)
    return;
  Continue=false;

  try
    {
    connSet.RemoveConnection(pconn.conn);
    }
  catch(ConnectionSet::ConnectionSetRemoveException &ne)
    {
    glog_Layer0.Debug(__FILE_AND_LINE__,ne.what(),Log::fatal_error);
    throw;
    }

  pconn.conn->Close();
  delete pconn.conn;
  pconn.conn=0;

  pconn.Used=false;
  }


void NetworkClientManager::Add(GenericMessage *GenMes)
  {
  try
    {
    *pconn.conn<<*GenMes;
    }
  catch(Connection::ConnectionWriteException &e) 
    {
    delete GenMes;
    GenMes=0;

    glog_Layer0.Debug(__FILE_AND_LINE__,e.what());

    /* There was an error writting on Connection, so we kill it */
    ClosePlayerConnection(&pconn);
    throw;
    }

  delete GenMes;
  GenMes=0;
  }

GenericMessage *NetworkClientManager::Get()
  {
  if(incoming.isAvailable())
    {
    return incoming.Get();
    }

  return 0;
  }

bool NetworkClientManager::isAvailable() const
  {
  return incoming.isAvailable();
  }


void NetworkClientManager::Wait()
  {
  try
    {
    connSet.Wait(0);
    }
  catch(ConnectionSet::ConnectionSetSelectException &e)
    {
    glog_Layer0.Debug(__FILE_AND_LINE__,e.what());
    }
  }

void NetworkClientManager::ReadPlayerConnection(PlayerConnection *pc,GenericMessage *gm)
  {
  *pc->conn>>*gm;
 
  /* Here we should test for the correction of the message */
  gm->ID=0;	  
  pc->LastMessageTime=gm->TimeToken=SDL_GetTicks();

  incoming.Add(gm);
  }

void NetworkClientManager::ClosePlayerConnection(PlayerConnection *pc)
  {
  pc->Used=false;
  try
    {
    connSet.RemoveConnection(pc->conn);
    }
  catch(ConnectionSet::ConnectionSetRemoveException &ne)
    {
    glog_Layer0.Debug(__FILE_AND_LINE__,ne.what());
    throw;
    }

  pc->conn->Close();
  pc->conn=0;
  }

void NetworkClientManager::StopReading()
{
    DoRead = false;
}

void NetworkClientManager::loop()
  {
  Wait();

  if(!Continue)
  {
     return;
  }

  if(pconn.Used)
  {
    try
    {
      pconn.conn->SendBuffer();
      if(DoRead) pconn.conn->ReceiveBuffer();
    }
    catch(Connection::ConnectionReadWriteException &e)
    {
      glog_Layer0.Debug(__FILE_AND_LINE__,e.what());
      /* There was an error reading or writting on Connection, so we kill it */
      ClosePlayerConnection(&pconn);
      throw;
    }

    if(connSet.isMessage(pconn.conn))
      {
      GenericMessage *gm=new GenericMessage;

      try
        {
        ReadPlayerConnection(&pconn,gm);
        }
      catch(Connection::ConnectionNoMessageException &e)
        {
        glog_Layer0.Debug(__FILE_AND_LINE__,e.what());

        delete gm;
        gm=0;
        }
      }
    }
  }

