/* $Id: NetworkClientManager.h,v 1.2 2002/03/08 12:21:24 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ARIANNE_NetworkClientManager_H
#define ARIANNE_NetworkClientManager_H

#include "Concurrent/Queue.h"
#include "Concurrent/Thread.h"
#include "Network/TCPConnection.h"

#include "0_Network/GenericMessage.h"
#include "0_Network/NetworkManager.h"

/** This the class that send and receives generic messages from the net. */
class NetworkClientManager: public NetworkManager
  {
  /** This is the basic structure to store a connection from a player. */
  struct PlayerConnection
    {
    /** Is connection used? */
    bool Used;
    /** The connection */
    TCPConnection *conn;
    /** Last time of a message arrival from client. */
    Uint32 LastMessageTime;
    };

  /** It is true if NetworkManager has to continue the execution */
  bool Continue;

  /** It is true if NetworkManager has to try to read messages */
  bool DoRead;

  /** The player connection */
  PlayerConnection pconn;
  /** A connection set to test for new messages */
  TCPConnectionSet connSet;

  /** Wait for a generic message to come. */
  void Wait();
  /** Read a generic message from players connection */
  void ReadPlayerConnection(PlayerConnection *pc,GenericMessage *gm);
  /** Close the player connection */
  void ClosePlayerConnection(PlayerConnection *pc);

  public:
  NetworkClientManager();
  ~NetworkClientManager();

  /** open a connection to the given host and start to receive messages.*/
  void Open(std::string const& Host,Uint16 Port);
  /** close the connection and stop the reception of messages. */
  void Close();

  /** send a generic message through the net */
  void Add(GenericMessage *GenMes);
  /** This blocks the caller until a message arrives */
  GenericMessage *Get();
  /** This returns true if there are messages awaiting */
  bool isAvailable() const;

  /** This member tells the NetworkManager to stop reading to the socket */
  void StopReading();

  virtual void loop();
  };

#endif
