/* $Id: MessageClientManager.cpp,v 1.1 2002/01/24 23:20:18 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "MessageClientManager.h"
#include "Concurrent/Templock.h"
#include "Log/Logger.h"
#include "DataTypes/Serializer.h"

MessageClientManager::MessageClientManager(NetworkManager *NetMan):
  MessageManager()
  {
  this->NetMan=NetMan;
  }

MessageClientManager::~MessageClientManager()
  {
  NetMan=0;
  }

void MessageClientManager::loop()
  {
  GenericMessage *gm=NetMan->Get();
  
  if(gm==0) 
    return;

  try
    {
    switch(gm->Type)
      {
      case ARIANNE_SEND_VERSION:
        {
        Message_S2C_SEND_VERSION *mes=new Message_S2C_SEND_VERSION;
        Serializer s(gm->Data);
        s >> *mes;

        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;
        }
      break;
      case ARIANNE_NLOGIN:
        {
        Message_S2C_NLOGIN *mes=new Message_S2C_NLOGIN;

        Serializer s(gm->Data);
        s >> *mes;

        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;
        }
      break;
      case ARIANNE_LOGOUT:
        {
        Message_S2C_LOGOUT *mes=new Message_S2C_LOGOUT;

        Serializer s(gm->Data);
        s >> *mes;

        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;
        }
      break;
      case ARIANNE_ACTION:
        {
        Message_S2C_ACTION *mes=new Message_S2C_ACTION;

        Serializer s(gm->Data);
        s >> *mes;

        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;
        }
      break;
      case ARIANNE_PERCEPTION:
        {
        Message_S2C_PERCEPTION *mes=new Message_S2C_PERCEPTION;

        Serializer s(gm->Data);
        s >> *mes;

        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;
        }
      break;
      default:
        throw MesManUnMessageException(__FILE_AND_LINE__);
      }
    }
  catch(MessageManager::MesManUnMessageException &e)
    {
    glog_Layer1.Debug(__FILE_AND_LINE__,e.what());
    delete gm;
    gm=0;
    }
}


void MessageClientManager::Add(Message *Mes)
  {
  GenericMessage *gm=new GenericMessage;
  gm->ID=Mes->GetID();
  gm->Type=Mes->GetType();
  gm->Flags=0;

  Serializer s;
  s << *Mes;

  s.tovector(gm->Data);

  try
    {
    NetMan->Add(gm);
    }
  catch(NetworkManager::NetManUnusedConException &e)
    {
    glog_Layer1.Debug(__FILE_AND_LINE__,e.what());

    delete Mes;
    Mes=0;

    throw;
    }

  delete Mes;
  Mes=0;    
  }

Message *MessageClientManager::Get()
  {
  if(incomming.isAvailable())
    {
    return incomming.Get();
    }

  return 0;
  }

bool MessageClientManager::isAvailable() const
  {
  return incomming.isAvailable();
  }
