/* $Id: libArianneclient.cpp,v 1.17 2002/04/30 15:22:46 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include "SDL_net.h"
#include "libArianneclient.h"
#include "Log/Logger.h"

using namespace std;

ArianneClient::ArianneClient():
  MesMan(&NetMan),
  LoginStageComplete(false),
  LogoutStageComplete(false)
  {
  }

ArianneClient::~ArianneClient()
  {
  //TODO: Empty and free MessageBuffer items.
  }


void ArianneClient::Login(LoginInfo const& login)
  {
  this->login=login;

  SDLNet_Init();

  try
    {
    NetMan.Open(login.Host,login.Port);
    }
  catch(...)
    {
    throw HostNotFound();
    }

  try
    {
    Message_C2S_NLOGIN *meslogin=new Message_C2S_NLOGIN(ARIANNE_SEND_LOGIN,login.Name);
    MesMan.Add(meslogin);
    }
  catch(...)
    {
    throw ConnectionLost();
    }

  try
    {
    while(!LoginStageComplete)
      {
      PumpMessages();

      if(LogoutStageComplete)
        {
        throw BadLogin();
        }
      }
    }
  catch(InvalidData e)
    {
    throw e;
    }
  catch(...)
    {
    throw ConnectionLost();
    }
  }

void ArianneClient::Logout(Stats const& stats)
  {
  try
    {
    Message_C2S_LOGOUT *meslogout=new Message_C2S_LOGOUT;
    //TODO: Build the string with the stats.
    meslogout->SetStatistics("");

    MesMan.Add(meslogout);

    while(!LogoutStageComplete)
      {
      PumpMessages();
      }

    NetMan.Close();
    }
  catch(InvalidData e)
    {
    throw e;
    }
  catch(...)
    {
    throw ConnectionLost();
    }

  SDLNet_Quit();
  }

bool ArianneClient::isLogin()
  {
  return LoginStageComplete;
  }



void ArianneClient::WaitMessage()
  {
  //TODO: Implement it. We need to modify Net and Mes Managers.
  throw InvalidData();
  }

void ArianneClient::PumpMessages()
  {
  NetMan.loop();
  MesMan.loop();

  Message *mes=MesMan.Get();

  if(mes)
    {
    switch(mes->GetType())
      {
      case ARIANNE_SEND_VERSION:
        {
        cout << "Receiving Version Informations" << std::endl;
        Message_S2C_SEND_VERSION *mesverinfo=(Message_S2C_SEND_VERSION *)mes;
        cout << "Arianne Server Version: " << mesverinfo->GetVersion() << std::endl;
        cout <<  mesverinfo->GetMOTD() << std::endl;

        break;
        }
        
      case ARIANNE_NLOGIN:
        {
        cout << "Receiving NLOGIN" << std::endl;
        Message_S2C_NLOGIN *mesnlogin = (Message_S2C_NLOGIN *)mes;
        Message_C2S_NLOGIN *reply=0;
        switch(mesnlogin->GetSubType())
          {
          case ARIANNE_METHOD_PASS:
            {
            if(login.Type==rsakey)
              {
              cout << "You are trying to login to a Password account with RSA Data" << endl;
              throw InvalidData();
              }

            cout << "Server asking for password." << std::endl;
            reply=new Message_C2S_NLOGIN(ARIANNE_SEND_PASS, login.Password);
            MesMan.Add(reply);
            break;
            }
          case ARIANNE_SEND_CONFIRM:
            {
            Object::ID PlayerID;
            // TODO: fix protocol, type must be of Object::ID.
            VectorToString(mesnlogin->GetData(), PlayerID);

            cout << "Server logged me in with ID: " << PlayerID << std::endl;
            cout << mesnlogin->GetInfo() << std::endl;

            AddController(PlayerID);
            LoginStageComplete=true;
            break;
            }
#ifdef HAVE_CRYPTO
          case ARIANNE_METHOD_RSA:
            {
            if(login.Type==password)
              {
              cout << "You are trying to login to a RSA account with Password Data" << endl;
              throw InvalidData();
              }

            cout << "Client need to use RSA."<< std::endl;
            key = new RSAKey(login.RSAKeyLocation);
            if(! key->IsValid())
              {
              cout << "Bad key." << std::endl;
              throw BadLogin();
              }
            h_w1 = mesnlogin->GetData();
            r.GetRandomBytes(w2, 20);
            reply=new Message_C2S_NLOGIN(ARIANNE_SEND_W2, w2);
            MesMan.Add(reply);
            }
            break;

          case ARIANNE_SEND_M:
            {
            int i;
            m = mesnlogin->GetData();
            if(m.size() != 20)
              {
              cout << "Server send bad M."<< std::endl;
              throw BadLogin();
              }

            w1.clear();

            for(i = 0; i<20; ++i)
              {
              w1.push_back(w2[i] ^ m[i]);
              }

            SHA1::GetHash(w1, hash);
            if(hash.size() != h_w1.size())
              {
              cout << "Prb while verifying H(w1)" << std::endl;
              throw BadLogin();
              }

            for(i = 0; i<(int)hash.size(); ++i)
              {
              if(hash[i] != h_w1[i])
                {
                glog_App.Debug("Server did not send the right M");
                throw BadLogin();
                }
              }
            key->SignMessage(m, signature);
            reply=new Message_C2S_NLOGIN(ARIANNE_SEND_S_M_, signature);
            MesMan.Add(reply);
            }
            break;
#endif
          default:
            {
            cerr << "NLOGIN Type not yet implemented: " << mesnlogin->GetSubType() << std::endl;
            }
          }
        break;
        }

      case ARIANNE_LOGOUT:
        {
        //TODO: Should the server reply to a Logout message of the client?
        cout << "Receiving Logout" << std::endl;

        Message_S2C_LOGOUT *meslogouts2c=(Message_S2C_LOGOUT *)mes;
        cout << meslogouts2c->GetByeString() << std::endl;

        LogoutStageComplete=true;
        break;
        }

      case ARIANNE_PERCEPTION:
        {
        cout << "Receiving Perception" << std::endl;
        MessageBuffer[0].push_back(((Message_S2C_PERCEPTION *)mes)->clone());
        break;
        }

      case ARIANNE_ACTION:
        {
        cout << "Receiving Action" << std::endl;
        MessageBuffer[1].push_back(((Message_S2C_ACTION *)mes)->clone());
        break;
        }

      default:
        {
        delete mes;
        throw InvalidData();
        }
      }  
    delete mes;
    }
  }


void ArianneClient::SendSystemInfo(Info const& info)
  {
  try
    {
    cout << "Sending System Info ( CPU Type & OS )" << std::endl;
    Message_C2S_SYSTEM_INFO *messysinfc2s=new Message_C2S_SYSTEM_INFO;
    messysinfc2s->SetOs(info.OS);
    messysinfc2s->SetCpu(info.CPU);
    MesMan.Add(messysinfc2s);
    }
  catch(...)
    {
    throw ConnectionLost();
    }
  }

void ArianneClient::SendAction(std::list<Action> const& actions)
  {
  try
    {
    cout << "Sending Action." << std::endl;
    Message_C2S_ACTION *mesaction_c2s=new Message_C2S_ACTION;

    mesaction_c2s->Setaction(actions);
    
    MesMan.Add(mesaction_c2s);
    }
  catch(NetworkManager::NetManUnusedConException e)
    {
    throw ConnectionLost();
    }
  catch(...)
    {
    throw InvalidData();
    }
  }


void ArianneClient::RecievePerception(Uint32 &timestamp,std::list<Object> &modified, std::list<Object> &deleted)
  {
  try
    {
    if(!MessageBuffer[0].empty())
      {    
      Message_S2C_PERCEPTION *mesperceptions2c=(Message_S2C_PERCEPTION *)MessageBuffer[0].front();
      MessageBuffer[0].pop_front();

      modified=mesperceptions2c->GetObjectsModified();
      deleted=mesperceptions2c->GetObjectsDeleted();
      timestamp=mesperceptions2c->Gettimestamp();

      delete mesperceptions2c;
      }
    }
  catch(...)
    {
    throw InvalidData();
    }
  }

bool ArianneClient::isAvailablePerception()
  {
  return (!MessageBuffer[0].empty());
  }


void ArianneClient::RecieveAction(std::list<std::list<Action> > &actions)
  {
  try
    {
    if(!MessageBuffer[1].empty())
      {    
      Message_S2C_ACTION *mesactions2c=(Message_S2C_ACTION *)MessageBuffer[1].front();
      MessageBuffer[1].pop_front();

      actions=mesactions2c->Getaction();

      delete mesactions2c;
      }
    }
  catch(...)
    {
    throw InvalidData();
    }
  }

bool ArianneClient::isAvailableAction()
  {
  return (!MessageBuffer[1].empty());
  }


std::list<Object::ID> const& ArianneClient::GetController()
  {
  return Controller;
  }



void ArianneClient::AddController(Object::ID const& id)
  {
  Controller.push_back(id);
  }
