#include "Connections.h"
#include "Log/Logger.h"

Connections::Connections():std::list<PlayerConnection>(),sconnSet(1)
  {
  /* We create the first PlayerConnectionSet */
  PlayerConnectionSet tmp=PlayerConnectionSet();
  tmp.connSet=new TCPConnectionSet(MAX_NUMBER_CONNECTIONS);
  tmp.used_connections=0;

  max_id=1;

  m_PlayerConnSetList.push_back(tmp);
  }

Connections::~Connections()
  {
  //TODO: Add proper delete methods. 
  }

void Connections::Open (char const* Host, Uint16 Port)
  {
  sconn.SetHost (Host, Port);
  try
    {
    sconn.Open ();
    sconnSet.AddConnection (&sconn);
    }
  catch (Connection::ConnectionOpenException & e)
    {
    glog_Layer0.Debug (__FILE_AND_LINE__, e.what (),Log::fatal_error);
    throw;
    }
  catch (ConnectionSet::ConnectionSetAddException & ne)
    {
    glog_Layer0.Debug (__FILE_AND_LINE__, ne.what (),Log::fatal_error);
    throw;
    }
  }

void Connections::Close ()
  { 
  try
    {
    sconnSet.RemoveConnection (&sconn);
    }
  catch (ConnectionSet::ConnectionSetRemoveException & ne)
    {
    glog_Layer0.Debug (__FILE_AND_LINE__, ne.what (),Log::managed_exception);
    }

  sconn.Close ();
  }


Connection* Connections::AcceptPlayer()
  {
  if(sconnSet.isData (&sconn))
    {
    try
      {
      return sconn.Accept ();
      }
    catch (Connection::ConnectionAcceptException & e)
      {
      glog_Layer0.Debug (__FILE_AND_LINE__, e.what (),Log::managed_exception);
      }
    }

  return 0;
  }

Connections::ID Connections::GetValidConnectionID()
  {
  Connections::ID id;

  if(!m_FreeConnectionID.empty())
    {
    id=m_FreeConnectionID.front();
    m_FreeConnectionID.pop_front();
    return id;
    }
  else
    {
    id=max_id;
    max_id++;
    return id;
    }
  }


Connections::ID Connections::AddPlayer()
  {
  try
    {
    TCPConnection *conn=(TCPConnection *)AcceptPlayer();
    if(conn)
      {
      return AddPlayer(conn);
      }
    else
      {
      return ERROR_CONNECTION_ID;
      }
    }
  catch (Connection::ConnectionException & e)
    {
    return ERROR_CONNECTION_ID;
    }
  }

Connections::ID Connections::AddPlayer(TCPConnection *conn)
  {
  for(std::list<PlayerConnectionSet>::iterator it=m_PlayerConnSetList.begin();it!=m_PlayerConnSetList.end();++it)
    {
    if((*it).used_connections<MAX_NUMBER_CONNECTIONS)
      {
      /* We add it here. */

      (*it).used_connections++;
      for(int i=0;i<MAX_NUMBER_CONNECTIONS;i++)
        {
        PlayerConnection tmp;
        tmp.id=GetValidConnectionID();
        tmp.conn=conn;
        tmp.connSet=(*it).connSet;

        (*it).connSet->AddConnection(conn);
        (*it).used_connections++;

        tmp.LastMessageTime=SDL_GetTicks();

        push_back(tmp);

        return tmp.id;
        }
      }
    }

  /* We create a new PlayerConnectionSet as the rest are already full. */
  PlayerConnectionSet tmp=PlayerConnectionSet();

  tmp.connSet=new TCPConnectionSet(MAX_NUMBER_CONNECTIONS);
  tmp.used_connections=0;

  m_PlayerConnSetList.push_back(tmp);

  /* Recursive call. */
  return AddPlayer(conn);
  }

void Connections::DelPlayer(Connections::ID const& b)
  {
  for(std::list<PlayerConnection>::iterator it=begin();it!=end();++it)
    {
    if((*it).id==b)
      {
      /* We return the ID to the queue */
      m_FreeConnectionID.push_back(b);

      (*it).connSet->RemoveConnection((*it).conn);
      (*it).conn->Close();

      for(std::list<PlayerConnectionSet>::iterator conn_it=m_PlayerConnSetList.begin();conn_it!=m_PlayerConnSetList.end();++conn_it)
        {
        if((*conn_it).connSet==(*it).connSet)
          {
          (*conn_it).used_connections--;
          //TODO: If it is 0 we could delete it.
          }
        }

      erase(it);

      return;
      }
    }

  throw ConnectionsException();
  }

void Connections::WaitData()
  {
  try
    {
    Uint32 start_time=SDL_GetTicks();
    sconnSet.Wait(0);

    for(std::list<PlayerConnectionSet>::iterator it=m_PlayerConnSetList.begin();it!=m_PlayerConnSetList.end();++it)
      {
      if((*it).used_connections>0)
        {
        (*it).connSet->Wait(0);
        }
      }
    Uint32 end_time=SDL_GetTicks();
    Uint32 wait_time=((end_time-start_time)<50)?(end_time-start_time):0;

    SDL_Delay(wait_time);
    }
  catch(ConnectionSet::ConnectionSetSelectException &e)
    {
    }
  }

bool Connections::isMessage(Connections::ID const& b)
  {
  PlayerConnection tmp=find(b);
  return tmp.connSet->isMessage(tmp.conn);
  }

bool Connections::isData()
  {
  if(sconnSet.isData (&sconn))
    {
    return true;
    }

  return false;
  }

PlayerConnection &Connections::find(ID const &b)
  {
  for(std::list<PlayerConnection>::iterator it=begin();it!=end();++it)
    {
    if((*it).id==b)
      {
      return *it;
      }
    }
  
  throw ConnectionsException();
  }


