/* $Id: NetworkServerManager.h,v 1.18 2002/04/26 12:50:07 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ARIANNE_NetworkServerManager_H
#define ARIANNE_NetworkServerManager_H

#include "Concurrent/Queue.h"
#include "Concurrent/Thread.h"
#include "Network/TCPConnection.h"

#include "0_Network/GenericMessage.h"
#include "0_Network/NetworkManager.h"

#include "Connections.h"

#include "Define.h"


/** This is the class that sends and recieves messages from the net. */
class NetworkServerManager: public LoopThread, public NetworkManager
  {
  /** Should the server continue? */
  bool Continue;

  Connections conns;

  Mutex mutex;

  /** Stats for the Server Stats */
  TemporalInfoLayer0 StatsL0;

  /** Block waiting for data available on connSet. */
  void WaitData();
  /** Accept a new connection and add it to pconn */
  void AcceptPlayerConnection();
  /** Read a generic message from the connection pc, where i is the connection 
     index of player connection */
  void ReadPlayerConnection(PlayerConnection &pc);
  /** Close the player connection */
  void ClosePlayerConnection(PlayerConnection &pc);

  /** This is useful for knowing when the last message was sent to the StatsManager */
  Uint32 LastTimeStatCommand;
  /** Send the stats to StatManager if time between last stats is greater 
      than a given value, or if Mandatory is true, send them inconditionally. 
      This method resets the stats. */
  void SentStatL0Command(bool Mandatory=false);

  /** inherited from LoopThread */
  virtual void loop();

  public:
  NetworkServerManager();
  ~NetworkServerManager();

  /** This open a connection to the given host and start to recieve messages.*/
  void Open(char const* Host,Uint16 Port);
  /** This close the connection and stop the reception of messages. */
  void Close();

  /** This send a generic message through the net */
  void Add(GenericMessage *GenMes);
  /** This blocks the caller until a message arrive. */
  GenericMessage *Get();
  /** This returns true if there are messages awaiting */
  bool isAvailable() const;

  /** This is used for internal usage only, to shutdown the server in an ordered
      way and don't lose data. */
  void SendPoisonToUpperLayer();
  };

#endif
