/* $Id: MessageServerManager.cpp,v 1.22 2002/04/05 17:00:24 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "MessageServerManager.h"
#include "Concurrent/Templock.h"
#include "Define.h"
#include "Log/Logger.h"
#include "2_Command/CommandManager.h"
#include "DataTypes/Serializer.h"
#include "SDL.h"

MessageServerManager::MessageServerManager(NetworkManager *NetMan):
  LoopThread("MessageManager"),
  MessageManager()
  {
  glog_Layer1.Debug ("Creating MessageManager on Server");
  assert(NetMan);

  this->NetMan=NetMan;
  LastTimeStatCommand=0;
  glog_Layer1.Debug ("Creating MessageManager on Server -DONE- ");
  }

MessageServerManager::~MessageServerManager()
  {
  NetMan=0;
  }

void MessageServerManager::SentStatL1Command(bool Mandatory)
  {
  if(((SDL_GetTicks()-LastTimeStatCommand)>TIME_BETWEEN_STATS_COMMANDS)||(Mandatory))
    {
    LastTimeStatCommand=SDL_GetTicks();
    /* We send the command to the manager. */
    StatL1Command *sL1Comm=new StatL1Command;
    sL1Comm->tmpInfL1=StatsL1;
    StatsL1.Reset();
    CommManager->AddCommand(sL1Comm);
    }
  }


void MessageServerManager::loop()
  {
  GenericMessage *gm=NetMan->Get();

  SentStatL1Command();

  ++StatsL1.MessagesRecv;
  Message *mesg=0;

  try
    {
    switch(gm->Type)
      {
      case ARIANNE_NLOGIN:
        {
        Message_C2S_NLOGIN *mes=new Message_C2S_NLOGIN;
        mesg = mes;

        Serializer s(gm->Data);
        s >> *mes;

        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;
        }
        break;
      case ARIANNE_LOGOUT:
        {
        Message_C2S_LOGOUT *mes=new Message_C2S_LOGOUT;
        mesg = mes;

        Serializer s(gm->Data);
        s >> *mes;

        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;
        }
        break;
      case ARIANNE_SYSTEM_INFO:
        {
        Message_C2S_SYSTEM_INFO *mes=new Message_C2S_SYSTEM_INFO;
        mesg = mes;

        Serializer s(gm->Data);
        s >> *mes;

        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;
        }
        break;
      case ARIANNE_ACTION:
        {
        Message_C2S_ACTION *mes=new Message_C2S_ACTION;
        mesg = mes;

        Serializer s(gm->Data);
        s >> *mes;

        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;
        }
        break;
      case ARIANNE_POISON:
        {
        if(gm->ID!=ERROR_CONNECTION_ID)
          {
          /* ATTACK: A player is trying to send us a POISON message.*/
          ++StatsL1.InvalidMessages;

          delete gm;
          gm=0;
          return;
          }

        Message_INTERNAL_POISON *mes=new Message_INTERNAL_POISON;
        mes->SetID(gm->ID);

        incomming.Add(mes);
        delete gm;
        gm=0;

        SentStatL1Command(true);

        /* We finish this layer */
        Stop();
        return;
        }

      default:
        {
        throw MesManUnMessageException(__FILE_AND_LINE__);
        }
      }
    }
  catch(MessageManager::MesManUnMessageException &e)
    {
    ++StatsL1.InvalidMessages;

    glog_Layer1.Debug(__FILE_AND_LINE__,e.what());
    delete gm;
    gm=0;
    }
  catch(Serializer::EOFException const& e)
    {
    ++StatsL1.InvalidMessages;

    glog_Layer1.Debug(__FILE_AND_LINE__,e.what());
    if(mesg)
      {
      delete mesg;
      mesg=0;
      }

    try
      {
      Message_S2C_LOGOUT *ReplyMes = new Message_S2C_LOGOUT;
      ReplyMes->SetID (gm->ID);
      ReplyMes->SetByeString ("You sent an invalid message.");
      Add(ReplyMes);
      }
    catch(Exception &e)
      {
      glog_Layer1.Debug(__FILE_AND_LINE__,e.what());
      }

    delete gm;
    gm = 0;
    }
  }


void MessageServerManager::Add(Message *Mes)
  {
  GenericMessage *gm=new GenericMessage;
  gm->ID=Mes->GetID();
  gm->Type=Mes->GetType();
  gm->Flags=0;

  Serializer s;
  s << *Mes;

  s.tovector(gm->Data);

  //TODO: Add compression library here.

  try
    {
    NetMan->Add(gm);
    }
  catch(NetworkManager::NetManUnusedConException &e)
    {
    glog_Layer1.Debug(__FILE_AND_LINE__,e.what());

    delete Mes;
    Mes=0;

    throw;
    }

  ++StatsL1.MessagesSent;

  delete Mes;
  Mes=0;
  }

Message *MessageServerManager::Get()
  {
  return incomming.Get();
  }

bool MessageServerManager::isAvailable() const
  {
  return incomming.isAvailable();
  }
