/* $Id: Bridges.cpp,v 1.21 2002/04/17 12:25:49 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Bridges.h"
#include "Log/Logger.h"
#include "Log/Log.h"
#include "Conversion/Conversion.h"
#include "0_Network/NetworkManager.h"


Message2CommandBridge::Message2CommandBridge(CommandManager *ComMan,MessageManager *MesMan):
  LoopThread("Message2CommandBridge")
  {
  glog_Layer2.Debug ("Creating Message2CommandBridge on Server");
  assert(ComMan);
  assert(MesMan);

  this->ComMan=ComMan;
  this->MesMan=MesMan;
  glog_Layer2.Debug ("Creating Message2CommandBridge on Server -DONE- ");
  }

Message2CommandBridge::~Message2CommandBridge()
  {
  }

void Message2CommandBridge::loop()
  {
  Message *mes=MesMan->Get();
  glog_Layer2.Debug("Receiving Message2Command type " + IntToString(mes->GetType()) + " from Client");

  Uint8 commType=0;

  switch(mes->GetType())
    {
    case ARIANNE_NLOGIN:
    case ARIANNE_LOGOUT:
    case ARIANNE_SYSTEM_INFO:
      {
      commType=MESSAGE_M2C_ADMIN_COMMAND;
      }
      break;
    case ARIANNE_ACTION:
      {
      commType=MESSAGE_M2C_ACTION_COMMAND;
      }
      break;
    case ARIANNE_PERCEPTION:
      {
      commType=MESSAGE_M2C_PERCEPTION_COMMAND;
      }
      break;
    case ARIANNE_POISON:
      {
      delete mes;
      mes=0;

      PoisonCommand *comm=new PoisonCommand();
      ComMan->AddCommand(comm);

      /* We finish this layer. */
      Stop();
      return;
      }
      break;
    default:
      {
      delete mes;
      mes=0;

      throw MessageManager::MesManUnMessageException(__FILE_AND_LINE__);
      }
    }

  MessageCommand *comm=new MessageCommand(commType);
  comm->SetMessage(mes);
  ComMan->AddCommand(comm);
  }

Command2MessageBridge::Command2MessageBridge(MessageManager *MesMan):
  LoopThread("Command2MessageBridge")
  {
  glog_Layer2.Debug ("Creating Command2MessageBridge on Server");
  assert(MesMan);

  this->MesMan=MesMan;
  glog_Layer2.Debug ("Creating Command2MessageBridge on Server -DONE- ");
  }

Command2MessageBridge::~Command2MessageBridge()
  {
  }

void Command2MessageBridge::loop()
  {
  Command *comm=GetCommand();

  if(comm->GetType()==POISON_COMMAND)
    {
    /* We finish this manager */
    delete comm;
    comm=0;

    Stop();
    return;        
    }

  Message *mes=((MessageCommand *)comm)->GetMessage();

  glog_Layer2.Debug("Delivering Command2Message type " + IntToString((Uint32)mes->GetType()) + " to Client");

  try
    {
    MesMan->Add(mes->clone());
    }
  catch(NetworkManager::NetManUnusedConException &e)
    {
    glog_Layer2.Debug(e.what(),Log::managed_exception);
    }
  catch(...)
    {
    glog_Layer2.Debug(__FILE_AND_LINE__,"Exception caught on Command2Message bridge");
    }

  delete comm;
  comm = 0;
  }
