/* $Id: CommandManager.cpp,v 1.23 2002/04/17 12:25:49 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "2_Command/CommandManager.h"

#include "Log/Logger.h"
#include "Concurrent/Templock.h"
#include "Conversion/Conversion.h"

#include "SDL.h"

#include <cassert>
#include <algorithm>

CommandManager::CommandManager():
  LoopThread("CommandManager"),
  Manager()
  {
  glog_Layer2.Debug ("Creating Command2MessageBridge on Server");
  glog_Layer2.Debug ("Creating Command2MessageBridge on Server -DONE- ");
  }

CommandManager::~CommandManager()
  {
  }

bool CommandManager::Subscribe(Manager *Man,Uint8 Type)
  {  
  assert(Type<NUMBER_OF_COMMANDS);
  TempLock templock(mutex);

  CommandList[Type].push_back(Man);
  Man->CommManager=this;
  return true;
  }

bool CommandManager::Unsubscribe(Manager *Man, Uint8 Type)
  {
  assert(Type<NUMBER_OF_COMMANDS);
  TempLock templock(mutex);

  std::list<Manager*>::iterator found=std::find(CommandList[Type].begin(),CommandList[Type].end(),Man);
  if(found!=CommandList[Type].end())
    {
    CommandList[Type].erase(found);
    return true;
    }
  
  return false;
  }

void CommandManager::SentStatL2Command(bool Mandatory)
  {
  if(((SDL_GetTicks()-LastTimeStatCommand)>TIME_BETWEEN_STATS_COMMANDS)||(Mandatory))
    {
    LastTimeStatCommand=SDL_GetTicks();
    /* We send the command to the manager. */
    StatL2Command *sL2Comm=new StatL2Command;
    sL2Comm->tmpInfL2=StatsL2;
    StatsL2.Reset();
    CommManager->AddCommand(sL2Comm);
    }
  }


void CommandManager::loop()
  {
  Command *comm=Commands.Get();

  SentStatL2Command();

  TempLock templock(mutex);

  Uint8 Type=comm->GetType();
  assert(Type<NUMBER_OF_COMMANDS);

  /* If there are no subcribed managers */
  if (CommandList[Type].begin() == CommandList[Type].end())
    {
    ++StatsL2.CommandsUnManaged;
    glog_Layer2.Debug("Unhandled Command " + IntToString(Type));
    }
  else
    {
    std::list<Manager*>::iterator i;
    std::list<Manager*>::iterator last=CommandList[Type].end();

    for (i = CommandList[Type].begin(); i != last; ++i)
      {
      ++StatsL2.CommandsManaged;
      (*i)->AddCommand(comm->clone());
      }
    }

  if(Type == POISON_COMMAND)
    {
    /* We finish this manager */
    delete comm;
    comm=0;

    SentStatL2Command(true);

    Stop();
    return;
    }

  delete comm;
  comm=0;
  }
