/* $Id: ActionManager.cpp,v 1.22 2002/04/26 19:37:48 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Define.h"

#include "3_Manager/ActionManager.h"
#include "2_Command/CommandManager.h"
#include "1_Message/MessageManager.h"

#include "3_Manager/ManagerCoordinator.h"

#include "Log/Logger.h"

#include <cassert>
#include <ctime>

using namespace std;

ActionManager::ActionManager(PlayerConnectionManager *playerConMan,RPManager *rpMan): 
  Manager(),
  LoopThread("ActionManager")
  {
  glog_Layer3.Debug ("Creating ActionManager on Server");
  assert(rpMan);
  assert(playerConMan);

  this->rpMan=rpMan;
  this->playerConMan=playerConMan;
  glog_Layer3.Debug ("Creating ActionManager on Server -DONE-");
  }

void ActionManager::ActionM (Message_C2S_ACTION * Mes)
  {
  assert (CommManager);

  glog_Layer3.Debug ("Command Action message");
  Uint32 timestamp=time(0);

  /* We only consider the first list of Actions, it has no sense that client send more by now */
  std::list<Action> actions=Mes->Getaction().front();
  bool correct=true;

  /* We verify the actions. */
  for(std::list<Action>::iterator it=actions.begin();it!=actions.end();++it)
    {
    if(!(*it).Verify())
      {
      correct=false;
      break;
      }
    (*it).PutAttrib("timestamp",IntToString(timestamp));
    }

  if(!correct) return;
  /* Syntax is correct, let's verify semantic. */

  TempLock ManagerLocked (gManagerCoordinatorMutex);

  /* We verify that all the actions belong to the player. */
  PlayerConnectionManager::iterator found=playerConMan->find(Mes->GetID());
  if(found!=playerConMan->end() && (*found).loginProcedureInfo.isReady())
    {
    Object::ID pID=(*found).PlayerID;

    for(std::list<Action>::iterator it=actions.begin();it!=actions.end();++it)
      {
      if((*it).GetSource()!=pID)
        {
        correct=false;
        break;
        }
      }
    }
  else
    {
    correct=false;
    }

  if(!correct) return;

  /* They are correct, We add them to RP Manager to be executed. */
  rpMan->Add(actions);

  (*found).perceptionInfo.ActionsToDeliver=actions;
  }

void ActionManager::MessageAdminType (Message * Mes)
  {
  try
    {
    switch (Mes->GetType ())
      {
      case ARIANNE_ACTION:
        {
        ActionM ((Message_C2S_ACTION *) Mes);
        }
        break;
      default:
        throw MessageManager::MesManUnMessageException (__FILE_AND_LINE__);
      }
    }
  catch (MessageManager::MesManUnMessageException &e)
    {
    glog_Layer3.Debug (__FILE_AND_LINE__, e.what ());

    throw ManagerUnCommandException (__FILE_AND_LINE__, Mes->GetType ());
    }
  }

void ActionManager::loop ()
  {
  Command *comm = Commands.Get ();

  try
    {
    switch (comm->GetType ())
      {
      case MESSAGE_M2C_ACTION_COMMAND:
        MessageAdminType (((MessageCommand *) comm)->GetMessage ());

        delete comm;
        comm = 0;
        break;
      case POISON_COMMAND:
        /* We finish this manager */
        delete comm;
        comm = 0;

        Stop ();
        return;
      default:
        throw ManagerUnCommandException (__FILE_AND_LINE__, comm->GetType ());
      }
    }
  catch (ManagerUnCommandException &e)
    {
    glog_Layer3.Debug (__FILE_AND_LINE__, e.what ());

    delete comm;
    comm = 0;
    }
  }
