/* $Id: AdminManager.cpp,v 1.32 2002/04/17 12:25:49 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "1_Message/MessagesTypes.h"
#include "1_Message/MessageManager.h"
#include "2_Command/CommandManager.h"
#include "3_Manager/AdminManager.h"
#include "Log/Logger.h"
#include <cassert>
#include "Define.h"
#include "Concurrent/Templock.h"
#include "Conversion/Conversion.h"

#include "SDL.h"

AdminManager::AdminManager(PlayerConnectionManager *pcm):
  Manager(),
  LoopThread("AdminManager"),
  PlayerConMan(pcm)
  {
  glog_Layer3.Debug ("Creating AdminManager on Server");
  glog_Layer3.Debug ("Creating AdminManager on Server -DONE-");
  };

void AdminManager::CloseConnection (Uint32 cID, std::string const &Text)
  {
  MessageCommand *comm = new MessageCommand (MESSAGE_C2M_ADMIN_COMMAND);
  Message_S2C_LOGOUT *ReplyMes = new Message_S2C_LOGOUT;
  ReplyMes->SetID (cID);
  ReplyMes->SetByeString (Text);
  comm->SetMessage (ReplyMes);

  CommManager->AddCommand (comm);
  }

void AdminManager::Login_SendLoginStage(Uint32 CId,std::string const& login)
  {
  PlayerConnectionManager::iterator found=PlayerConMan->find(CId);
  if(found!=PlayerConMan->end())
    {
    if((*found).loginProcedureInfo.StateOfID == PlayerInfo::invalid)
      {
      glog_Layer3.Debug ("We received a new NLOGIN:SEND_LOGIN message.");

      if(!(*found).loginProcedureInfo.BeginLogin(login))
        {
        glog_Layer3.Debug ("Receiving Login to an unknown account.");
        CloseConnection (CId,std::string ("Login incorrect."));
        PlayerConMan->Release(CId);
        return;
        }

      Uint8 ReplyType = 0;
      if((*found).loginProcedureInfo.StateOfID == PlayerInfo::waitPassword)
        {
        ReplyType = ARIANNE_METHOD_PASS;
        }
      #ifdef HAVE_CRYPTO
      else
      if((*found).loginProcedureInfo.StateOfID == PlayerInfo::create_w1)
        {
        ReplyType = ARIANNE_METHOD_RSA;
        }
      #endif
      else
        {
        glog_Layer3.Debug ("The State of the identification is bogus.");
        glog_Layer3.Debug ("We disconnect the client.");
        CloseConnection (CId,std::string ("I'm confused while login."));
        PlayerConMan->Release (CId);
        return;
        }

      MessageCommand *mescomm;
      Message_S2C_NLOGIN *ReplyMes;
      mescomm = new MessageCommand(MESSAGE_C2M_ADMIN_COMMAND);

      #ifdef HAVE_CRYPTO
      if((*found).loginProcedureInfo.StateOfID== PlayerInfo::create_w1)
        {
        ReplyMes = new Message_S2C_NLOGIN(ReplyType,(*found).loginProcedureInfo.GetHOfw1());
        }
      else
      #endif
        {
        ReplyMes = new Message_S2C_NLOGIN(ReplyType);
        }

      ReplyMes->SetID(CId);
      mescomm->SetMessage (ReplyMes);
      CommManager->AddCommand (mescomm);
      return;
      }
    }

  /* Error case */
  glog_Layer3.Debug ("Receiving NLOGIN:SEND_LOGIN twice.");
  glog_Layer3.Debug ("We disconnect the client.");
  CloseConnection (CId,std::string ("You sent NLOGIN:SEND_LOGIN twice."));
  PlayerConMan->Release (CId);
  }

void AdminManager::Login_SendPassStage(Uint32 CId,std::string const& password)
  {
  PlayerConnectionManager::iterator found=PlayerConMan->find(CId);
  if(found!=PlayerConMan->end())
    {
    if((*found).loginProcedureInfo.StateOfID == PlayerInfo::waitPassword)
      {
      if(!(*found).loginProcedureInfo.VerifyLogin(password))
        {
        glog_Layer3.Debug ("Incorrect Password.");
        CloseConnection (CId,std::string ("You sent incorrect Password."));
        PlayerConMan->Release (CId);
        return;
        }

      std::string lastLogin = (*found).loginProcedureInfo.GetLastLogin();

      /* If it is an old connection, we close the old one */
      PlayerConnectionManager::iterator found_oldConnection=PlayerConMan->find((*found).loginProcedureInfo.LoginInformation.getObjectID());
      if(found_oldConnection!=PlayerConMan->end())
        {
        glog_Layer3.Debug ("Double login: Disconnect older one.");
        CloseConnection ((*found).ConnectionID,std::string ("You log in once again."));
        PlayerConMan->Release ((*found_oldConnection).ConnectionID);
        }

      /* We complete now the Login stage */
      (*found).PlayerID=(*found).loginProcedureInfo.LoginInformation.getObjectID();
      (*found).loginProcedureInfo.CompleteLogin();

      glog_Layer3.Debug(lastLogin);
      MessageCommand *mescomm;
      Message_S2C_NLOGIN *ReplyMes;

      mescomm = new MessageCommand(MESSAGE_C2M_ADMIN_COMMAND);
      ReplyMes = new Message_S2C_NLOGIN(ARIANNE_SEND_CONFIRM,(*found).PlayerID,lastLogin);
      ReplyMes->SetID (CId);
      mescomm->SetMessage (ReplyMes);
      CommManager->AddCommand (mescomm);
      return;
      }
    }

  /* Error case */
  glog_Layer3.Debug ("Receiving NLOGIN:SEND_PASS in the wrong state.");
  glog_Layer3.Debug ("We disconnect the client.");
  CloseConnection (CId,std::string("You sent bogus NLOGIN:SEND_PASS."));
  PlayerConMan->Release (CId);
  }

#ifdef HAVE_CRYPTO
void AdminManager::Login_Send_W2(Uint32 CId,std::vector<Uint8> const &w2)
  {
  glog_Layer3.Debug ("Receiving W2");
  PlayerConnectionManager::iterator found=PlayerConMan->find(CId);
  if(found!=PlayerConMan->end())
    {
    if((*found).loginProcedureInfo.StateOfID == PlayerInfo::wait_w2)
      {
      if((*found).loginProcedureInfo.Setw2(w2))
        {
        MessageCommand *mescomm;
        Message_S2C_NLOGIN *ReplyMes;

        mescomm = new MessageCommand(MESSAGE_C2M_ADMIN_COMMAND);
        ReplyMes = new Message_S2C_NLOGIN(ARIANNE_SEND_M, (*found).loginProcedureInfo.Getm());
        ReplyMes->SetID (CId);
        mescomm->SetMessage (ReplyMes);
        CommManager->AddCommand (mescomm);
        glog_Layer3.Debug ("Sending M");
        return;
        }
      else
        {
        glog_Layer3.Debug ("Receiving bad w2.");
        glog_Layer3.Debug ("We disconnect the client.");
        CloseConnection (CId,std::string ("You sent NLOGIN:SEND_W2 with bad W2"));
        PlayerConMan->Release (CId);
        return;
        }
      }
    }

  /* Error case */
  glog_Layer3.Debug ("Receiving NLOGIN:SEND_W2 in the wrong state.");
  glog_Layer3.Debug ("We disconnect the client.");
  CloseConnection (CId,std::string ("You sent bogus NLOGIN:SEND_W2."));
  PlayerConMan->Release (CId);
  }

void AdminManager::Login_SendSignedM(Uint32 CId,std::vector<Uint8> const &signature)
  {
  PlayerConnectionManager::iterator found=PlayerConMan->find(CId);
  if(found!=PlayerConMan->end())
    {
    if((*found).loginProcedureInfo.StateOfID == PlayerInfo::wait_signature)
      {
      if(!(*found).loginProcedureInfo.VerifyRSA(signature))
        {
        glog_Layer3.Debug ("Incorrect Signature.");
        CloseConnection (CId,std::string ("You sent incorrect Signature."));
        PlayerConMan->Release (CId);
        return;
        }

      std::string lastLogin = (*found).loginProcedureInfo.GetLastLogin();

      /* If it is an old connection, we close the old one */
      PlayerConnectionManager::iterator found_oldConnection=PlayerConMan->find((*found).loginProcedureInfo.LoginInformation.getObjectID());
      if(found_oldConnection!=PlayerConMan->end())
        {
        glog_Layer3.Debug ("Double login: Disconnect older one.");
        CloseConnection ((*found_oldConnection).ConnectionID,std::string ("You log in once again."));
        PlayerConMan->Release ((*found_oldConnection).ConnectionID);
        }

      /* We complete now the Login stage */
      (*found).PlayerID=(*found).loginProcedureInfo.LoginInformation.getObjectID();
      (*found).loginProcedureInfo.CompleteLogin();

      glog_Layer3.Debug(lastLogin);
      MessageCommand *mescomm;
      Message_S2C_NLOGIN *ReplyMes;

      mescomm = new MessageCommand(MESSAGE_C2M_ADMIN_COMMAND);
      ReplyMes = new Message_S2C_NLOGIN(ARIANNE_SEND_CONFIRM,(*found).PlayerID,lastLogin);
      ReplyMes->SetID (CId);
      mescomm->SetMessage (ReplyMes);
      CommManager->AddCommand (mescomm);
      return;
      }
    }

  /* Error case */
  glog_Layer3.Debug ("Receiving NLOGIN:SEND_S_M_ in the wrong state.");
  glog_Layer3.Debug ("We disconnect the client.");
  CloseConnection (CId,std::string ("You sent bogus NLOGIN:SEND_S_M_."));
  PlayerConMan->Release (CId);
  }
#endif
void AdminManager::Login (Message_C2S_NLOGIN * Mes)  
  {
  assert (CommManager);

  Uint32 const CId = Mes->GetID();
  switch (Mes->GetSubType())
    {
    case ARIANNE_SEND_LOGIN:
      {
      std::string Login; VectorToString(Mes->GetData(),Login);
      Login_SendLoginStage(CId,Login);
      }
    break;
    case ARIANNE_SEND_PASS:
      {
      std::string Password; VectorToString(Mes->GetData(),Password);
      Login_SendPassStage(CId,Password);
      }
    break;

    #ifdef HAVE_CRYPTO
    case ARIANNE_SEND_W2:
      {
      Login_Send_W2(CId,Mes->GetData());
      }
    break;

    case ARIANNE_SEND_S_M_:
      {
      Login_SendSignedM(CId,Mes->GetData());
      }
    break;
    #endif

    /* Default means wrong type... */
    default:
      glog_Layer3.Debug ("Receiving unknown type NLOGIN Message: " + IntToString(Mes->GetSubType()));
      glog_Layer3.Debug ("We disconnect the client.");
      CloseConnection (CId,std::string ("You sent bogus NLOGIN message."));
      PlayerConMan->Release (CId);
      return;
    }
  }

void AdminManager::Logout (Message_C2S_LOGOUT * Mes)
  {
  assert (CommManager);

  glog_Layer3.Debug ("Command Logout message");

  // TODO: Parse stats and send it to statsManager.
  std::string stats=((Message_C2S_LOGOUT *) Mes)->GetStatistics();

  CloseConnection (Mes->GetID (), std::string ("Correct Logout Procedure."));

  /* We release now the connection */
  PlayerConMan->Release (Mes->GetID ());
  }

void
AdminManager::SystemInfo (Message_C2S_SYSTEM_INFO * Mes)
{
  glog_Layer3.Debug ("Command System Info message");

  //<- Process here the SYSTEM_INFO message
  if (Mes->GetOs () == "Win32")
    ++StatsL3.OS_Win32;
  if (Mes->GetOs () == "Linux")
    ++StatsL3.OS_Linux;
  if (Mes->GetOs () == "BeOS")
    ++StatsL3.OS_BeOS;
  if (Mes->GetOs () == "MacOS")
    ++StatsL3.OS_MacOS;
  if (Mes->GetOs () == "Other")
    ++StatsL3.OS_Other;
}

void AdminManager::NewConnection(Uint32 cID,std::string const &Text)
  {
  MessageCommand *mescomm;
  Message_S2C_SEND_VERSION *ReplyMes;

  PlayerConMan->Use(cID);
  PlayerConnectionManager::iterator found=PlayerConMan->find(cID);
  (*found).loginProcedureInfo.IPaddress=Text;

  mescomm = new MessageCommand(MESSAGE_C2M_ADMIN_COMMAND);
  ReplyMes = new Message_S2C_SEND_VERSION();
  ReplyMes->SetID(cID);

  //TODO: Convert to C++ code

  std::string MOTD;

  //FIXME-MAC: this needs fixed so that it finds the MOTD on MacOS
  #ifdef MACOSX
  FILE *Handle = fopen("../Resources/MOTD.arianne", "rb");
  #else
  FILE *Handle=fopen("MOTD.arianne","rb");
  #endif
  if(!Handle)
    {
    throw MOTDNotFoundException();
    }

  char Buffer[512];
  while(!feof(Handle))
    {
    int read=fread(Buffer,1,511,Handle);
    if(read>0) 
      {
      Buffer[read]=0;
      MOTD+=Buffer;
      }
    }
  fclose(Handle);
        
  ReplyMes->SetMOTD(MOTD);
  mescomm->SetMessage (ReplyMes);
  CommManager->AddCommand (mescomm);
  }


void
AdminManager::MessageAdminType (Message * Mes)
  {
  try
    {
    /* If player connection is not used we throw an exception. */
    if(PlayerConMan->find(Mes->GetID())==PlayerConMan->end())
      {
      throw BogusOldMessageException (__FILE_AND_LINE__);
      }

    switch (Mes->GetType ())
      {
      case ARIANNE_NLOGIN:
        Login ((Message_C2S_NLOGIN *) Mes);
        break;
      case ARIANNE_LOGOUT:
        Logout ((Message_C2S_LOGOUT *) Mes);
        break;
      case ARIANNE_SYSTEM_INFO:
        SystemInfo ((Message_C2S_SYSTEM_INFO *) Mes);
        break;
      default:
        throw MessageManager::MesManUnMessageException (__FILE_AND_LINE__);
      }
    }
  catch (MessageManager::MesManUnMessageException &e)
    {
    glog_Layer3.Debug (__FILE_AND_LINE__, e.what ());
    throw ManagerUnCommandException (__FILE_AND_LINE__, Mes->GetType ());
    }
  catch (PlayerConnectionManager::ManagerUsedConnectionException &e)
    {
    glog_Layer3.Debug (__FILE_AND_LINE__, e.what ());
    }
  catch (PlayerConnectionManager::ManagerUnUsedConnectionException &e)
    {
    glog_Layer3.Debug (__FILE_AND_LINE__, e.what ());
    }
  }

void AdminManager::SentStatL3Command (bool Mandatory)
  {
  if (((SDL_GetTicks () - LastTimeStatCommand)>TIME_BETWEEN_STATS_COMMANDS) || Mandatory)
    {
    LastTimeStatCommand = SDL_GetTicks ();
    /* We send the command to the manager. */
    StatL3Command *sL3Comm = new StatL3Command;
    sL3Comm->tmpInfL3 = StatsL3;
    StatsL3.Reset ();
    CommManager->AddCommand (sL3Comm);
    }
  }

void AdminManager::loop ()
  {
  Command *comm = Commands.Get ();

  SentStatL3Command ();

  try
    {
    /** We grant EXCLUSIVE MUTUAL ACCESS here */
    TempLock tmp(gManagerCoordinatorMutex);

    switch (comm->GetType ())
      {
      case MESSAGE_M2C_ADMIN_COMMAND:
        MessageAdminType (((MessageCommand *) comm)->GetMessage ());
        break;

      case CLOSECONNECTION_COMMAND:
        PlayerConMan->Release (((CloseConnectionCommand *) comm)->cID);
        break;

      case NEWCONNECTION_COMMAND:
        {
        NewConnection(((NewConnectionCommand *)comm)->cID,((NewConnectionCommand *)comm)->ipAddr);
        }
        break;

      case POISON_COMMAND:
        SentStatL3Command (true);
        Stop ();
        return;

      default:
        glog_Layer3.Debug(__FILE_AND_LINE__, "unknown message-type");
      }
    }
  catch (Exception &e)
    {
    glog_Layer3.Debug (__FILE_AND_LINE__, e.what ());
    }
/*  catch (...)
    {
    glog_Layer3.Debug("Unmanaged exception on AdminManager");
    }*/

  delete comm;
  }

