/* $Id: AdminManager.h,v 1.19 2002/04/10 20:59:47 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ARIANNE_AdminManager_H
#define ARIANNE_AdminManager_H

#include "1_Message/Message.h"
#include "Concurrent/Thread.h"

#include "1_Message/MessagesTypes.h"

#include "PlayerConnectionManager.h"
#include "ManagerCoordinator.h"

/** This manager does all the processing associated to the login, logout, and
    the reception of system info.
    This class is responsible for checking double logins, false password, and
    in the last case do the authentification process. */
class AdminManager: public Manager, public LoopThread
  {
  /** This method verify the message and send it to the appropiate method for being
      processed. */ 
  void MessageAdminType(Message *Mes);

  /** This method process the New Login Provedure */
  void Login(Message_C2S_NLOGIN *Mes);
  void Login_SendLoginStage(Uint32 CId,std::string const& login);
  void Login_SendPassStage(Uint32 CId,std::string const& password);

  #ifdef HAVE_CRYPTO
  void Login_Send_W2(Uint32 CId,std::vector<Uint8> const &w2);
  void Login_SendSignedM(Uint32 CId,std::vector<Uint8> const &signature);
  #endif
  
  /** This method process the Logout messages */
  void Logout(Message_C2S_LOGOUT *Mes);
  /** This method process the System Info messages */
  void SystemInfo(Message_C2S_SYSTEM_INFO *Mes);

  /** This method will send a Logout message closing the connection */
  void CloseConnection(Uint32 cID,std::string const &Text);

  /** This method is executed when a new connection appears. */
  void NewConnection(Uint32 cID,std::string const &ipAddress);

  /** Stats for the Server Stats */
  TemporalInfoLayer3 StatsL3;


  /** This is useful for knowing when the last message was sent to the StatsManager */
  Uint32 LastTimeStatCommand;
  /** Send the stats to StatManager if time between last stats is greater 
      than a given value, or if Mandatory is true, send them inconditionally. 
      This method resets the stats. */
  void SentStatL3Command(bool Mandatory=false);

  PlayerConnectionManager *PlayerConMan;

  /* entry-point, must be provided to LoopThread */
  virtual void loop();

  public:
  AdminManager(PlayerConnectionManager *PlayerConMan);
  virtual ~AdminManager() {};

  class BogusOldMessageException: public Exception
    {
    public:
    BogusOldMessageException(std::string const& where = "UNKNOWN")
      {
      message = where + " - Error receiving old message on closed connexion.";  
      }
    };

  class MOTDNotFoundException: public Exception
    {
    public:
    MOTDNotFoundException(std::string const& where = "UNKNOWN")
      {
      message = where + " - Error Arianne.MOTD not found.";  
      }
    };

  };

#endif
