/* $Id: LoginInfo.cpp,v 1.17 2002/03/17 21:39:28 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>

#include "Types.h"
#include "3_Manager/LoginInfo.h"

LoginInfo::~LoginInfo ()
{
#ifdef HAVE_CRYPTO
    delete key;
    key = 0;
#endif
}

LoginInfo::LoginInfo ()
{
  LoginType = invalid;
  login = "";
  hash = "";
#ifdef HAVE_CRYPTO
  key = 0;
#endif
  id = 0;
  objectid = Object::Invalid;
  accreditation = 0;
  last_ip = "";
  last_datetime = "";
  failures = 0;
}

LoginInfo::LoginInfo (std::string const &login)
{
  LoginType = invalid;
  this->login = login;
  hash = "";
#ifdef HAVE_CRYPTO
  key = 0;
#endif
  id = 0;
  objectid = Object::Invalid;
  accreditation = 0;
  last_ip = "";
  last_datetime = "";
  failures = 0;
}

LoginInfo::LoginInfo (std::string const &login, std::string const &mhash)
{
  this->login = login;
  hash = mhash;
#ifdef HAVE_CRYPTO
  key = 0;
#endif
  if ((login != "") && (hash != ""))
    LoginType = pass;
  else
    LoginType = invalid;
  id = 0;
  objectid = "";
  accreditation = 0;
  last_ip = "";
  last_datetime = "";
  failures = 0;
}

bool LoginInfo::invalidate ()
{
  LoginType = invalid;
  login = "";
  hash = "";
#ifdef HAVE_CRYPTO
  if (key != 0)
    delete key;
  key = 0;
#endif
  id = 0;
  objectid = Object::Invalid;
  accreditation = 0;
  last_ip = "";
  last_datetime = "";
  failures = 0;
  return true;
}

#ifdef HAVE_CRYPTO
LoginInfo::LoginInfo (std::string const &log, RSAKey * mkey)
{
  login = log;
  hash = "";
  key = mkey;
  if ((login != "") && (key != 0))
    LoginType = RSA;
  else
    LoginType = invalid;
  accreditation = 0;
  last_ip = "";
  last_datetime = "";
  failures = 0;
}
#endif

std::string const &
LoginInfo::getLogin () const
{
  return login;
}

bool LoginInfo::setLogin (std::string const &log)
{
  if (log != "")
    login = log;
  else
    return false;
  return true;
}

std::string const &
LoginInfo::getHash () const
{
  return hash;
}

bool LoginInfo::setHash (std::string const &mhash)
{
  if (mhash != "")
    hash = mhash;
  else
    return false;
  if (login != "")
    LoginType = pass;
  return true;
}

#ifdef HAVE_CRYPTO
RSAKey *
LoginInfo::getKey () const
{
  return key;
}

bool LoginInfo::setKey (RSAKey *mkey)
{
  if (mkey == 0)
    return false;
  if (key != 0)
    delete key;
  key = mkey;
  if (login != "")
    LoginType = RSA;
  return true;
}
#endif

bool LoginInfo::isValid () const
{
  return (LoginType != invalid);
}

LoginInfo::status LoginInfo::getLoginType () const
{
  return LoginType;
}

Uint32 LoginInfo::getID () const
{
  return id;
}

bool LoginInfo::setID (Uint32 nid)
{
  id = nid;
  return true;
}

Object::ID LoginInfo::getObjectID () const
{
  return objectid;
}

bool LoginInfo::setObjectID (Object::ID nid)
{
  objectid = nid;
  return true;
}

Uint8 LoginInfo::getAccreditation()
{
  return accreditation;
}

bool LoginInfo::setAccreditation(Uint8 acc)
{
  accreditation = acc;
  return true;
}

std::string const&
LoginInfo::getCurrentIP() const
{
  return current_ip;
}

bool LoginInfo::setCurrentIP(std::string const& ip)
{
  current_ip = ip;
  return true;
}

std::string const&
LoginInfo::getLastIP() const
{
  return last_ip;
}

bool LoginInfo::setLastIP(std::string const& ip)
{
  last_ip = ip;
  return true;
}

std::string const&
LoginInfo::getLastDateTime() const
{
  return last_datetime;
}

bool LoginInfo::setLastDateTime(std::string const& dt)
{
  last_datetime = dt;
  return true;
}

Uint16 LoginInfo::getFailures() const
{
  return failures;
}

bool LoginInfo::setFailures(Uint16 nfail)
{
  failures = nfail;
  return true;
}

bool LoginInfo::increaseFailures()
{
  failures++;
  return true;
}

