/* $Id: LoginInfo.h,v 1.17 2002/03/17 21:39:28 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2001 by Benjamin Lerman
    email                : quisar@quisar.ambre.net
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
 *  This class contains all login informations needed to allow login.      *
 ***************************************************************************/
#ifndef ARIANNE_login_h_
#define ARIANNE_login_h_

#include <string>
#include "Types.h"
#include "Crypto/Crypto.h"

#include "4_Object/Object.h"
//DOC: Document LoginInfo class.

/** This class contains all login informations needed to allow login.*/
class LoginInfo 
  {
  public:
  enum status {invalid,
  #ifdef HAVE_CRYPTO
               RSA,
  #endif
               pass};

  LoginInfo();
  LoginInfo(std::string const& login);
  LoginInfo(std::string const& login, std::string const& hash);
  #ifdef HAVE_CRYPTO
  LoginInfo(std::string const& login, RSAKey *key);
  #endif
  ~LoginInfo();

  /** Returns the Connection ID */
  Uint32 getID() const;
  /** Sets the Connection ID */
  bool setID(Uint32 nid);

  /** Returns the Object ID */
  Object::ID getObjectID() const;
  /** Sets the Object ID */
  bool setObjectID(Object::ID noid);

  /** Returns the accreditation */
  Uint8 getAccreditation();
  /** Sets the accreditation */
  bool setAccreditation(Uint8 acc);
  
  /** Returns the login */
  std::string const& getLogin() const;
  /** Sets the login */
  bool setLogin(std::string const& login);

  /** Returns the hash (password) */
  std::string const& getHash() const;
  /** Sets the hash (password) */
  bool setHash(std::string const& hash);

  #ifdef HAVE_CRYPTO
  /** Returns the RSA key */
  RSAKey *getKey() const;
  /** Sets the RSA key */
  bool setKey(RSAKey *mkey);
  #endif

  /** Returns the current IP address */
  std::string const& getCurrentIP() const;
  /** Sets the current IP address */
  bool setCurrentIP(std::string const& ip);

  /** Returns the last IP address */
  std::string const& getLastIP() const;
  /** Sets the last IP address */
  bool setLastIP(std::string const& ip);

  /** Returns the last DateTime */
  std::string const& getLastDateTime() const;
  /** Sets the last DateTime */
  bool setLastDateTime(std::string const& dt);

  /** Returns the number of failures since last login */
  Uint16 getFailures() const;
  /** Sets the number of failures since last login */
  bool setFailures(Uint16 nfail);
  /** Increase the number of failures since last login by one */
  bool increaseFailures();

  bool isValid() const;
  status getLoginType() const;
  bool invalidate();
 
  private:
  Uint32 id;
  Object::ID objectid;
  Uint8 accreditation;
  std::string login;
  std::string hash;
  #ifdef HAVE_CRYPTO
  RSAKey *key;
  #endif
  std::string current_ip;
  std::string last_ip;
  std::string last_datetime;
  Uint16 failures;
  status LoginType;
  };

#endif

