/* $Id: PerceptionManager.cpp,v 1.28 2002/04/26 19:37:48 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Define.h"
#include "2_Command/CommandManager.h"
#include "3_Manager/PerceptionManager.h"
#include "3_Manager/ManagerCoordinator.h"
#include "Log/Logger.h"
#include <cassert>
#include <ctime>
#include "SDL.h"

using namespace std;

PerceptionManager::PerceptionManager(PlayerConnectionManager *PlayerConMan,RPManager *RPMan): 
  Manager(),
  LoopThread("PerceptionManager")
  {
  glog_Layer3.Debug ("Creating PerceptionManager on Server");
  assert(PlayerConMan);
  assert(RPMan);

  this->PlayerConMan=PlayerConMan;
  this->RPMan=RPMan;
  glog_Layer3.Debug ("Creating PerceptionManager on Server -DONE-");
  };


bool PerceptionManager::BuildDeltaPerception (Object::ID pID, Message_S2C_PERCEPTION * mes)
  { 
  bool should_send=false;
  std::list<Object> actualPerception=RPMan->GetPerception(pID);
  glog_Layer3.Debug ("Perception.size(): "+IntToString(actualPerception.size()));

  /* This piece of code is for adding outgoing actions. */
  for(PlayerConnectionManager::iterator i=PlayerConMan->begin();i!=PlayerConMan->end();++i)
    {
    if((*i).loginProcedureInfo.isReady())
      {
      for(std::list<Object>::const_iterator oit=actualPerception.begin();oit!=actualPerception.end();++oit)
        {
        PlayerConnectionManager::iterator res=PlayerConMan->find((*oit).GetAttrib("id"));
        if(res!=PlayerConMan->end())
          {
          /* We add to the "oit" player the actions that the "i" player want to do */
          (*res).perceptionInfo.ActionsToSend.push_back((*i).perceptionInfo.ActionsToDeliver);
          }
        }
      }
    }    

  PlayerConnectionManager::iterator found=PlayerConMan->find(pID);
  if(found==PlayerConMan->end())
    {
    /* Object not found */
    throw Object::ObjectNotFoundException();
    }

  std::list<Object> lastPerception=(*found).perceptionInfo.LastPerception;

  //TODO: The deleted list can contain only the Object::ID, so we will save a lot of memory. */
  std::list<Object> deleted;
  std::list<Object> modified;
  std::list<Object> next_LastPerception;

  if(lastPerception.empty())
    {
    should_send=true;
    modified=actualPerception;
    next_LastPerception=actualPerception;
    actualPerception.clear();
    }

  std::list<Object>::const_iterator last=lastPerception.end();
  for(std::list<Object>::const_iterator it=lastPerception.begin();it!=last;++it)
    {
    bool exist = false;

    for(std::list<Object>::iterator j=actualPerception.begin();j!=actualPerception.end();++j)
      {
      if ((*j).GetAttrib("id") == (*it).GetAttrib("id"))
        {
	    exist = true;
        
        /* If both Objects are the same but has not exactly the same attribs. */
	    if(!(*j==*it))
	      {
	      modified.push_back(*j);
          should_send=true;
	      }

        next_LastPerception.push_back(*j);
        actualPerception.erase(j);
	    break;
        }
      }

    if(!exist)
      {
      deleted.push_back(*it);
      should_send=true;
      }
    }

  /* We add new elements that don't exist on the previous perception */
  for(std::list<Object>::const_iterator j=actualPerception.begin();j!=actualPerception.end();++j)
    {
    should_send=true;
    modified.push_back(*j);
    next_LastPerception.push_back(*j);
    }


  (*found).perceptionInfo.LastPerception=next_LastPerception;
  mes->SetObjectsModified(modified);
  mes->SetObjectsDeleted(deleted);
  mes->Setperception(Perception(IMAGE));
  mes->Settimestamp(time(0));
  mes->SetuID(pID);

  return should_send;

  }  

void PerceptionManager::PerceptionM ()
  {
  /** We grant EXCLUSIVE MUTUAL ACCESS here */
  TempLock ManagerLocked (gManagerCoordinatorMutex);
  PlayerConnectionManager::iterator i;

  for(i=PlayerConMan->begin();i!=PlayerConMan->end();++i)
    {
    if((*i).loginProcedureInfo.isReady())
      {
      Object::ID pID=(*i).PlayerID;

      try
        {
        glog_Layer3.Debug ("Command Perception message");
        MessageCommand *comm = new MessageCommand (MESSAGE_C2M_PERCEPTION_COMMAND);
        Message_S2C_PERCEPTION *ReplyMes = new Message_S2C_PERCEPTION;

        ReplyMes->SetID ((*i).ConnectionID);
        bool should_send=BuildDeltaPerception (pID, ReplyMes);
        comm->SetMessage (ReplyMes);

        if(should_send)
          { 
          CommManager->AddCommand (comm);
          }
        else
          {
          delete comm;
          }
        }
      catch (Object::ObjectNotFoundException &e)
        {
        glog_Layer3.Debug (__FILE__, e.what (),Log::error);
        }
      }
    }

  for(i=PlayerConMan->begin();i!=PlayerConMan->end();++i)
    {
    if((*i).loginProcedureInfo.isReady())
      {
      MessageCommand *comm = new MessageCommand (MESSAGE_C2M_ACTION_COMMAND);
      Message_S2C_ACTION *ReplyMes = new Message_S2C_ACTION;
      ReplyMes->SetID ((*i).ConnectionID);
      std::list<std::list<Action> > ActionList=(*i).perceptionInfo.ActionsToSend;
      ActionList.push_back(RPMan->Get((*i).PlayerID));

      ReplyMes->Setaction(ActionList);
      comm->SetMessage (ReplyMes);

      bool isEmpty=true;
      /* If there is no actions to send, we save the message. */
      for(std::list<std::list<Action> >::const_iterator it=ActionList.begin();it!=ActionList.end();++it)
        {
        if(!(*it).empty())
          {
          isEmpty=false;
          break;
          }
        }

      if(!isEmpty)
        {
        glog_Layer3.Debug ("Command Action message");
        CommManager->AddCommand (comm);
        }
      else
        {
        delete comm;
        }

      /* Now we empty the action queue of this player. */
      (*i).perceptionInfo.ActionsToSend.clear();
      (*i).perceptionInfo.ActionsToDeliver.clear();
      }
    }
  }

void PerceptionManager::loop ()
  {
  /* This piece of code still use polling, with is not elegant. A elegant way of
     doing this will be signaling the PerceptionManager each X units of time. */
  SDL_Delay (RPMan->GetTurnTime());

  if(Commands.isAvailable ())
    {
    Command *comm = Commands.Get ();
    if (comm->GetType () == POISON_COMMAND)
      {
      /* We finish this manager */
      delete comm;
      comm = 0;

      Stop ();
      return;
      }
    }

  PerceptionM ();
  }
