/* $Id: StatsManager.cpp,v 1.23 2002/04/05 14:00:45 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Define.h"
#include "InitPath.h"

#include "2_Command/CommandManager.h"
#include "3_Manager/StatsManager.h"

#include "Log/Logger.h"
#include "Parser/Parser.h"
#include "Conversion/Conversion.h"

#include <cassert>
#include <ctime>
#include <cstdio>
#include <fstream>

using namespace std;

#ifndef WIN32
#include <unistd.h> // getpid
#endif

/** The Arianne Ini file name */
#define INI_FILE "Server.xml"


bool StatsManager::LoadStats()
  {
  std::ifstream HandleDef((g_etcdir+INI_FILE).c_str());
  if(!HandleDef)
    {
    glog_Layer3.Debug( __FILE_AND_LINE__ , "Unable to find "+string(INI_FILE)+" file",Log::fatal_error);
    return false;
    }
  else
    {
    PersistentString
    pvServerName (g_etcdir+INI_FILE, "ServerCharacteristics", "ServerName");
    PersistentString
    pvServerNote (g_etcdir+INI_FILE, "ServerCharacteristics", "ServerNote");
    PersistentString
    pvServerOS (g_etcdir+INI_FILE, "ServerCharacteristics", "ServerOS");
    PersistentString
    pvServerIP (g_etcdir+INI_FILE, "ServerCharacteristics", "ServerIP");

    serverInf.ServerName = pvServerName;
    serverInf.ServerNote = pvServerNote;
    serverInf.ServerOS = pvServerOS;
    serverInf.ServerIP = pvServerIP;
    serverInf.MaxNumberPlayers = MAX_NUMBER_CONNECTIONS;
    }

  std::ifstream HandleStats((g_vardir+"LATEST").c_str());
  if(!HandleStats)
    {
    glog_Layer3.Debug (__FILE_AND_LINE__, "Server running for first time.");

    allTimeperInf.StartTime = time (0);
    perInf.StartTime = time (0);
    }
  else
    {
    PersistentInt
    pvNumberOfPlayers (g_vardir+"LATEST", "AllTimeStats", "NumberOfPlayers");
    PersistentInt
    pvUptime (g_vardir+"LATEST", "AllTimeStats", "Uptime");
    PersistentInt
    pvStartTime (g_vardir+"LATEST", "AllTimeStats", "StartTime");
    PersistentInt
    pvBytesRecv (g_vardir+"LATEST", "AllTimeStats", "BytesRecv");
    PersistentInt
    pvBytesSend (g_vardir+"LATEST", "AllTimeStats", "BytesSend");
    PersistentInt
    pvOS_Win32 (g_vardir+"LATEST", "AllTimeStats", "OS_Win32");
    PersistentInt
    pvOS_Linux (g_vardir+"LATEST", "AllTimeStats", "OS_Linux");
    PersistentInt
    pvOS_BeOS (g_vardir+"LATEST", "AllTimeStats", "OS_BeOS");
    PersistentInt
    pvOS_MacOS (g_vardir+"LATEST", "AllTimeStats", "OS_MacOS");
    PersistentInt
    pvOS_Other (g_vardir+"LATEST", "AllTimeStats", "OS_Other");
    PersistentInt
    pvServerCrashes (g_vardir+"LATEST", "AllTimeStats", "ServerCrashes");

    allTimeperInf.NumberOfPlayers = pvNumberOfPlayers;
    allTimeperInf.Uptime = pvUptime;
    allTimeperInf.StartTime = pvStartTime;
    allTimeperInf.BytesRecv = pvBytesRecv;
    allTimeperInf.BytesSend = pvBytesSend;
    allTimeperInf.OS_Win32 = pvOS_Win32;
    allTimeperInf.OS_Linux = pvOS_Linux;
    allTimeperInf.OS_BeOS = pvOS_BeOS;
    allTimeperInf.OS_MacOS = pvOS_MacOS;
    allTimeperInf.OS_Other = pvOS_Other;
    allTimeperInf.ServerCrashes = pvServerCrashes;

    std::ifstream Crash((g_vardir+"SERVER_RUNNING").c_str());
    if(Crash)
      {
      ++allTimeperInf.ServerCrashes;
      }

    perInf.StartTime = time (0);
    }

  return true;
  }

bool StatsManager::SaveStats ()
  {
  return dump ((g_vardir+"LATEST").c_str());
  }


bool StatsManager::FlushStats ()
  {
  std::string Buffer;
  Buffer = g_logdir+IntToString(time (0) / REPORT_FILE_GENERATION_TIME)+".log";

  if (LastTimeDump == (Uint32)(time (0) / REPORT_FILE_GENERATION_TIME))
    {
    LastTimeDump = time (0) / REPORT_FILE_GENERATION_TIME;
    return dump (Buffer);
    }
  else
    {
    Buffer = g_logdir+IntToString(LastTimeDump)+".log";
    bool r = dump (Buffer);

    /* We update the permanent stats */
    perInf.NumberOfPlayers += tmpInfL4.NumberOfPlayers;
    perInf.Uptime = (Uint32) difftime (time (0), perInf.StartTime);
    perInf.BytesRecv += tmpInfL0.BytesRecvSesion;
    perInf.BytesSend += tmpInfL0.BytesSentSesion;
    perInf.OS_Win32 += tmpInfL3.OS_Win32;
    perInf.OS_Linux += tmpInfL3.OS_Linux;
    perInf.OS_BeOS += tmpInfL3.OS_BeOS;
    perInf.OS_MacOS += tmpInfL3.OS_MacOS;
    perInf.OS_Other += tmpInfL3.OS_Other;

    allTimeperInf.NumberOfPlayers += tmpInfL4.NumberOfPlayers;
    allTimeperInf.BytesRecv += tmpInfL0.BytesRecvSesion;
    allTimeperInf.BytesSend += tmpInfL0.BytesSentSesion;
    allTimeperInf.OS_Win32 += tmpInfL3.OS_Win32;
    allTimeperInf.OS_Linux += tmpInfL3.OS_Linux;
    allTimeperInf.OS_BeOS += tmpInfL3.OS_BeOS;
    allTimeperInf.OS_MacOS += tmpInfL3.OS_MacOS;
    allTimeperInf.OS_Other += tmpInfL3.OS_Other;


    /* And reset temporal one */
    tmpInfL0.Reset ();
    tmpInfL1.Reset ();
    tmpInfL2.Reset ();
    tmpInfL3.Reset ();
    tmpInfL4.Reset ();

    LastTimeDump = time (0) / REPORT_FILE_GENERATION_TIME;
    return r;
    }
  }

bool StatsManager::dump (std::string const& Filename)
  {
  std::ofstream dump(Filename.c_str());
  if(!dump)
    return false;

  dump << "<file>"<< endl <<"<title>Stats file for Arianne </title>" << std::endl;
  time_t actualtime = time (0);
  dump << "<time>" << ctime (&actualtime) << "</time>" << std::endl;

  dump << "<ServerCharacteristics>" << endl;
  dump << "  <ServerName>" << serverInf.ServerName << "</ServerName>" << std::endl;
  dump << "  <ServerNote>" << serverInf.ServerNote << "</ServerNote>" << std::endl;
  dump << "  <ServerOS>" << serverInf.ServerOS << "</ServerOS>"<< std::endl;
  dump << "  <ServerIP>" << serverInf.ServerIP << "</ServerIP>" << std::endl;
  dump << "  <MaxNumberPlayers>" << serverInf.MaxNumberPlayers << "</MaxNumberPlayers>" << std::endl;
  dump << "</ServerCharacteristics>" << endl;
  dump << std::endl;

  PermanentInfo tmpStats = perInf;
  tmpStats.NumberOfPlayers += tmpInfL4.NumberOfPlayers;
  tmpStats.Uptime = (Uint32) difftime (time (0), perInf.StartTime);
  tmpStats.BytesRecv += tmpInfL0.BytesRecvSesion;
  tmpStats.BytesSend += tmpInfL0.BytesSentSesion;
  tmpStats.OS_Win32 += tmpInfL3.OS_Win32;
  tmpStats.OS_Linux += tmpInfL3.OS_Linux;
  tmpStats.OS_BeOS += tmpInfL3.OS_BeOS;
  tmpStats.OS_MacOS += tmpInfL3.OS_MacOS;
  tmpStats.OS_Other += tmpInfL3.OS_Other;

  PermanentInfo ATtmpStats = allTimeperInf;
  ATtmpStats.NumberOfPlayers += tmpInfL4.NumberOfPlayers;
  ATtmpStats.Uptime = allTimeperInf.Uptime + tmpStats.Uptime;
  ATtmpStats.BytesRecv += tmpInfL0.BytesRecvSesion;
  ATtmpStats.BytesSend += tmpInfL0.BytesSentSesion;
  ATtmpStats.OS_Win32 += tmpInfL3.OS_Win32;
  ATtmpStats.OS_Linux += tmpInfL3.OS_Linux;
  ATtmpStats.OS_BeOS += tmpInfL3.OS_BeOS;
  ATtmpStats.OS_MacOS += tmpInfL3.OS_MacOS;
  ATtmpStats.OS_Other += tmpInfL3.OS_Other;

  dump << "<AllTimeStats>" << endl;
  dump << "  <NumberOfPlayers>" << ATtmpStats.NumberOfPlayers << "</NumberOfPlayers>" << std::endl;
  dump << "  <Uptime>" << ATtmpStats.Uptime << "</Uptime>" << std::endl;
  dump << "  <StartTime>" << ATtmpStats.StartTime << "</StartTime>" << std::endl;
  dump << "  <BytesRecv>" << ATtmpStats.BytesRecv << "</BytesRecv>" << std::endl;
  dump << "  <BytesSend>" << ATtmpStats.BytesSend << "</BytesSend>" << std::endl;
  dump << "  <OS_Win32>" << ATtmpStats.OS_Win32 << "</OS_Win32>" << std::endl;
  dump << "  <OS_Linux>" << ATtmpStats.OS_Linux << "</OS_Linux>" << std::endl;
  dump << "  <OS_BeOS>" << ATtmpStats.OS_BeOS << "</OS_BeOS>" << std::endl;
  dump << "  <OS_MacOS>" << ATtmpStats.OS_MacOS << "</OS_MacOS>" << std::endl;
  dump << "  <OS_Other>" << ATtmpStats.OS_Other << "</OS_Other>" << std::endl;
  dump << "  <ServerCrashes>" << ATtmpStats.ServerCrashes << "</ServerCrashes>"<< std::endl;
  dump << "</AllTimeStats>" << endl;
  dump << std::endl;

  dump << "<ActualExecutionStats>" << endl;
  dump << "  <NumberOfPlayers>" << tmpStats.NumberOfPlayers << "</NumberOfPlayers>" << std::endl;
  dump << "  <Uptime>" << tmpStats.Uptime << "</Uptime>" << std::endl;
  dump << "  <StartTime>" << tmpStats.StartTime << "</StartTime>" << std::endl;
  dump << "  <BytesRecv>" << tmpStats.BytesRecv << "</BytesRecv>" << std::endl;
  dump << "  <BytesSend>" << tmpStats.BytesSend << "</BytesSend>" << std::endl;
  dump << "  <OS_Win32>" << tmpStats.OS_Win32 << "</OS_Win32>" << std::endl;
  dump << "  <OS_Linux>" << tmpStats.OS_Linux << "</OS_Linux>" << std::endl;
  dump << "  <OS_BeOS>" << tmpStats.OS_BeOS << "</OS_BeOS>" << std::endl;
  dump << "  <OS_MacOS>" << tmpStats.OS_MacOS << "</OS_MacOS>" << std::endl;
  dump << "  <OS_Other>" << tmpStats.OS_Other << "</OS_Other>" << std::endl;
  dump << "  <ServerCrashes>" << tmpStats.ServerCrashes << "</ServerCrashes>"<< std::endl;
  dump << "</ActualExecutionStats>" << endl;
  dump << std::endl;

  dump << "</file>" << endl;

  return dump.good();
  }


StatsManager::StatsManager ():
Manager (), LoopThread ("StatsManager")
  {
  glog_Layer3.Debug ("Creating StatsManager on Server");
  try
    {
    LoadStats ();
    }
  catch(PersistentVar::ParserFileNotFoundException &e)
    {
    glog_Layer3.Debug("Can't open file or syntax is not correct",Log::fatal_error);
    throw;
    }
  LastTimeDump = time (0) / REPORT_FILE_GENERATION_TIME;

  std::ofstream Handle((g_vardir+"SERVER_RUNNING").c_str());
#ifndef WIN32
  Handle << getpid() << endl;
#endif
  glog_Layer3.Debug ("Creating StatsManager on Server -DONE-");
  }

StatsManager::~StatsManager ()
  {
  remove ((g_vardir+"SERVER_RUNNING").c_str());

  SaveStats ();
  }

void
StatsManager::loop ()
  {
  Command *comm = Commands.Get();

  try
    {
    switch (comm->GetType ())
      {
      case STAT_L0_COMMAND:
        tmpInfL0 = tmpInfL0 + ((StatL0Command *) comm)->tmpInfL0;

        delete comm;
        comm = 0;
        break;
      case STAT_L1_COMMAND:
        tmpInfL1 = tmpInfL1 + ((StatL1Command *) comm)->tmpInfL1;

        delete comm;
        comm = 0;
        break;
      case STAT_L2_COMMAND:
        tmpInfL2 = tmpInfL2 + ((StatL2Command *) comm)->tmpInfL2;

        delete comm;
        comm = 0;
        break;
      case STAT_L3_COMMAND:
        tmpInfL3 = tmpInfL3 + ((StatL3Command *) comm)->tmpInfL3;

        delete comm;
        comm = 0;
        break;
      case STAT_L4_COMMAND:
        tmpInfL4 = tmpInfL4 + ((StatL4Command *) comm)->tmpInfL4;

        delete comm;
        comm = 0;
        break;
      case STAT_L5_COMMAND:

        delete comm;
        comm = 0;
        break;
      case POISON_COMMAND:
        /* We finish this manager */
        delete comm;
        comm = 0;

        SaveStats ();
        Stop ();

        return;
        break;
      default:
        throw ManagerUnCommandException (__FILE_AND_LINE__, comm->GetType ());
      }
    }
  catch (ManagerUnCommandException &e)
    {
    glog_Layer3.Debug (__FILE_AND_LINE__, e.what ());

    delete comm;
    comm = 0;
    }

  FlushStats ();
  }
