/* $Id: MiguelActions.cpp,v 1.13 2002/04/28 07:48:19 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "4_Object/Object.h"
#include "4_Object/Action.h"
#include "5_Rp/WorldManager.h"
#include "5_Rp/RPManager.h"

#include <vector>
#include <cstdio>
#include <cmath>

#include "Log/Logger.h"
#include "4_Object/Subtypes.h"
#include "Conversion/StringTool.h"

using namespace std;

/**
Move
  type=move
  source
  destination=x y z 
  orientation=y r p

This action moves the object from one place to another in straigh line.
TODO: A* like algorithm would be created on the client, server works with pair of points only 
*/
pair<RPManager::ActionResult,bool> MoveActionPrototype(WorldManager& worldMan, Object& object, Action const& action)
  {
  //TODO: Orientation is ignored.
  string objectPosition=object.GetAttrib("position");
  string objectDestination=action.GetAttrib("destination");

  Position ObjectPos(objectPosition);
  Position ObjectDes(objectDestination);

  /* We make sure we are not trying to move too much. 1 meter per turn.
     This value should be based on some Object metric. */
  float Distance=ObjectPos.Distance(ObjectDes);
  float speed=10;;

  if(action.HasAttrib("speed"))
    {
    speed=atof(action.GetAttrib("speed").c_str());
    if(speed>10)
      {
      speed=10;
      }
    }

  if(Distance>10)
    {
    float x,y,z;

    x=ObjectDes.m_x-ObjectPos.m_x;
    y=ObjectDes.m_y-ObjectPos.m_y;
    z=ObjectDes.m_z-ObjectPos.m_z;

    float xp=(10*x)/Distance;
    float yp=(10*y)/Distance;
    float zp=(10*z)/Distance;
   
    /* We update position */
    char buffer[256];
    sprintf(buffer,"%f %f %f", ObjectPos.m_x+xp, ObjectPos.m_y+yp, ObjectPos.m_z+zp);

    object.PutAttrib("position",buffer);

    glog_Layer5.Debug(std::string("Action Move not complete: ")+ buffer + std::string(" -Object ID : ")+object.GetAttrib("id"));

    /* Also update orientation */
    float roll=atan2(ObjectPos.m_y+yp,ObjectPos.m_x+xp);
    sprintf(buffer,"%f %f %f", 0.0, roll, 0.0);

    object.PutAttrib("orientation",buffer);

     /* And finally speed */
    sprintf(buffer,"%f %f %f", xp, yp, zp);

    object.PutAttrib("speed",buffer);

    /* Action is not finished, we don't eliminate it. */
    return pair<RPManager::ActionResult,bool> (RPManager::incomplete,true);
    }
  else
    {
    object.PutAttrib("position",objectDestination);
    object.DelAttrib("speed");
    glog_Layer5.Debug(std::string("Action Move Executed : ")+ objectDestination + std::string(" -Object ID : ")+object.GetAttrib("id"));

    /* Action is finished, we eliminate it. */
    return pair<RPManager::ActionResult,bool> (RPManager::complete_success,false);
    }
  }

/**
Chat
  type=chat
  source
  content=text   

This action just execute a chat message on server.
*/
pair<RPManager::ActionResult,bool>  ChatActionPrototype(WorldManager& worldMan, Object& object, Action const& action)
  {
  glog_Layer5.Debug(std::string("Action Chat Executed: ")+ action.GetAttrib("content") + std::string(" -Object ID : ")+object.GetAttrib("id"));
  return pair<RPManager::ActionResult,bool> (RPManager::complete_success,false);
  }

/**
Cancel
  type=cancel

This action just execute a cancel.
*/
pair<RPManager::ActionResult,bool>  CancelActionPrototype(WorldManager& worldMan, Object& object, Action const& action)
  {
  glog_Layer5.Debug(std::string("Action Cancel Executed: ")+ std::string(" -Object ID : ")+object.GetAttrib("id"));
  return pair<RPManager::ActionResult,bool> (RPManager::complete_success,false);
  }

/**
Get
  type=get
  source
  slot
  objectID 

This action gets an Object and add it to the Slot of the source Object.
The Object is removed from the map.
*/
pair<RPManager::ActionResult,bool>  GetActionPrototype(WorldManager& worldMan, Object& object, Action const& action)
  {
  if(!object.isUsedSlot(action.GetAttrib("slot")))
    {
    Object object2insert=worldMan.Get(action.GetAttrib("objectID"));

    Position ObjectPos(object.GetAttrib("position"));
    Position Object2insertPos(object2insert.GetAttrib("position"));

    /* We make sure we are not trying to get something that is far: 1 meter.
       This value should be based on some Object metric. */
    Uint32 Distance=Uint32(ObjectPos.Distance(Object2insertPos));

    if(Distance<10)
      {
      /* We add the object to the slot AND remove it from the world. */
      worldMan.Del(object2insert);

      object2insert.DelAttrib("position");
      object.AddObject2Slot(action.GetAttrib("slot"),object2insert);

      glog_Layer5.Debug(std::string("Action Get Executed: ")+ object2insert.GetAttrib("id") + std::string(" -Object ID : ")+object.GetAttrib("id"));
      return pair<RPManager::ActionResult,bool> (RPManager::complete_success,true);
      }
    else
      {
      /* We add the object to the slot AND remove it from the world. */
      glog_Layer5.Debug("Action Get Invalid: Too far from object" + std::string(" -Object ID : ")+object.GetAttrib("id"));
      return pair<RPManager::ActionResult,bool> (RPManager::complete_fail,false);
      }
    }

  glog_Layer5.Debug("Action Get Invalid: Slot is used." + std::string(" -Object ID : ")+object.GetAttrib("id"));
  return pair<RPManager::ActionResult,bool> (RPManager::complete_fail,false);
  }

/**
Put
  type=put
  source
  slot

This action puts an Object on floor and remove it to the Slot of the source Object.
The Object is added to the map.
*/
pair<RPManager::ActionResult,bool>  PutActionPrototype(WorldManager& worldMan, Object& object, Action const& action)
  {
  if(object.isUsedSlot(action.GetAttrib("slot")))
    {
    Object object2insert=object.RemoveObject2Slot(action.GetAttrib("slot"));
    object2insert.PutAttrib("position",object.GetAttrib("position"));

    /* We add the object to the world. */
    //TODO: We perhaps should check that there is room for the object in the floor. */
    worldMan.Add(object2insert);
    glog_Layer5.Debug(std::string("Action Put Executed: ")+ object2insert.GetAttrib("id")+ std::string(" -Object ID : ")+object.GetAttrib("id"));
    return pair<RPManager::ActionResult,bool> (RPManager::complete_success,true);
    }

  glog_Layer5.Debug("Action Put Invalid: Slot is not used."+ std::string(" -Object ID : ")+object.GetAttrib("id"));
  return pair<RPManager::ActionResult,bool> (RPManager::complete_fail,false);
  }
