/* $Id: MiguelRPManager.cpp,v 1.11 2002/04/23 19:11:53 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "MiguelRPManager.h"
#include "MiguelActions.h"
#include "Log/Logger.h"

using namespace std;

MiguelRPManager::MiguelRPManager(WorldManager *worldMan): RPManager(worldMan)
  {
  Register("move",MoveActionPrototype);
  Register("chat",ChatActionPrototype);
  Register("get",GetActionPrototype);
  Register("put",PutActionPrototype);
  Register("cancel",CancelActionPrototype);
  }

MiguelRPManager::~MiguelRPManager()
  {
  }

void MiguelRPManager::Register(Action::Type const& type, ActionPrototype prototype)
  {
  pair<std::map<Action::Type, ActionPrototype>::iterator,bool> result=m_actionManager.insert(pair<Action::Type, ActionPrototype>(type,prototype));
  if(result.second==false)
    {
    std::map<Action::Type, ActionPrototype>::iterator it=m_actionManager.find(type);
    if(it!=m_actionManager.end())
      {
      (*it).second=prototype;
      }
    else
      {
      throw std::exception();
      }
    }
  }

void MiguelRPManager::Unregister(Action::Type const& type)
  {
  m_actionManager.erase(type);
  }


RPManager::ActionResult MiguelRPManager::Execute(Action const& action)
  {
  try
    {
    Action::Type actionType=action.GetType();
    Object object=worldMan->Get(action.GetSource());

    std::map<Action::Type, ActionPrototype>::iterator it=m_actionManager.find(actionType);
    if(it!=m_actionManager.end())
      {
      pair<RPManager::ActionResult,bool> result=(*it).second(*worldMan,object,action);

      /* We add the modified object to the world */
      if(result.second==true)
        {
        worldMan->Add(object);
        }

      return result.first;
      }
    else
      {
      throw std::exception();
      }  
    }
  catch(...)
    {
    glog_Layer5.Debug("An exception happened inside action method: Contact arianne-general@lists.sf.net about the event");
    return RPManager::complete_fail;
    }
  }
