/* $id: PythonRPManager.h,v 1.3 2002/03/08 12:21:25 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2002 by Brian Rowe
    email                : briprowe@yahoo.com
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cassert>
#include <iostream>

// Added include prefixes -waba
#include "PythonRPManager.h"
#include "Log/Logger.h"
#include "InitPath.h"
#include "script/Py2Attributes.h"

PythonRPManager::PythonRPManager(WorldManager *worldMan):
  RPManager(worldMan), 
  #ifndef WIN32
  pyth(g_libdir + "arianne/python/"),
  #else
  pyth("python/"),
  #endif
  m_tab(worldMan)
  {

  worldman = worldMan;  // Not responsible for destroying this

  // Start the Python Interpreter
  try 
    {
      
      // Tell Python to load the Arianne module
      glog_Layer5.Debug("Attempting to load the main Python module");
      arianne = pyth.LoadModule("arianne");
      PyObject *ptrs = CreatePyPointers(m_tab.GetPyPointers());
      pyth.RunFunction(arianne, "SetPointers", ptrs);
      glog_Layer5.Debug("Attempting to load the main Python module -DONE-");
      
      /**************************
       This is being dropped for now.
       **************************
      // Get a list of scripts
      PyObject *scriptList;
      scriptList = pyth.RunFunction(arianne, "GetScripts", NULL);
      
      // for every script in the list, craeate an RPScript and enter it
      // into the system
      BuildScripts(scriptList);
       *************************/
      
      //Py_XDECREF(scriptList);
      Py_XDECREF(ptrs);
    }
  catch(PythonLanguage::PyException &e)
    {
      //Py_XDECREF(scriptList);
      glog_Layer5.Debug(e.what(), Log::fatal_error); 
      throw;
    }
}

PythonRPManager::~PythonRPManager()
{
  try 
    {
      // For every script in the system tell it to terminate
 //     std::map<Action::Type, RPScript>::iterator end = m_Scripts.end();
 //     std::map<Action::Type, RPScript>::iterator iter;
 //     for( iter = m_Scripts.begin(); iter < end; iter++)
 //	{
 //	  (*iter).second.Shutdown();
 //	}
      Py_XDECREF(arianne);
      // Tell Python to shutdown
      pyth.Shutdown();
    }
  catch(PythonLanguage::PyStopException &e)
    {
      glog_Layer5.Debug(e.what(), Log::error);
    }
  catch(PythonLanguage::PyException &e)
    {
      glog_Layer5.Debug(e.what(), Log::error);
    }
}

/******************************************************
I'm going to remove this for now...
*******************************************************
RPManager::ActionResult PythonRPManager::Execute(Action const& action)
{
  // I am assuming that python is running
  assert(Py_IsInitialized());
  // I am assuming that the action is formated properly
  assert(action.Verify());
  
  try
    {
      // If the type is not registered, log the fact that a non-registered
      // action has been requested and return a blank ActionResult
      std::map<Action::Type, RPScript>::iterator end = m_Scripts.end();
      if ( m_Scripts.find(action.type()) == end )
	throw(std::exception());
      
      // Execute the action
      std::pair<RPManager::ActionResult, bool> result;
      result = m_Scripts[action.type()].Execute(action);
      
      return(result.first);      
    }
  catch(std::exception &e)
    {
      glog_Layer5.Debug("An unknown Action has been attempted", Log::warning);
      return RPManager::complete_fail;
    }
}
********************************************************/

RPManager::ActionResult PythonRPManager::Execute(const Action& action)
{
  PyObject *args;
  
  // Convert the action into a dictionary to be sent to Python.
  Py2Attributes pyob(action);
  args = pyob.GetPython();

  // Function parameters are sent wrapped in tuples
  PyObject* tup = PyTuple_New(1);
  PyTuple_SetItem(tup, 0, args);
  Py_INCREF(arianne);
  PyObject *result = pyth.RunFunction(arianne, "Execute", tup);
  Py_DECREF(arianne);
  long res = PyInt_AsLong(result);
  switch(res)
    {
    case 1: // complete_success
      Py_XDECREF(args);
      Py_XDECREF(result);
      return(RPManager::complete_success);
    case 2: // complete_fail
      Py_XDECREF(args);
      Py_XDECREF(result);
      return(RPManager::complete_fail);
    case 3:  // incomplete
      Py_XDECREF(args);
      Py_XDECREF(result);
      return(RPManager::incomplete);
    default: // Bad!!
      Py_XDECREF(args);
      Py_XDECREF(result);
      glog_Layer5.Debug("Invalid results returned from a Python script", 
                        Log::warning);
      return(RPManager::complete_fail);
    }
}

/*  Unused for now
void BuildScripts(PyObject *scriptList)
{
  assert(Py_IsInitialized());
  assert(PyList_Check(scriptList));   // assuming that scriptList is a list

  // GetItem functions do not create a new reference,
  // so I should not have to DECREF element....
  PyObject *element;

  // For every element in scriptList
  for(int i = 0; i < PyList_Size(scriptList); i++)
    {

      element = PyList_GetItem(scriptList, i);

      // if the first element is not a string  then
      // log the event as a handled_exception and go to the next element
      if ( !PyString_Check(PyTuple_GetItem(element, 0)) )
	{
	  glog_Layer5.Debug("Tried to build an RPScript with invalid input",
			    Log::managed_exception);
	  continue;
	}
      // if the first element is not in m_Scripts then go to the next element
      std::string type = PyString_AsString(PyTuple_GetItem(element, 0));
      if ( m_Scripts.find(type) == m_Scripts.end() )
	{
	  glog_Layer5.Debug("Tried to load a script twice", Log::warning);
	  continue;
	}
      
      // build a PyRPScript with the second element and a pointer to the
      // WorldManager so that the script can manipulate objects.
      PyRPScript script(PyTuple_GetItem(element, 1), worldman);
      
      // insert w/ first element = key and the PyRPScript = value into
      // m_Scripts
      m_Scripts[type] = script;
    }
  
} */

PyObject *PythonRPManager::CreatePyPointers(std::vector<void*> ptrs)
{
  // Check the length of ptrs
  int i = ptrs.size();

  // create a new tuple
  PyObject *retval = PyTuple_New(i);

  for ( int x = 0; x < i; x++ )
    {
      if (PyTuple_SetItem(retval,x,PyCObject_FromVoidPtr(ptrs[x], NULL)) != 0)
	throw PythonLanguage::PyException();
    }


  return(retval);
}
