/* $Id: RPManager.h,v 1.23 2002/04/26 19:37:48 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef ARIANNE_RPManager_H 
#define ARIANNE_RPManager_H

#include "Manager/Manager.h"

#include "Concurrent/Thread.h"
#include "Concurrent/Mutex.h"

#include "4_Object/Object.h"
#include "4_Object/Action.h"

#include "5_Rp/WorldManager.h"

/* How much milliseconds between updates. Turn time. */
#define MINIMUM_TURN_TIME 100
#define DEFAULT_TURN_TIME 2500
#define MAXIMUM_TURN_TIME 10000

//TODO: Integrate Brian's work here

/** This class holds the RP Manager.*/
class RPManager: public Manager, public LoopThread
  {
  public:
  enum ActionResult {complete_success, complete_fail, incomplete};

  private:
  RPManager(RPManager const&b);
  RPManager &operator=(RPManager const &b);

  protected:
  Mutex mutex;

  /** Very simple representation for the scheduler:
      - Each Object send a list<Action>.
      - RP Engine run at most one action per turn.
      - If Action is not complete it is not pop_front of the list.
      - If the list is empty, it is removed.
      - Only one entry per ObjectID.
   **/
  typedef std::pair<Object::ID,std::list<Action> > Entry;
  std::list<Entry> m_scheduler[2];

  Uint8 m_scheduler_turn;

  /** A List of actions that the RPManager send to clients. */
  std::list<Entry> m_outGoingActions;

  /** Turns executed until now. */
  Uint32 m_turns;

  /** Delay between turns in ms */
  Uint32 m_turntime;

  WorldManager *worldMan;

  virtual ActionResult Execute(Action const& action)=0;  

  public:   
  RPManager(WorldManager *worldMan);
  virtual ~RPManager();

  /** This add an action to the RP for being processed. */
  void Add(std::list<Action> const& action);
  std::list<Action> Get(Object::ID const& id);

  /** This method get the perception of the object */
  std::list<Object> GetPerception(Object const& object) const;
  /** This method get the perception of the object */
  std::list<Object> GetPerception(Object::ID const& id) const;

  Uint32 GetTurnTime() { return m_turntime; }  
  Uint32 SetTurnTime(Uint32 turntime);
  Uint32 GetTurn() { return m_turns; };

  virtual void loop();
  };

#endif
