/* $Id: ServerTab.c,v 1.5 2002/04/17 12:25:49 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2002 by Brian Rowe
    email                : briprowe@yahoo.com
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// You must include Python.h before any standard headers are included
// (see http://www.python.org/doc/current/api/includes.html) 
// -waba
#include <Python.h>

// This is C, so cassert isn't in the search path.
// And since cassert simply includes assert.h... -waba
#include <assert.h>

#ifdef MACOSX
  char* environ;
#endif

static PyObject *ErrorObject;

/** Define the return values accepted by the server. */
static PyObject *complete_success;
static PyObject *complete_fail;
static PyObject *incomplete;

/** These are function pointers into the server.  They will allow scripts
    to execute actions within the server. */
static PyObject *(*GetObject)(PyObject *args);
static PyObject *(*Add)(PyObject *args);
static void      (*Del)(PyObject* args);
static PyObject *(*isInRange)(PyObject* args);
static PyObject *(*GetValidID)(void);

/** Function sets the value of the function pointers using data passed in by
    the server. */
static PyObject*
Tab_SetPointers(PyObject *self, PyObject *args)
{
  assert(PyTuple_Check(args));

  /*
  PyObject *ptrs[PyList_Size(args)];

  int i;
  for( i = 0; i < PyList_Size(args); i++ )
    {
      ptrs[i] = PyList_GetItem(args, i);
      } */

  GetObject = PyCObject_AsVoidPtr(PyTuple_GetItem(args, 0));
  Add = PyCObject_AsVoidPtr(PyTuple_GetItem(args, 1));
  Del = PyCObject_AsVoidPtr(PyTuple_GetItem(args, 2));
  isInRange = PyCObject_AsVoidPtr(PyTuple_GetItem(args, 3));
  GetValidID = PyCObject_AsVoidPtr(PyTuple_GetItem(args, 4));

  return(PyTuple_New(0));
}

/** This calls GetObject on the server */
static PyObject*
Tab_GetObject(PyObject* self, PyObject* args)
{
  return(GetObject(args));
}

/** This calls Add on the server */
static PyObject*
Tab_Add(PyObject *self, PyObject *args)
{
  return(Add(args));
}

/** This calls Del on the server */
static PyObject*
Tab_Del(PyObject *self, PyObject *args)
{
  Del(args);
  return(PyTuple_New(0));
}

/** This calls isInRange on the server */
static PyObject*
Tab_isInRange(PyObject *self, PyObject* args)
{
  return(isInRange(args));
}

/** This calls GetValidID on the server */
static PyObject*
Tab_GetValidID(PyObject* self, PyObject* args)
{
  return(GetValidID());
}

static struct PyMethodDef ServerTab_methods[] =
  {
    { "GetObject", Tab_GetObject, METH_VARARGS },
    { "Add", Tab_Add, METH_VARARGS },
    { "Del", Tab_Del, METH_VARARGS },
    { "isInRange", Tab_isInRange, METH_VARARGS },
    { "GetValidID", Tab_GetValidID, METH_VARARGS },
    { "SetPointers", Tab_SetPointers, METH_VARARGS },
    { NULL, NULL }
  };

void
initServerTab()
{
  PyObject *m, *d;
  
  m = Py_InitModule("ServerTab", ServerTab_methods);

  d = PyModule_GetDict(m);
  ErrorObject = Py_BuildValue("s", "ServerTab.error");
  complete_success = PyInt_FromLong(1);
  complete_fail = PyInt_FromLong(2);
  incomplete = PyInt_FromLong(3);
  PyDict_SetItemString(d, "error", ErrorObject);
  PyDict_SetItemString(d, "complete_success", complete_success);
  PyDict_SetItemString(d, "complete_fail", complete_fail);
  PyDict_SetItemString(d, "incomplete", incomplete);
  
  if ( PyErr_Occurred() )
    Py_FatalError("Failed initializing ServerTab");
}
