/* $Id: WorldManager.cpp,v 1.31 2002/04/18 18:25:00 pchitescu Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "5_Rp/WorldManager.h"
#include "Log/Logger.h"
#include "Concurrent/Templock.h"
#include "InitPath.h"

#include <fstream>
#include <cstdio>

WorldManager::WorldManager():
  LoopThread("WorldManager")
  {
  glog_Layer3.Debug ("Creating WorldManager on Server");
  MaxUsedID="0";
  glog_Layer3.Debug ("Creating WorldManager on Server -DONE-");
  }

WorldManager::~WorldManager()
  {
  }

void WorldManager::Load()  
  {
  /* Will throw a exception on error. */
  std::string classfile = g_sharedir+"ObjectClass.arianne";
  try
    {
    m_objectman.Load(classfile);
    }
  catch(ObjectClassManager::ObjectClassException &e)
    {
    glog_App.Debug(e.what(),Log::fatal_error);
    throw;
    }

  std::ifstream in;
  std::string filename = g_sharedir+"WorldFile.arianne";
  in.open(filename.c_str());
  if(!in)
    {
    glog_App.Debug("WorldManager::Load(): failed to open world-file",Log::fatal_error);
    throw NoWorldFileException(__FILE_AND_LINE__);
    }

  std::cerr << "WorldManager::Load(): loading world from '" << filename << "'\n";
  TempLock tmp (mutex);
  world.Load(in,MaxUsedID);
  }

void WorldManager::Save()
  {
  std::ofstream out;
  TempLock tmp (mutex);
  out.open((g_sharedir+"WorldFile.arianne").c_str());
  if(!out)
    {
    throw NoWorldFileException(__FILE_AND_LINE__);
    }
  
  world.Save(out);
  }


/* Interface -- */
Object WorldManager::Get(Object::ID const& id) 
  {
  TempLock tmp (mutex);
  return world.Get(id);
  }

void WorldManager::Add(Object const& object)
  {
  TempLock tmp (mutex);
  world.Add(object);
  }

void WorldManager::Del(Object::ID const& id)
  {
  TempLock tmp (mutex);
  world.Del(id);
  }

void WorldManager::Del(Object const& object)
  {
  TempLock tmp (mutex);
  world.Del(object);
  }

Object::ID WorldManager::GetValidID()
  {
  TempLock tmp (mutex);
  Uint32 validID=atoi(MaxUsedID.c_str())+1;

  //TODO: Convert the num to a string.
  char buffer[128];
  sprintf(buffer,"%i",validID);
  MaxUsedID=buffer;

  return MaxUsedID;
  }


/** Returns objects in the given cylinder. */
std::list<Object> WorldManager::isInRange(Object::ID const& Center,Uint32 Distance)
  {
  TempLock tmp (mutex);
  return world.isInRange(Center,Distance);
  }

std::list<Object> WorldManager::isInRange(Object const& object,Uint32 Distance)
  {
  TempLock tmp (mutex);
  return isInRange(object.GetAttrib("position"),Distance);
  }


void WorldManager::loop()
  {
  Command *comm=Commands.Get();

  try
    {
    switch(comm->GetType())
      {
	  case POISON_COMMAND:
	    /* We finish this manager */
		delete comm;
		comm=0;

	    Stop();
		return;        
		break;
      default:
        throw ManagerUnCommandException(__FILE_AND_LINE__, comm->GetType());
      }
    }
  catch(Manager::ManagerUnCommandException &e)
    {
    glog_Layer5.Debug(__FILE_AND_LINE__,e.what());

    delete comm;
    comm=0;
    }
  }

void WorldManager::CreateRandomPlayer(std::string type,std::string name, Object::ID id)
  {
  TempLock tmplock (mutex);
  Object tmp=m_objectman.Create(type);

  tmp.PutAttrib("id",id);
  tmp.PutAttrib("type",type);

  //TODO The Z value should be correct. 
  char buffer[128];
  sprintf(buffer,"%i %i %i",rand()%150,rand()%100,0);
  tmp.PutAttrib("position",buffer);
  sprintf(buffer,"%i %i %i",0,rand()%256,0);
  tmp.PutAttrib("orientation",buffer);
  sprintf(buffer,"%i %i",50,50);
  tmp.PutAttrib("volume",buffer);

  /* We set the name of the player */
  tmp.PutAttrib("name",name);

  Add(tmp);
  }

