/* $Id: Arianneserver.cpp,v 1.28 2002/04/19 12:41:34 pchitescu Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Arianneserver.h"

Semaphore gWaitPointSemaphore(0);
bool gShutdownRequested(false);

ArianneServer::ArianneServer():
  MesMan(&NetMan),
  Comm2Mes(&MesMan),
  Mes2Comm(&CommMan,&MesMan),
  AdminMan(&PlayerConMan),
  PlayerConMan(&WorldMan),
  ActionMan(&PlayerConMan,&RPMan),
  PerceptionMan(&PlayerConMan,&RPMan),
  StatsMan(),
  WorldMan(),
  RPMan(&WorldMan),
  commandLine(&PlayerConMan,&WorldMan,&RPMan)
  {
  WorldMan.Load();
  }

ArianneServer::~ArianneServer()
  {
  WorldMan.Save();
  }

void ArianneServer::Init()
  {
  glog_App.Debug("Telling Network Manager about Command Manager");
  NetMan.Attach(&CommMan);

  glog_App.Debug("Telling Message Manager about Command Manager");
  MesMan.Attach(&CommMan);

  glog_App.Debug("Telling Command Manager about Command Manager");
  CommMan.Attach(&CommMan);

  glog_App.Debug("Telling Command Line Parser about Command Manager");
  commandLine.Attach(&CommMan);

  glog_App.Debug("Subscribing Admin Server Manager to Message Admin commands");
  CommMan.Subscribe(&AdminMan,MESSAGE_M2C_ADMIN_COMMAND);

  glog_App.Debug("Subscribing Admin Server Manager to New Connection commands");
  CommMan.Subscribe(&AdminMan,NEWCONNECTION_COMMAND);

  glog_App.Debug("Subscribing Admin Server Manager to Close Connection commands");
  CommMan.Subscribe(&AdminMan,CLOSECONNECTION_COMMAND);
  glog_App.Debug("Subscribing Action Server Manager to Message Action commands");
  CommMan.Subscribe(&ActionMan,MESSAGE_M2C_ACTION_COMMAND);
  glog_App.Debug("Subscribing Perception Server Manager to Perception time commands");
  CommMan.Subscribe(&PerceptionMan,MESSAGE_M2C_PERCEPTION_COMMAND);

  glog_App.Debug("Subscribing Command2Message Bridge to Message Admin commands");
  CommMan.Subscribe(&Comm2Mes,MESSAGE_C2M_ADMIN_COMMAND);
  glog_App.Debug("Subscribing Command2Message Bridge to Message Action commands");
  CommMan.Subscribe(&Comm2Mes,MESSAGE_C2M_ACTION_COMMAND);
  glog_App.Debug("Subscribing Command2Message Bridge to Message Perception commands");
  CommMan.Subscribe(&Comm2Mes,MESSAGE_C2M_PERCEPTION_COMMAND);

  glog_App.Debug("Subscribing Admin Server Manager to Poison commands");
  CommMan.Subscribe(&AdminMan,POISON_COMMAND);
  glog_App.Debug("Subscribing Stats Manager to Poison commands");
  CommMan.Subscribe(&StatsMan,POISON_COMMAND);
  glog_App.Debug("Subscribing Action Server Manager to Poison commands");
  CommMan.Subscribe(&ActionMan,POISON_COMMAND);
  glog_App.Debug("Subscribing Perception Server Manager to Poison commands");
  CommMan.Subscribe(&PerceptionMan,POISON_COMMAND);
  glog_App.Debug("Subscribing Command2Message Bridge to Poison commands");
  CommMan.Subscribe(&Comm2Mes,POISON_COMMAND);
  glog_App.Debug("Subscribing Message2Command Bridge to Poison commands");
  CommMan.Subscribe(&Mes2Comm,POISON_COMMAND);
  glog_App.Debug("Subscribing RP Manager to Poison commands");
  CommMan.Subscribe(&RPMan,POISON_COMMAND);
  glog_App.Debug("Subscribing World Manager to Poison commands");
  CommMan.Subscribe(&WorldMan,POISON_COMMAND);



  glog_App.Debug("Subscribing Stats Manager to  Layer0 Statistics commands");
  CommMan.Subscribe(&StatsMan,STAT_L0_COMMAND);
  glog_App.Debug("Subscribing Stats Manager to  Layer1 Statistics commands");
  CommMan.Subscribe(&StatsMan,STAT_L1_COMMAND);
  glog_App.Debug("Subscribing Stats Manager to  Layer2 Statistics commands");
  CommMan.Subscribe(&StatsMan,STAT_L2_COMMAND);
  glog_App.Debug("Subscribing Stats Manager to  Layer3 Statistics commands");
  CommMan.Subscribe(&StatsMan,STAT_L3_COMMAND);
  glog_App.Debug("Subscribing Stats Manager to  Layer4 Statistics commands");
  CommMan.Subscribe(&StatsMan,STAT_L4_COMMAND);

  glog_App.Debug("Creating Server connection");
  NetMan.Open(0,3124);

  glog_App.Debug("Launching Server NetworkManager");
  NetMan.Start();

  glog_App.Debug("Launching Server MessageManager");
  MesMan.Start();

  glog_App.Debug("Launching CommandManager");
  CommMan.Start();

  glog_App.Debug("Launching Command to Message bridge");
  Comm2Mes.Start();

  glog_App.Debug("Launching Message to Command bridge");
  Mes2Comm.Start();

  glog_App.Debug("Launching Admin Server Manager");
  AdminMan.Start();

  glog_App.Debug("Launching Action Server Manager");
  ActionMan.Start();

  glog_App.Debug("Launching Perception Server Manager");
  PerceptionMan.Start();

  glog_App.Debug("Launching Stats Manager");
  StatsMan.Start();

  glog_App.Debug("Launching World Manager");
  WorldMan.Start();

  glog_App.Debug("Launching RP Manager");
  RPMan.Start();
  }

void ArianneServer::Run(bool ConsoleEnabled)
  {
  if(!ConsoleEnabled)
    {
    gWaitPointSemaphore.Wait();
    }
  else 
    {
    commandLine.Start();
    }

  glog_App.Debug("Disconnecting players");
  /* We close all the open connection sending them a Logout message. */
  for(PlayerConnectionManager::iterator i=PlayerConMan.begin();i!=PlayerConMan.end();++i)
    {
    MessageCommand *comm = new MessageCommand (MESSAGE_C2M_ADMIN_COMMAND);
    Message_S2C_LOGOUT *ReplyMes = new Message_S2C_LOGOUT;
    ReplyMes->SetByeString("Server is shuting down");
    ReplyMes->SetID ((*i).ConnectionID);  
    comm->SetMessage (ReplyMes);
    CommMan.AddCommand (comm);
    }

  SDL_Delay(1000);
  }

void ArianneServer::Close()
  {
  glog_App.Debug("Stopping Server NetworkManager");
  NetMan.Stop();
  SDL_Delay(1000);

  glog_App.Debug("Closing Server connection");
  NetMan.Close();

  NetMan.SendPoisonToUpperLayer();

  glog_App.Debug("Stopping Server MessageManager");
  glog_App.Debug("Stopping Message to Command bridge");
  glog_App.Debug("Stopping Command to Message bridge");

  NetMan.Wait();
  MesMan.Wait();
  Mes2Comm.Wait();
  Comm2Mes.Wait();
  SDL_Delay(1000);

  glog_App.Debug("Stopping Perception Server Manager");
  glog_App.Debug("Stopping Action Server Manager");
  glog_App.Debug("Stopping Admin Server Manager");
  glog_App.Debug("Stopping Stats Server Manager");
  glog_App.Debug("Stopping World Server Manager");
  glog_App.Debug("Stopping RP Manager");

  PerceptionMan.Wait();
  ActionMan.Wait();
  AdminMan.Wait();
  StatsMan.Wait();
  WorldMan.Wait();
  RPMan.Wait();

  glog_App.Debug("Stopping CommandManager");
  CommMan.Wait();
  }



