/* $Id: CommandLineParser.cpp,v 1.13 2002/04/26 18:15:58 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "CommandLineParser.h"
#include "Conversion/StringTool.h"

#include "2_Command/CommandManager.h"

#include "3_Manager/ManagerCoordinator.h"

#include "1_Message/Message.h"
#include "1_Message/MessagesTypes.h"

extern bool gShutdownRequested;

using namespace std;

CommandLineParser::CommandLineParser(PlayerConnectionManager *playerConMan,WorldManager *worldMan,RPManager *rpMan):Manager()
  {
  this->playerConMan=playerConMan;
  this->worldMan=worldMan;
  this->rpMan=rpMan;
  }

CommandLineParser::~CommandLineParser()
  {
  }

#ifdef WIN32
std::istream& a_readline(std::istream& in, std::string & stri)
  {
  char Buffer[256];
  int size=0;

  cout << ": ";
  in.getline(Buffer,255,'\n');

  for(int i=0;i<256;++i)
    {  
    ++size;  
    if(Buffer[i]=='\r') 
      {
      Buffer[i]=0;
      break;
      }
    if(Buffer[i]=='\n')
      {
      Buffer[i]=0;
      break;
      }
    if(Buffer[i]==0)
      {
      break;
      }
    }
  
  stri=Buffer;
  return in;
  }
#endif

void CommandLineParser::Start()
  {
  cout << "Arianne ALPHA 2, Copyright (C) 2002 Miguel Angel Blanch Lardin" << std::endl;
  cout << "Arianne comes with ABSOLUTELY NO WARRANTY; for details" << std::endl;
  cout << "type `show w'.  This is free software, and you are welcome" << std::endl;
  cout << "to redistribute it under certain conditions; type `show c'" << std::endl; 
  cout << "for details." << std::endl;


  while(!gShutdownRequested)
    {
    string commandline;

    #ifndef WIN32 
    std::getline(std::cin, commandline);
    #else
    a_readline(std::cin, commandline);
    #endif

    if(cin.eof())
      {
      cout << "Input stream has reach EOF" << endl;
      return;
      }

    vector<string> pieces=Split(commandline);

    /* Now we test the first component. */
    if(pieces[0]=="list")
      {
      cout << "ConnectionID\tPlayerID" << std::endl;

      /** We grant EXCLUSIVE MUTUAL ACCESS here */
      TempLock tmp(gManagerCoordinatorMutex);
      for(PlayerConnectionManager::iterator i=playerConMan->begin();i!=playerConMan->end();++i)
        {
        cout << (*i).ConnectionID << "\t\t" << (*i).PlayerID << std::endl;
        }
      continue;
      }

    if(pieces[0]=="disconnect")
      {
      if(pieces.size()==1)
        {
        cout << "Syntax: disconnect <ConnectionID>" << std::endl;
        continue;
        }      

      if (playerConMan->find(atoi(pieces[1].c_str()))!=playerConMan->end())
        {
        /* Very tricky, we made serve thinks that player send a Logout message. */
        cout << "Disconnecting player using connection " << atoi(pieces[1].c_str()) << std::endl;
        MessageCommand *comm = new MessageCommand (MESSAGE_M2C_ADMIN_COMMAND);
        Message_C2S_LOGOUT *ReplyMes = new Message_C2S_LOGOUT;
        ReplyMes->SetID (atoi(pieces[1].c_str()));  
   	    comm->SetMessage (ReplyMes);
        CommManager->AddCommand (comm);
        }
      else
	    {
 	    cout << "Connection " << atoi(pieces[1].c_str()) << " is not currently in use" << std::endl;
	    }

      continue;
      }

    if(pieces[0]=="create")
      {
      if(pieces.size()==1)
        {
        cout << "Syntax: create <type>" << std::endl;
        continue;
        }      

      try
        {
        cout << "Creating new object of type "<< pieces[1] << std::endl;
        Object::ID id=worldMan->GetValidID();
        worldMan->CreateRandomPlayer(pieces[1],pieces[1],id);
        cout << "Object of type "<< pieces[1] << " created"<< std::endl;
        }
      catch(...)
        {
        cout << "An error happen while creating object" << std::endl;
        }
        

      /* We now should keep adding the rest of attributes requiered by the object. */

      continue;
      }

    if(pieces[0]=="turntime")
      {
      if(pieces.size()==1)
        {
        cout << "Turntime is " << rpMan->GetTurnTime() << " ms" << std::endl;
        continue;
        }      

      try
        {
        cout << "New turntime is " << rpMan->SetTurnTime(strtoul(pieces[1].data(),0,0))
	     << " ms" << std::endl;
        }
      catch(...)
        {
        cout << "An error happened while setting turn time" << std::endl;
        }
        
      continue;
      }

    if(pieces[0]=="help")
      {
      cout << "Command List For Server : " << std::endl; 
      cout << " * create (Object) - Create a new object of a specified type." << std::endl; 
      cout << "   + Object - The name of the object to create." << std::endl; 
      cout << " * disconnect (ID) - Disconnect a client connection from server" << std::endl; 
      cout << "   + ID - Specify the connection number of the client to be disconnected." << std::endl; 
      cout << " * list - Create a list of all clients connected and their connection numbers." << std::endl; 
      cout << " * turntime [time] - Shows or sets the perception and turn time." << std::endl; 
      cout << "   + time - New turn time in milliseconds." << std::endl; 
      cout << " * show (c or w) - Show the copyright of this software server." << std::endl; 
      cout << "   + c - Show terms and conditions for GNU GENERAL PUBLIC LICENSE." << std::endl; 
      cout << "   + w - Show GNU GENERAL PUBLIC LICENSE Preamble." << std::endl; 
      cout << " * exit - Log off all clients and terminate server." << std::endl; 
      cout << " * help - This help screen." << std::endl; 

      continue;
      }

    if(pieces[0]=="exit")
      {
      gShutdownRequested=true;
      continue;
      }

    if(pieces[0]=="show" && (pieces.size()>1) && pieces[1]=="w")
      {
      cout <<"		    GNU GENERAL PUBLIC LICENSE" << std::endl;
      cout <<"		       Version 2, June 1991" << std::endl;
      cout << std::endl;
      cout <<" Copyright (C) 1989, 1991 Free Software Foundation, Inc." << std::endl;
      cout <<"                       59 Temple Place, Suite 330, Boston, MA  02111-1307  USA" << std::endl;
      cout <<" Everyone is permitted to copy and distribute verbatim copies" << std::endl;
      cout <<" of this license document, but changing it is not allowed." << std::endl;
      cout << std::endl;
      cout <<"			    Preamble" << std::endl;
      cout << std::endl;
      cout <<"  The licenses for most software are designed to take away your" << std::endl;
      cout <<"freedom to share and change it.  By contrast, the GNU General Public" << std::endl;
      cout <<"License is intended to guarantee your freedom to share and change free" << std::endl;
      cout <<"software--to make sure the software is free for all its users.  This" << std::endl;
      cout <<"General Public License applies to most of the Free Software" << std::endl;
      cout <<"Foundation's software and to any other program whose authors commit to" << std::endl;
      cout <<"using it.  (Some other Free Software Foundation software is covered by" << std::endl;
      cout <<"the GNU Library General Public License instead.)  You can apply it to" << std::endl;
      cout <<"your programs, too." << std::endl;
      cout << std::endl;
      cout <<"  When we speak of free software, we are referring to freedom, not" << std::endl;
      cout <<"price.  Our General Public Licenses are designed to make sure that you" << std::endl;
      cout <<"have the freedom to distribute copies of free software (and charge for" << std::endl;
      cout <<"this service if you wish), that you receive source code or can get it" << std::endl;
      cout <<"if you want it, that you can change the software or use pieces of it" << std::endl;
      cout <<"in new free programs; and that you know you can do these things." << std::endl;
      cout << std::endl;
      cout <<"  To protect your rights, we need to make restrictions that forbid" << std::endl;
      cout <<"anyone to deny you these rights or to ask you to surrender the rights." << std::endl;
      cout <<"These restrictions translate to certain responsibilities for you if you" << std::endl;
      cout <<"distribute copies of the software, or if you modify it." << std::endl;
      cout << std::endl;
      cout <<"  For example, if you distribute copies of such a program, whether" << std::endl;
      cout <<"gratis or for a fee, you must give the recipients all the rights that" << std::endl;
      cout <<"you have.  You must make sure that they, too, receive or can get the" << std::endl;
      cout <<"source code.  And you must show them these terms so they know their" << std::endl;
      cout <<"rights." << std::endl;
      cout << std::endl;
      cout <<"  We protect your rights with two steps: (1) copyright the software, and" << std::endl;
      cout <<"(2) offer you this license which gives you legal permission to copy," << std::endl;
      cout <<"distribute and/or modify the software." << std::endl;
      cout << std::endl;
      cout <<"  Also, for each author's protection and ours, we want to make certain" << std::endl;
      cout <<"that everyone understands that there is no warranty for this free" << std::endl;
      cout <<"software.  If the software is modified by someone else and passed on, we" << std::endl;
      cout <<"want its recipients to know that what they have is not the original, so" << std::endl;
      cout <<"that any problems introduced by others will not reflect on the original" << std::endl;
      cout <<"authors' reputations." << std::endl;
      cout << std::endl;
      cout <<"  Finally, any free program is threatened constantly by software" << std::endl;
      cout <<"patents.  We wish to avoid the danger that redistributors of a free" << std::endl;
      cout <<"program will individually obtain patent licenses, in effect making the" << std::endl;
      cout <<"program proprietary.  To prevent this, we have made it clear that any" << std::endl;
      cout <<"patent must be licensed for everyone's free use or not licensed at all." << std::endl;
      cout << std::endl;
      cout <<"  The precise terms and conditions for copying, distribution and" << std::endl;
      cout <<"modification follow." << std::endl;
      continue;
      }

    if(pieces[0]=="show" && (pieces.size()>1) && pieces[1]=="c")
      {
      cout <<"		    GNU GENERAL PUBLIC LICENSE" << std::endl;
      cout <<"   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION" << std::endl;
      cout << std::endl;
      cout <<"  0. This License applies to any program or other work which contains" << std::endl;
      cout <<"a notice placed by the copyright holder saying it may be distributed" << std::endl;
      cout <<"under the terms of this General Public License.  The \"Program\", below," << std::endl;
      cout <<"refers to any such program or work, and a \"work based on the Program\"" << std::endl;
      cout <<"means either the Program or any derivative work under copyright law:" << std::endl;
      cout <<"that is to say, a work containing the Program or a portion of it," << std::endl;
      cout <<"either verbatim or with modifications and/or translated into another" << std::endl;
      cout <<"language.  (Hereinafter, translation is included without limitation in" << std::endl;
      cout <<"the term \"modification\".)  Each licensee is addressed as \"you\"." << std::endl;
      cout << std::endl;
      cout <<"Activities other than copying, distribution and modification are not" << std::endl;
      cout <<"covered by this License; they are outside its scope.  The act of" << std::endl;
      cout <<"running the Program is not restricted, and the output from the Program" << std::endl;
      cout <<"is covered only if its contents constitute a work based on the" << std::endl;
      cout <<"Program (independent of having been made by running the Program)." << std::endl;
      cout <<"Whether that is true depends on what the Program does." << std::endl;
      cout << std::endl;
      cout <<"  1. You may copy and distribute verbatim copies of the Program's" << std::endl;
      cout <<"source code as you receive it, in any medium, provided that you" << std::endl;
      cout <<"conspicuously and appropriately publish on each copy an appropriate" << std::endl;
      cout <<"copyright notice and disclaimer of warranty; keep intact all the" << std::endl;
      cout <<"notices that refer to this License and to the absence of any warranty;" << std::endl;
      cout <<"and give any other recipients of the Program a copy of this License" << std::endl;
      cout <<"along with the Program." << std::endl;
      cout << std::endl;
      cout <<"You may charge a fee for the physical act of transferring a copy, and" << std::endl;
      cout <<"you may at your option offer warranty protection in exchange for a fee." << std::endl;
      cout << std::endl;
      cout <<"  2. You may modify your copy or copies of the Program or any portion" << std::endl;
      cout <<"of it, thus forming a work based on the Program, and copy and" << std::endl;
      cout <<"distribute such modifications or work under the terms of Section 1" << std::endl;
      cout <<"above, provided that you also meet all of these conditions:" << std::endl;
      cout << std::endl;
      cout <<"    a) You must cause the modified files to carry prominent notices" << std::endl;
      cout <<"    stating that you changed the files and the date of any change." << std::endl;
      cout << std::endl;
      cout <<"    b) You must cause any work that you distribute or publish, that in" << std::endl;
      cout <<"    whole or in part contains or is derived from the Program or any" << std::endl;
      cout <<"    part thereof, to be licensed as a whole at no charge to all third" << std::endl;
      cout <<"    parties under the terms of this License." << std::endl;
      cout << std::endl;
      cout <<"    c) If the modified program normally reads commands interactively" << std::endl;
      cout <<"    when run, you must cause it, when started running for such" << std::endl;
      cout <<"    interactive use in the most ordinary way, to print or display an" << std::endl;
      cout <<"    announcement including an appropriate copyright notice and a" << std::endl;
      cout <<"    notice that there is no warranty (or else, saying that you provide" << std::endl;
      cout <<"    a warranty) and that users may redistribute the program under" << std::endl;
      cout <<"    these conditions, and telling the user how to view a copy of this" << std::endl;
      cout <<"    License.  (Exception: if the Program itself is interactive but" << std::endl;
      cout <<"    does not normally print such an announcement, your work based on" << std::endl;
      cout <<"    the Program is not required to print an announcement.)" << std::endl;
      cout << std::endl;
      cout <<"These requirements apply to the modified work as a whole.  If" << std::endl;
      cout <<"identifiable sections of that work are not derived from the Program," << std::endl;
      cout <<"and can be reasonably considered independent and separate works in" << std::endl;
      cout <<"themselves, then this License, and its terms, do not apply to those" << std::endl;
      cout <<"sections when you distribute them as separate works.  But when you" << std::endl;
      cout <<"distribute the same sections as part of a whole which is a work based" << std::endl;
      cout <<"on the Program, the distribution of the whole must be on the terms of" << std::endl;
      cout <<"this License, whose permissions for other licensees extend to the" << std::endl;
      cout <<"entire whole, and thus to each and every part regardless of who wrote it." << std::endl;
      cout << std::endl;
      cout <<"Thus, it is not the intent of this section to claim rights or contest" << std::endl;
      cout <<"your rights to work written entirely by you; rather, the intent is to" << std::endl;
      cout <<"exercise the right to control the distribution of derivative or" << std::endl;
      cout <<"collective works based on the Program." << std::endl;
      cout << std::endl;
      cout <<"In addition, mere aggregation of another work not based on the Program" << std::endl;
      cout <<"with the Program (or with a work based on the Program) on a volume of" << std::endl;
      cout <<"a storage or distribution medium does not bring the other work under" << std::endl;
      cout <<"the scope of this License." << std::endl;
      cout << std::endl;
      cout <<"  3. You may copy and distribute the Program (or a work based on it," << std::endl;
      cout <<"under Section 2) in object code or executable form under the terms of" << std::endl;
      cout <<"Sections 1 and 2 above provided that you also do one of the following:" << std::endl;
      cout << std::endl;
      cout <<"    a) Accompany it with the complete corresponding machine-readable" << std::endl;
      cout <<"    source code, which must be distributed under the terms of Sections" << std::endl;
      cout <<"    1 and 2 above on a medium customarily used for software interchange; or," << std::endl;
      cout << std::endl;
      cout <<"    b) Accompany it with a written offer, valid for at least three" << std::endl;
      cout <<"    years, to give any third party, for a charge no more than your" << std::endl;
      cout <<"    cost of physically performing source distribution, a complete" << std::endl;
      cout <<"    machine-readable copy of the corresponding source code, to be" << std::endl;
      cout <<"    distributed under the terms of Sections 1 and 2 above on a medium" << std::endl;
      cout <<"    customarily used for software interchange; or," << std::endl;
      cout << std::endl;
      cout <<"    c) Accompany it with the information you received as to the offer" << std::endl;
      cout <<"    to distribute corresponding source code.  (This alternative is" << std::endl;
      cout <<"    allowed only for noncommercial distribution and only if you" << std::endl;
      cout <<"    received the program in object code or executable form with such" << std::endl;
      cout <<"    an offer, in accord with Subsection b above.)" << std::endl;
      cout << std::endl;
      cout <<"The source code for a work means the preferred form of the work for" << std::endl;
      cout <<"making modifications to it.  For an executable work, complete source" << std::endl;
      cout <<"code means all the source code for all modules it contains, plus any" << std::endl;
      cout <<"associated interface definition files, plus the scripts used to" << std::endl;
      cout <<"control compilation and installation of the executable.  However, as a" << std::endl;
      cout <<"special exception, the source code distributed need not include" << std::endl;
      cout <<"anything that is normally distributed (in either source or binary" << std::endl;
      cout <<"form) with the major components (compiler, kernel, and so on) of the" << std::endl;
      cout <<"operating system on which the executable runs, unless that component" << std::endl;
      cout <<"itself accompanies the executable." << std::endl;
      cout << std::endl;
      cout <<"If distribution of executable or object code is made by offering" << std::endl;
      cout <<"access to copy from a designated place, then offering equivalent" << std::endl;
      cout <<"access to copy the source code from the same place counts as" << std::endl;
      cout <<"distribution of the source code, even though third parties are not" << std::endl;
      cout <<"compelled to copy the source along with the object code." << std::endl;
      cout << std::endl;
      cout <<"  4. You may not copy, modify, sublicense, or distribute the Program" << std::endl;
      cout <<"except as expressly provided under this License.  Any attempt" << std::endl;
      cout <<"otherwise to copy, modify, sublicense or distribute the Program is" << std::endl;
      cout <<"void, and will automatically terminate your rights under this License." << std::endl;
      cout <<"However, parties who have received copies, or rights, from you under" << std::endl;
      cout <<"this License will not have their licenses terminated so long as such" << std::endl;
      cout <<"parties remain in full compliance." << std::endl;
      cout << std::endl;
      cout <<"  5. You are not required to accept this License, since you have not" << std::endl;
      cout <<"signed it.  However, nothing else grants you permission to modify or" << std::endl;
      cout <<"distribute the Program or its derivative works.  These actions are" << std::endl;
      cout <<"prohibited by law if you do not accept this License.  Therefore, by" << std::endl;
      cout <<"modifying or distributing the Program (or any work based on the" << std::endl;
      cout <<"Program), you indicate your acceptance of this License to do so, and" << std::endl;
      cout <<"all its terms and conditions for copying, distributing or modifying" << std::endl;
      cout <<"the Program or works based on it." << std::endl;
      cout << std::endl;
      cout <<"  6. Each time you redistribute the Program (or any work based on the" << std::endl;
      cout <<"Program), the recipient automatically receives a license from the" << std::endl;
      cout <<"original licensor to copy, distribute or modify the Program subject to" << std::endl;
      cout <<"these terms and conditions.  You may not impose any further" << std::endl;
      cout <<"restrictions on the recipients' exercise of the rights granted herein." << std::endl;
      cout <<"You are not responsible for enforcing compliance by third parties to" << std::endl;
      cout <<"this License." << std::endl;
      cout << std::endl;
      cout <<"  7. If, as a consequence of a court judgment or allegation of patent" << std::endl;
      cout <<"infringement or for any other reason (not limited to patent issues)," << std::endl;
      cout <<"conditions are imposed on you (whether by court order, agreement or" << std::endl;
      cout <<"otherwise) that contradict the conditions of this License, they do not" << std::endl;
      cout <<"excuse you from the conditions of this License.  If you cannot" << std::endl;
      cout <<"distribute so as to satisfy simultaneously your obligations under this" << std::endl;
      cout <<"License and any other pertinent obligations, then as a consequence you" << std::endl;
      cout <<"may not distribute the Program at all.  For example, if a patent" << std::endl;
      cout <<"license would not permit royalty-free redistribution of the Program by" << std::endl;
      cout <<"all those who receive copies directly or indirectly through you, then" << std::endl;
      cout <<"the only way you could satisfy both it and this License would be to" << std::endl;
      cout <<"refrain entirely from distribution of the Program." << std::endl;
      cout << std::endl;
      cout <<"If any portion of this section is held invalid or unenforceable under" << std::endl;
      cout <<"any particular circumstance, the balance of the section is intended to" << std::endl;
      cout <<"apply and the section as a whole is intended to apply in other" << std::endl;
      cout <<"circumstances." << std::endl;
      cout << std::endl;
      cout <<"It is not the purpose of this section to induce you to infringe any" << std::endl;
      cout <<"patents or other property right claims or to contest validity of any" << std::endl;
      cout <<"such claims; this section has the sole purpose of protecting the" << std::endl;
      cout <<"integrity of the free software distribution system, which is" << std::endl;
      cout <<"implemented by public license practices.  Many people have made" << std::endl;
      cout <<"generous contributions to the wide range of software distributed" << std::endl;
      cout <<"through that system in reliance on consistent application of that" << std::endl;
      cout <<"system; it is up to the author/donor to decide if he or she is willing" << std::endl;
      cout <<"to distribute software through any other system and a licensee cannot" << std::endl;
      cout <<"impose that choice." << std::endl;
      cout << std::endl;
      cout <<"This section is intended to make thoroughly clear what is believed to" << std::endl;
      cout <<"be a consequence of the rest of this License." << std::endl;
      cout << std::endl;
      cout <<"  8. If the distribution and/or use of the Program is restricted in" << std::endl;
      cout <<"certain countries either by patents or by copyrighted interfaces, the" << std::endl;
      cout <<"original copyright holder who places the Program under this License" << std::endl;
      cout <<"may add an explicit geographical distribution limitation excluding" << std::endl;
      cout <<"those countries, so that distribution is permitted only in or among" << std::endl;
      cout <<"countries not thus excluded.  In such case, this License incorporates" << std::endl;
      cout <<"the limitation as if written in the body of this License." << std::endl;
      cout << std::endl;
      cout <<"  9. The Free Software Foundation may publish revised and/or new versions" << std::endl;
      cout <<"of the General Public License from time to time.  Such new versions will" << std::endl;
      cout <<"be similar in spirit to the present version, but may differ in detail to" << std::endl;
      cout <<"address new problems or concerns." << std::endl;
      cout << std::endl;
      cout <<"Each version is given a distinguishing version number.  If the Program" << std::endl;
      cout <<"specifies a version number of this License which applies to it and \"any" << std::endl;
      cout <<"later version\", you have the option of following the terms and conditions" << std::endl;
      cout <<"either of that version or of any later version published by the Free" << std::endl;
      cout <<"Software Foundation.  If the Program does not specify a version number of" << std::endl;
      cout <<"this License, you may choose any version ever published by the Free Software" << std::endl;
      cout <<"Foundation." << std::endl;
      cout << std::endl;
      cout <<"  10. If you wish to incorporate parts of the Program into other free" << std::endl;
      cout <<"programs whose distribution conditions are different, write to the author" << std::endl;
      cout <<"to ask for permission.  For software which is copyrighted by the Free" << std::endl;
      cout <<"Software Foundation, write to the Free Software Foundation; we sometimes" << std::endl;
      cout <<"make exceptions for this.  Our decision will be guided by the two goals" << std::endl;
      cout <<"of preserving the free status of all derivatives of our free software and" << std::endl;
      cout <<"of promoting the sharing and reuse of software generally." << std::endl;
      cout << std::endl;
      cout <<"			    NO WARRANTY" << std::endl;
      cout << std::endl;
      cout <<"  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY" << std::endl;
      cout <<"FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN" << std::endl;
      cout <<"OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES" << std::endl;
      cout <<"PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED" << std::endl;
      cout <<"OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF" << std::endl;
      cout <<"MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS" << std::endl;
      cout <<"TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE" << std::endl;
      cout <<"PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING," << std::endl;
      cout <<"REPAIR OR CORRECTION." << std::endl;
      cout << std::endl;
      cout <<"  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING" << std::endl;
      cout <<"WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR" << std::endl;
      cout <<"REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES," << std::endl;
      cout <<"INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING" << std::endl;
      cout <<"OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED" << std::endl;
      cout <<"TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY" << std::endl;
      cout <<"YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER" << std::endl;
      cout <<"PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE" << std::endl;
      cout <<"POSSIBILITY OF SUCH DAMAGES." << std::endl;
      continue;
      }

    cerr << "Command not found. Type HELP" << std::endl;
    }
  }
