/* $Id: DatabaseHandler.cpp,v 1.26 2002/04/15 12:32:13 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <fstream>
#include <sstream>


#include "DataBase/DatabaseHandler.h"
#include "Log/Logger.h"
#include "Crypto/Crypto.h"
#include "4_Object/Object.h"

using namespace std;


MYSQLConnection::MYSQLConnection(std::string const& db,
                                 std::string const& host,
                                 std::string const& user,
                                 std::string const& passwd)
{
  if ((mysql = mysql_init(NULL)))
  {
    if (!mysql_connect(mysql, host.c_str(), user.c_str(), passwd.c_str()))
    {
      throw MYSQLConnectionException(
              "MYSQLConnection: Could not connect to server : ",
              mysql_error(mysql));
    }
    else
    {
      glog_Common.Debug("MYSQLConnection: Succeeded to connect to server");
    }

    if( mysql_select_db(mysql, db.c_str()))
    {
      throw MYSQLConnectionException(
              "MYSQLConnection: mysql_select_db() failed : ",
              mysql_error(mysql));
    }
  }
  else
  {
    throw MYSQLConnectionException("MYSQLConnection: mysql_init() failed : ",
                                   mysql_error(mysql));
  }
}


MYSQLConnection::~MYSQLConnection()
{
  mysql_close(mysql);
}


MYSQL* MYSQLConnection::getConnection()
{
  return mysql;
}


// Class DataBase

DataBase::DataBase(int nbcon, std::string const& db, std::string const& host,
                   std::string const& user, std::string const& passwd)
{
  MYSQLConnection *con;

  int i;
  for (i = 0; i < nbcon; i++)
  {
    con = new MYSQLConnection(db, host, user, passwd);

    // If we can't create the connection we notify it.
    if (!con->getConnection())
    {
      con = 0;
      throw DatabaseException("Couldn't make MYSQLConnection object");
    }
    else
    {
      ConQueue.Add(con);
    }
  }
}


bool DataBase::getLogin(std::string const &login, LoginInfo &l)
{
  GetConnection gcon(this);

  Uint32 id;
  Object::ID objectid;
  std::string datetime;
  std::string ip_addr;
  Uint16 nfail;
  bool password;

  int res;
  MYSQL_RES* result = NULL;
  MYSQL_ROW row;
  MYSQL* mysql = gcon.con->getConnection();

  std::ostringstream sout;
  bool ret_value = false;

  // SELECT id,objectid,login_procedure FROM LOGINS WHERE login = "<login>";
  sout << "SELECT "
       << "id,objectid,login_procedure,last_datetime,last_ip,failures "
       << "FROM LOGINS "
       << "WHERE login = \""
       << login << "\"";

  res = mysql_query(mysql, sout.str().c_str());
  if (res != 0)
  {
    throw DatabaseException("DataBase::getLogin - Query Error #", res);
    return false;
  }

  // Receive the result
  result = mysql_store_result(mysql);
  if (!result)
  {
    throw DatabaseException(
            "DataBase::getLogin - Could not obtain Query Result");
    return false;
  }

  // Process the rows.
  row = mysql_fetch_row(result);
  if (!row)
  {
    throw QueryException("No user with login name " + login);
    return false;
  }

  glog_Common.Debug("Got LOGIN info for user " + login);

  id = atoi(row[0]);
  // We need to consider the case on which that players has a NULL ObjectID.
  objectid = ((row[1] == 0) ? Object::Invalid : row[1]);
  password = ((std::string)row[2] == std::string("password"));
  datetime = ((row[3] == 0) ? "unknown" : (std::string)row[3]);
  ip_addr = ((row[4] == 0) ? "unknown" : (std::string)row[4]);
  nfail = ((row[5] == 0) ? 0 : atoi(row[5]));

  //We free the above query.
  mysql_free_result(result);

  l.setLogin(login);
  l.setID(id);
  l.setObjectID(objectid);
  l.setLastDateTime(datetime);
  l.setLastIP(ip_addr);
  l.setFailures(nfail);

  if (password)
  {
    // SELECT hash FROM PASSWORDS WHERE id = <id>;
    sout.str("");
    sout << "SELECT hash FROM PASSWORDS WHERE id = " << id;

    res = mysql_query( mysql, sout.str().c_str());
    if (res != 0)
    {
      throw DatabaseException("DataBase::getLogin - Query Error (password) #",
                              res);
      return false;
    }

    result = mysql_store_result(mysql);
    if (!result)
    {
      throw DatabaseException(
              "DataBase::getLogin - Could not obtain Query Result (password)");
      return false;
    }

    row = mysql_fetch_row(result);
    if (!row)
    {
      throw QueryException("Could not get password info for user " + login);
      return false;
    }

    glog_Common.Debug("Got PASSWORD info for user " + login);
    l.setHash((std::string) row[0]);
    ret_value = true;

    mysql_free_result( result );
  }
  else
#ifdef HAVE_CRYPTO
  {
    RSAKey *key;

    sout.str("");
    sout << "SELECT n,e FROM RSAKEYS WHERE id = \"" << id << "\"";
    res = mysql_query(mysql, sout.str().c_str());
    if (res != 0)
    {
      throw DatabaseException("DataBase::getLogin - Query Error (RSAKey) #",
                              res);
      return false;
    }
    result = mysql_store_result(mysql);
    if (!result)
    {
      throw DatabaseException(
               "DataBase::getLogin - Could not obtain Query Result (RSAKey)");
      return false;
    }
    row = mysql_fetch_row(result);
    if (!row)
    {
      throw QueryException("Could not get RSA key for user " + login);
      return false;
    }

    key = new RSAKey((std::string)row[0], (std::string)row[1]);
    l.setKey(key);
    ret_value = true;

    mysql_free_result(result);
  }
#else
  {
    throw DatabaseException(
        "Error: The login procedure is RSA while there is no crypto support");
    ret_value = false;
  }
#endif

  return ret_value;
}


bool DataBase::setLogin(LoginInfo const &l)
  {
  GetConnection gcon(this);

  MYSQL* mysql;
  MYSQL_RES* result = NULL;
  MYSQL_ROW row;

  int res, id = 0;
  std::ostringstream sout;

  if (!l.isValid())
  {
    return false;
  }

  mysql = gcon.con->getConnection();

  //SELECT id FROM LOGINS WHERE login = <login>
  sout << "SELECT id FROM LOGINS WHERE login = \"" << l.getLogin() << "\"";

  res = mysql_query(mysql, sout.str().c_str());
  if (res != 0)
  {
    throw DatabaseException("DataBase::setLogin - Query Error #", res);
    return false;
  }

  result = mysql_store_result(mysql);
  if (!result)
  {
    throw DatabaseException(
            "DataBase::setLogin - Could not obtain Query Result");
    return false;
  }

  row = mysql_fetch_row(result);
  id = (row ? atoi(row[0]) : 0);

  mysql_free_result(result);

  res = mysql_query(mysql, "BEGIN");
  if (res != 0)
  {
    throw DatabaseException("DataBase::setLogin - Query Error #", res);
    return false;
  }

  if (id != 0)
  {
    sout.str("");
    sout << "REPLACE INTO LOGINS "
	 << "(id,objectid,login,login_procedure,last_datetime,last_ip,failures) "
         << "VALUES("
         << id << "," << l.getObjectID() << ","
         << "\"" << l.getLogin() << "\",";

    if(l.getLoginType() == LoginInfo::pass )
    {
      sout << "\"password\",";
    }
    else
    {
      sout << "\"RSA\",";
    }
    sout << "\"" << l.getLastDateTime() << "\",\"" << l.getLastIP() << "\","
	 << l.getFailures() << ")";
  }
  else
  {
    sout.str("");
    sout << "INSERT INTO LOGINS "
         << "(objectid,login,login_procedure,last_datetime,last_ip,failures) "
         << "VALUES(";

    Object::ID result = l.getObjectID();
    if (result == "")
    {
      result = "NULL";
    }

    sout << result << ",\"" << l.getLogin() << "\",";
    if (l.getLoginType() == LoginInfo::pass)
    {
      sout << "\"password\",";
    }
    else
    {
      sout << "\"RSA\",";
    }
    sout << "\"" << l.getLastDateTime() << "\",\"" << l.getLastIP() << "\","
	 << l.getFailures() << ")";
  }

  res = mysql_query(mysql, sout.str().c_str());
  if (res != 0)
  {
    throw DatabaseException(
            "DatabaseHandler::setLogin REPLACE INTO LOGINS... failed - #",
            res);
    return false;
  }

  // get the id of the newly created row, if id is 0 (i.e., not specified yet)
  if (id == 0)
  {
    id = mysql_insert_id(mysql);
  }

#ifdef HAVE_CRYPTO
  if (l.getLoginType() == LoginInfo::pass)
  {
#endif
    sout.str("");
    sout << "REPLACE INTO PASSWORDS (id, hash) VALUES(" << id << ",\""
         << l.getHash() << "\")";
    res = mysql_query(mysql, sout.str().c_str());
#ifdef HAVE_CRYPTO
  }
  else
  {
    std::string n;
    std::string e;
    RSAKey *key;
    key = l.getKey();
    key->GetPublicKey(n,e);
    //REPLACE INTO RSAKEYS (id, n, e) VALUES(id, n, e)

    sout.str("");
    sout << "REPLACE INTO RSAKEYS(id, n, e) VALUES(" << id << ",\"" << n
         << "\",\"" << e << "\")";
    mysql_query(mysql, sout.str().c_str());
  }
#endif
  if (res != 0)
  {
    throw DatabaseException("DataBase::setLogin - REPLACE INTO ... failed - #",
                            res);
    return false;
  }
  
  res = mysql_query(mysql, "COMMIT");
  if (res != 0)
  {
    mysql_query(mysql, "ROLLBACK");
    throw DatabaseException("DataBase::setLogin - COMMIT Query Error #", res);
    return false;
  }

  return true;
}

bool DataBase::updateLogin(LoginInfo const& l)
{
  GetConnection gcon(this);

  MYSQL* mysql;
  int res;
  std::ostringstream sout;

  if (!l.isValid())
  {
    return false;
  }

  mysql = gcon.con->getConnection();

  sout << "UPDATE LOGINS SET ";
  if (l.getFailures() == 0)
  {
    /* login succeeded, so update last datetime and IP too */
    sout << "last_datetime=NOW(),"
         << "last_ip=\"" << l.getCurrentIP() << "\",";
  }
  sout << "failures=" << l.getFailures()
       << " WHERE login=\"" << l.getLogin() << "\"";

  res = mysql_query(mysql, sout.str().c_str());
  if (res != 0)
  {
    throw DatabaseException(
            "DatabaseHandler::updateLogin UPDATE LOGINS... failed - #",
            res);
    return false;
  }

  return true;
}

bool DataBase::deleteLogin(int ID)
{
  GetConnection gcon(this);
  MYSQL* mysql = gcon.con->getConnection();

  std::ostringstream sout;

  sout << "DELETE FROM LOGINS WHERE id = " << ID;
  mysql_query(mysql, sout.str().c_str());
  
  sout.str("");
  sout << "DELETE FROM PASSWORDS WHERE id = " << ID;
  mysql_query(mysql, sout.str().c_str());

#ifdef HAVE_CRYPTO
  sout.str("");
  sout << "DELETE FROM RSAKEYS WHERE id = " << ID;
  mysql_query(mysql, sout.str().c_str());
#endif

  return true;
}


bool DataBase::deleteLogin(std::string const &login)
{
  GetConnection gcon(this);
  MYSQL* mysql = gcon.con->getConnection();

  MYSQL_RES* result;
  MYSQL_ROW row;
  int res;
  std::ostringstream sout;

  // SELECT id FROM LOGINS WHERE login = <login>
  sout << "SELECT id FROM LOGINS WHERE login = \"" << login << "\"";
  res = mysql_query(mysql, sout.str().c_str());
  if (res != 0)
  {
    throw DatabaseException("DataBase::deleteLogin - Query Error #", res);
    return false;
  }

  result = mysql_store_result(mysql);
  if (!result)
  {
    throw DatabaseException(
            "DataBase::deleteLogin - Could not obtain Query Result");
    return false;
  }
  row = mysql_fetch_row(result);
  if (!row)
  {
    throw QueryException("DataBase::deleteLogin - No rows returned");
    return false;
  }

  int id = atoi(row[0]);
  mysql_free_result( result );

  /* We release the connection */
  gcon.Release();
  return deleteLogin(id);
}


bool DataBase::getAllUser(std::list<std::string> &alist)
{
  alist.clear();
  GetConnection gcon(this);
  MYSQL* mysql = gcon.con->getConnection();

  MYSQL_RES* result;
  MYSQL_ROW row;
  mysql_query(mysql, "SELECT login FROM LOGINS");
  result = mysql_store_result(mysql);

  if (!result)
  {
    throw DatabaseException(
            "DetaBase::getAllUser - Could not obtain user list");
    return false;
  }

  while ((row = mysql_fetch_row(result)))
  {
    alist.push_back(std::string((char*)row[0]));
  }

  mysql_free_result(result);
  return true;
}

bool DataBase::executeFileQuery(std::string const& path)
{
  std::ifstream file(path.c_str());
  std::string line;
  std::ostringstream sout;

  if (!file)
  {
    throw DatabaseException(
            "Database::executeFileQuery - Could not open the query file");
    return false;
  }

  GetConnection gcon(this);
  MYSQL* mysql = gcon.con->getConnection();

  try
  {
    while (std::getline(file, line))
    {
      sout << line;
    }
  }
  catch (std::exception e )
  {
    throw DatabaseException("DataBase::executeFileQuery - " +
                            (std::string)e.what());
    return false;
  }

  if (mysql_query(mysql, sout.str().c_str()) != 0)
  {
    throw DatabaseException("Database::executeFileQuery - " +
                            (std::string)mysql_error(mysql));
    return false;
  }

  return true;
}


bool DataBase::executeQuery(std::string const& myquery)
{
  GetConnection gcon(this);

  MYSQL* mysql = gcon.con->getConnection();

  if (mysql_query(mysql, myquery.c_str()) != 0)
  {
    throw DatabaseException("Database::executeQuery - " +
                            (std::string)mysql_error(mysql));
    return false;
  }
  return true;
}

bool DataBase::deleteTable(std::string const& table)
{
  GetConnection gcon(this);
  MYSQL* mysql = gcon.con->getConnection();

  std::ostringstream sout;
  int res;

  //DROP TABLE IF EXISTS <table>
  sout << "DROP TABLE IF EXISTS " << table;

  res = mysql_query(mysql, sout.str().c_str());
  if (res != 0)
  {
    throw DatabaseException("DataBase::deleteTable - Error " +
                            (std::string)mysql_error(mysql) + " #", res);
    return false;
  }
  else
  {
    glog_Common.Debug("DataBase::deleteTable - successful");
    return true;
  }
  return false;
}

DataBase::~DataBase()
{
  while(ConQueue.isAvailable())
  {
    delete ConQueue.Get();
  }
}
