/* $Id: DatabaseHandler.h,v 1.20 2002/03/18 09:05:24 mutable Exp $ */
/***************************************************************************
    copyright            : (C) 2001 by Benjamin Lerman
    email                : quisar@quisar.ambre.net
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
 * This class handles all interaction with the DataBase. It keeps always   *
 * a pool of connection with the mysql client that it give away to his own *
 * member when they asked. It is thread safe.                              *
 ***************************************************************************/
#ifndef ARIANNE_database_h_
#define ARIANNE_database_h_

#include <mysql.h>
#include <list>

#include <stdio.h>

#include "3_Manager/LoginInfo.h"
#include "Concurrent/Queue.h"


class MYSQLConnection;

/** Database class */
class DataBase
{
  // not copyable
  DataBase(DataBase const&);
  // not assignable
  DataBase& operator=(DataBase const&);
   
  public:
  friend class GetConnection;
  DataBase(int nbcon, std::string const& db, std::string const& host,
           std::string const& user, std::string const& passwd);
  ~DataBase();

  /** Retrieve LoginInfo from DB */
  bool getLogin(std::string const& login, LoginInfo& l);
  /** Store LoginInfo in DB */
  bool setLogin(LoginInfo const& l);
  /** Update LoginInfo */
  bool updateLogin(LoginInfo const& l);
  /** Delete login account using ID */
  bool deleteLogin(int ID);
  /** Delete login account using login name */
  bool deleteLogin(std::string const& login);
  /** Retrieve a list of users */
  bool getAllUser(std::list<std::string> &alist);
  bool executeFileQuery(std::string const& path);
  bool executeQuery(std::string const& query);
  /** Delete tabel from DB */
  bool deleteTable(std::string const& table);

  private:
  //TODO: Is it really necessary to use a Queue or just a std::queue will do
  //      the work?
  Queue<MYSQLConnection> ConQueue;
};


/** Wrapper class for MYSQL*.
    We need to add each MYSQL object to AList, so we need a C++ object */
class MYSQLConnection
{
  // not copyable
  MYSQLConnection(MYSQLConnection const&);
  // not assignable
  MYSQLConnection& operator=(MYSQLConnection const&);

  MYSQL* mysql;
  
  public:
  MYSQLConnection();
  MYSQLConnection(std::string const& db, std::string const& host,
                  std::string const& user, std::string const& passwd);
  ~MYSQLConnection();

  MYSQL* getConnection();
};


/** GetConnection class */
class GetConnection
{
  // not copyable
  GetConnection(GetConnection const&);
  // not assignable
  GetConnection& operator=(GetConnection const&);

  public:
  friend class DataBase;

  private: 
  GetConnection(DataBase *db)
  {
    this->db = db;
    con = db->ConQueue.Get();
  }

  ~GetConnection()
  {
    if (con != 0)
    {
      db->ConQueue.Add(con);
    }
  }

  void inline Release()
  {
    if (con != 0)
    {
      db->ConQueue.Add(con);
    }
    con = 0;
  }

  MYSQLConnection* Get()
  { 
    return con;
  }

  MYSQLConnection *con;
  DataBase *db;
};


/** DatabaseException: thrown on general Database error */
class DatabaseException : public Exception
{
  public:
    DatabaseException(std::string const& msg = "Database Error",
                      int err = 0): Exception()
    {
      message = msg;
      if (err != 0)
      {
        char buf[8];
        sprintf(buf, "%d", err);
        message += (std::string)buf;
      }
    }
};

/** Query Excpetion: thrown on an unexpected query result */
class QueryException : public Exception
{
  public:
    QueryException(std::string const& msg = "Query Error"): Exception()
    {
      message = msg;
    }
};

/** MYSQLConnectionException: thrown when connecting to the MYSQL server
    fails. */
class MYSQLConnectionException : public Exception
{
  public:
    MYSQLConnectionException(std::string const& msg = "MYSQLConnection Error: ",
                             std::string const& err = "UNKNOWN"):
                             Exception()
    {
      message = msg + err;
    }
};

#endif
