/* $id: Py2Attributes.cpp,v 1.3 2002/03/08 12:21:25 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2002 by Brian Rowe
    email                : briprowe@yahoo.com
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cassert>

#include "Py2Attributes.h"
#include "Exceptions.h"

Py2Attributes::Py2Attributes(PyObject* obj)
{
  assert(PyDict_Check(obj));
  
  *this = GetCAttribs(obj);
}

Py2Attributes::~Py2Attributes()
{

}

PyObject* Py2Attributes::GetPython(void)
{
  PyObject* retval = PyDict_New();

  std::list<std::string> attribs = this->ListAttribs();
  std::list<std::string>::iterator iter;
  
  for( iter=attribs.begin(); iter != attribs.end(); iter++ )
    {
      PyObject *key = PyString_FromString((*iter).c_str());
      std::string str = *iter;
      std::string const temp = Attributes::GetAttrib(str);
      PyObject *val = PyString_FromString(temp.c_str());
      
      if ( PyDict_SetItem(retval, key, val) == -1 )
	{
	  Py_XDECREF(retval);
	  Py_XDECREF(key);
	  Py_XDECREF(val);
	  throw("Error creating PyObject");
	}
 
     // Py_DECREF(key); Not necessary, since the retval "owns" the reference now
     // Py_DECREF(val);
    }
  
  return(retval);
}

Attributes Py2Attributes::GetCAttribs(PyObject* obj)
{
  assert(PyDict_Check(obj));

  PyObject* list = PyDict_Keys(obj);
  std::list<std::string> attribs;

  for( int i = 0; i < PyList_Size(list); i++ )
    {
      attribs.push_back(PyString_AsString(PyList_GetItem(list, i)));
    }

  std::list<std::string>::iterator iter;
  Attributes retval;

  for( iter=attribs.begin(); iter != attribs.end(); iter++)
    {
      std::string val = PyString_AsString(PyDict_GetItemString(obj,
                                          const_cast<char*>((*iter).c_str())));
      retval.PutAttrib(*iter, val);
    }

  Py_XDECREF(list);
  return retval;
}

void Py2Attributes::SetAttrib(PyObject* key, PyObject* val)
{
  assert(PyString_Check(key) && PyString_Check(val));
  
  std::string k = PyString_AsString(key);
  std::string v = PyString_AsString(val);

  this->PutAttrib(k, v);
}

PyObject* Py2Attributes::GetAttrib(PyObject* key)
{
  assert(PyString_Check(key));
  
  std::string tmp = PyString_AsString(key);
  std::string const val = Attributes::GetAttrib(tmp);
  
  PyObject* retval = PyString_FromString(val.c_str());

  return(retval);
}

bool Py2Attributes::operator==(Attributes const &rhs)
{
  return(Attributes(*this)==rhs);
}

bool Py2Attributes::operator==(PyObject *rhs)
{
  if( !PyDict_Check(rhs) )
    return(false);

  Py2Attributes tmp(rhs);

  return(tmp == (*this));
}

