#ifndef __PythonLanguage_h_
#define __PythonLanguage_h_

/***************************************************************************
    copyright            : (C) 2002 by Brian Rowe
    email                : briprowe@yahoo.com
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <string>
#include <vector>

#include "Python.h"

#include "Language.h"
#include "Exceptions.h"

class PythonLanguage : Language
{
 private:
  std::string pathToExecutable;

 public:
  PythonLanguage();
  PythonLanguage(const PythonLanguage &rhs);
  PythonLanguage(std::string const &path);

  virtual ~PythonLanguage();

  void Startup();   // throws PyStartException
  void Shutdown();  // throws PyStopException

  /** This changes the path that Python uses as a root when searching
      for scripts to load. */
  void AddPath(std::string path = ".");  // throws PyPathException
 
  PyObject* LoadModule(std::string modname = "");       // throws PyException
  PyObject* RunFunction(PyObject* module,               // throws PyException
                        std::string funcname,
                        PyObject* args);
  

  // Exceptions
  
  /** This is the basic exception that will be thrown on any Python
   *  error.  It knows how to get the error message from the Python
   *  system. It will return the error to the program in the same way 
   *  that a std::exception will.
   *  */
  /*
  class PyException: public Exception
    {
    private:
      //      PyObject *errType, *errValue, *traceback;
      //std::string errorMsg;
      //std::vector<PyObject*> &GetError();

    public:
      //PyException(const std::string err);
      //PyException(const char* err);
      PyException() { };
      virtual ~PyException();

      //      std::string what();
      }; */
    
   /** These are identical to PyException,  but I gave them different
   *  classes, because I want to be able to catch them in different
   *  catch() blocks.
   */
  class PyException : public Exception { };
  class PyStartException : public PyException { };
  class PyStopException : public PyException { };
  class PyPathException : public PyException { };

};

#endif
