/***************************************************************************
    copyright            : (C) 2002 by Brian Rowe
    email                : briprowe@yahoo.com
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "ScriptInterface.h"
#include <iostream>
#include <cassert>

// This is to help the "friend" functions find the ScriptInterface
ScriptInterface *script_inter = 0;

ScriptInterface::ScriptInterface(WorldManager *wptr)
{
  // There must only be 1 copyt of the ScriptInterface
  assert(!script_inter);
  
  worldman = wptr;
  script_inter = this;
}

ScriptInterface::ScriptInterface(const ScriptInterface& rhs)
{
  worldman = rhs.worldman;
  script_inter = this;
}

ScriptInterface::~ScriptInterface()
{
  script_inter = 0;
}

Object ScriptInterface::GetOb(Object::ID const& id)
{
  return(worldman->Get(id));
}

void ScriptInterface::AddOb(Object const& object)
{
  worldman->Add(object);
}

void ScriptInterface::DelOb(Object::ID const& id)
{
  worldman->Del(id);
}

void ScriptInterface::DelOb(Object const& object)
{
  worldman->Del(object);
}

std::list<Object> ScriptInterface::isInRangeOb(std::string const& Center, 
					     Uint32 Distance)
{
  return(worldman->isInRange(Center, Distance));
}

std::list<Object> ScriptInterface::isInRangeOb(Object const& object, 
					     Uint32 Distance)
{
  return(worldman->isInRange(object, Distance));
}

Object::ID ScriptInterface::GetValidIDOb()
{
  return(worldman->GetValidID());
}


std::vector<void*> ScriptInterface::GetPyPointers()
{
  std::vector<void*> ptrs;
  
  ptrs.push_back((void*)(&PyGetObject));
  ptrs.push_back((void*)(&PyAdd));
  ptrs.push_back((void*)(&PyDel));
  ptrs.push_back((void*)(&PyisInRange));
  ptrs.push_back((void*)(&PyGetValidID));

  return(ptrs);
}

PyObject* PyGetObject(PyObject* args)
{
  Object::ID id = PyString_AsString(PyTuple_GetItem(args, 0));
  Object obj = script_inter->GetOb(id);
  Py2Attributes pyob(obj);
  return(pyob.GetPython());
}

PyObject* PyAdd(PyObject* args)
{
  Py2Attributes pyob(PyTuple_GetItem(args, 0));
  Object obj(pyob);
  script_inter->AddOb(obj);
  PyObject *retval = PyTuple_New(0);
  return(retval);
}

void PyDel(PyObject* args)
{
  PyObject *param = PyTuple_GetItem(args, 0);
  if ( PyDict_Check(param) )
    {
      Py2Attributes pyob(param);
      Object obj(pyob);
      script_inter->DelOb(obj);
    }
  if ( PyString_Check(param) )
    {
      Object::ID id(PyString_AsString(param));
      script_inter->DelOb(id);
    }
}

PyObject* PyisInRange(PyObject* args)
{
  PyObject *first, *distance;
  std::list<Object> li;

  first = PyTuple_GetItem(args, 0);
  distance = PyTuple_GetItem(args, 1);
  
  if ( PyDict_Check(first) )
    {
      Py2Attributes pyob(first);
      Object obj(pyob);
      li = script_inter->isInRangeOb(obj, PyInt_AsLong(distance));
    }
  if ( PyString_Check(first) )
    {
      std::string id = PyString_AsString(first);
      
      li = script_inter->isInRangeOb(id, PyInt_AsLong(distance));
    }
  
  PyObject *retval=0;
  std::list<Object>::iterator iter;
  for( iter=li.begin(); iter != li.end(); iter++ )
    {
      Py2Attributes pyob = *iter;
      PyList_Append(retval, pyob.GetPython());
    }
  return(retval);
}

PyObject* PyGetValidID()
{
  Object::ID id = script_inter->GetValidIDOb();
  PyObject *retval = PyString_FromString(id.c_str());
  return(retval);
}
