/* $Id: smain.cpp,v 1.28 2002/04/05 17:00:23 arianne_rpg Exp $ */
/***************************************************************************
    copyright            : (C) 2000 by Miguel Angel Blanch Lardin
    email                : x5101920@fedro.ugr.es
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <csignal>

#include <signal.h>

#include "SDL.h"
#include "SDL_net.h"

#include "Arianneserver.h"
#include "Define.h"
#include "DataBase/DatabaseHandler.h"

#include "Parser/Parser.h"
#include "Log/Logger.h"
#include "Extern.h"
#include "InitPath.h"

/** The Arianne Ini file name */
#define INI_FILE "Server.xml"

// prototypes

void Initialization();
void Exit();
void sighandler(int sig);

// globals
typedef void (*SignalHandler_t)(int);
SignalHandler_t last_sighandler;

DataBase *GlobalDb = 0;
bool g_ConsoleEnabled=true;

void Initialization()
  {
  try 
    {
    // init path-settings
    if(!InitPath(INI_FILE))
      {
      std::cerr << "Error: Unable to find \"Server.xml\"." << std::endl;
      exit(1);
      }

    // init SDL and SDL_net
    SDL_Init(SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE);
    SDLNet_Init();

    // read the Database-settings from the config-file and init the database
    PersistentString pvHostName (g_etcdir+INI_FILE, "DataBase", "HostName");
    PersistentString pvUserName (g_etcdir+INI_FILE, "DataBase", "UserName");
    PersistentString pvPassword (g_etcdir+INI_FILE, "DataBase", "Password");
    PersistentString pvDatabase (g_etcdir+INI_FILE, "DataBase", "Database");
    GlobalDb = new DataBase(10,std::string(pvDatabase),std::string(pvHostName),std::string(pvUserName),std::string(pvPassword));
      

    /* We want Segmentation fault signal to have the default behaviour: Crash & Core dumped. */
    signal(SIGSEGV,SIG_DFL);

    /* Manipulate signal for accepting Control-C as finish key */
    last_sighandler=signal(SIGINT,sighandler);
    signal(SIGTERM, sighandler);
    }
  catch(std::exception& e)
    {
    glog_App.Debug(__FILE__,e.what(),Log::fatal_error);
    exit(1);
    }
  catch(...)
    {
    glog_App.Debug(__FILE__,"Uncaught unknown exception",Log::unmanaged_exception);
    exit(1);
    }
  }

void Exit()
  {
  try 
    {
    delete GlobalDb;
    GlobalDb = 0;

    SDLNet_Quit();
    SDL_Quit();
    }
  catch(std::exception& e)
    {
    glog_App.Debug(__FILE__,e.what(),Log::fatal_error);
    exit(1);
    }
  }

void sighandler(int sig)
  {
  static int i = 0;
  if(sig==SIGINT || sig==SIGTERM)
    {
    /* If its the first time, we try to shutdown gracefully */
    if(i++==0)
      {
      if(g_ConsoleEnabled)
        {
        glog_App.Debug("Type `exit' to shut down server.");
        }
      else
        {
        glog_App.Debug("Shutting down server due to user Request - (Ctrl+C)");
        }

      gWaitPointSemaphore.Post();
      gShutdownRequested=true;
      }
    else
      {
      glog_App.Debug("Killing server.",Log::fatal_error);
      exit(-1);
      }
    }
  else
    {
    (*last_sighandler)(sig);
    }
  }

bool ReadParams(int argc,char **argv)
  {
  // scan the commandline for parameters
  for(int i=1; i<argc; ++i)
    {
    if(std::string(argv[i]) == std::string("-h"))
      {
      std::cout << "SYNTAX: arianne-server [-nc] [-h]" << std::endl;
      return false;
      }
    else if(std::string(argv[i]) == std::string("-nc"))
      {
      g_ConsoleEnabled = false;
      }
    }
  return true;
  }

int main(int Num,char **arg)
  {
  Log::toFile("server.txt",true);

  if(!ReadParams(Num,arg))
    {
    return -1;
    }   

  Initialization();
  glog_App.Debug("Initialized Server");

  /* Just to control when server is closed. */
  try
    {
    ArianneServer server;
  
    server.Init();

    server.Run(g_ConsoleEnabled);
 
    server.Close();
    }
  catch (std::exception& e)
    {
    glog_App.Debug(e.what(),Log::managed_exception);
    }
  catch (...)
    {
    glog_App.Debug("Uncaught unknown exception",Log::unmanaged_exception);
    }  

  glog_App.Debug("Shutdowning down Server");

  Exit();
  return 0;
  }

