/* $Id: account-manager.cpp,v 1.19 2002/04/05 21:47:11 guidoj Exp $ */
/***************************************************************************
    copyright            : (C) 2001 by Benjamin Lerman
    email                : quisar@quisar.ambre.net
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
 * This application is used for filling the database with new accounts,    *
 * view the accounts and delete it, all without knowing SQL.               *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string>

#include "Crypto/Crypto.h"
#include "Crypto/MD5.h"
#include "3_Manager/LoginInfo.h"
#include "DataBase/DatabaseHandler.h"
#include "Path/FilePath.h"

#include <iostream>
#include <vector>

DataBase *db=0;

std::list<std::string>options;


#ifdef WIN32
std::istream& a_readline(std::istream& in, std::string & stri)
  {
  char *Buffer=new char[256];
  int size=0;

  in.getline(Buffer,256,'\n');

  for(int i=0;i<256;++i)
    {  
    ++size;  
    if(Buffer[i]=='\r') 
      {
      Buffer[i]=0;
      break;
      }
    if(Buffer[i]=='\n')
      {
      Buffer[i]=0;
      break;
      }
    if(Buffer[i]==0)
      {
      break;
      }
    }
  
  stri=Buffer;
  delete Buffer;

  return in;
  }
#endif


void init()
{
 options.push_back("1) Create the tables, or reinitialize it.");
 options.push_back("2) Create new account.                   ");
 options.push_back("3) Delete an account.                    ");
 options.push_back("4) View informations for an account.     ");
 options.push_back("5) List all accounts.                    ");
 options.push_back("6) Verify login procedure.               ");
 options.push_back("7) Remove all ObjectID.                  ");
}
  

void usage(std::string name)
{
 std::cout << "Usage: " << name << " -h -d [database] -u [dbuser] -p [dbpass]" << std::endl;
}

bool deleteuser()
{
 std::string login;

 std::cout << "Enter login: ";
 #ifndef WIN32 
 std::getline(std::cin, login);
 #else
 a_readline(std::cin, login);
 #endif
 std::cout << "Deleting " << login << ": ";
 if(db->deleteLogin(login))
  std::cout << "Success." << std::endl;
 else
 {
  std::cout << "Error..." << std::endl;
  return 1;
 }
 return 0;
}

bool newuser()
{
 std::string login;
 std::string junk;
 std::string password;
 std::string hash;
 std::string path;
#ifdef HAVE_CRYPTO
 RSAKey *key;
 int mode;
 RAND rand;
#endif
 LoginInfo l;

 std::cout << "Enter login: ";
 #ifndef WIN32
 std::getline(std::cin, login);
 #else
 a_readline(std::cin, login);
 #endif
 l.setLogin(login);
#ifdef HAVE_CRYPTO
 std::cout << "Enter mode (1:password,2:RSA): ";
 std::cin >> mode;
 #ifndef WIN32
 std::getline(std::cin, junk);
 #else
 a_readline(std::cin, junk);
 #endif

 switch(mode)
 {
  case 1:
#endif
   std::cout << "Enter password: ";
 #ifndef WIN32
   std::getline(std::cin, password);
 #else
 a_readline(std::cin, password);
 #endif
#ifdef HAVE_CRYPTO
   //  This line was for using MD5 Hash instead.
   //  hash= MD5Hash(password);
   SHA1::cryptPW(rand, password, hash);
   l.setHash(hash);
   break;
#else
   l.setHash(password);
#endif

#ifdef HAVE_CRYPTO
  case 2:
   key = new RSAKey();
   std::cout << "Enter file with the public key (type enter to generate a new key): ";
 #ifndef WIN32
   std::getline(std::cin, path);
 #else
 a_readline(std::cin, path);
 #endif
   if(path == "")
   {
    key->GenerateKey();
    std::cout << "Enter file to save private key: ";
 #ifndef WIN32
    std::getline(std::cin, path);
 #else
    a_readline(std::cin, path);
 #endif
    if(! key->SaveKey(path))
    {
     std::cerr << "Error: Unable to save the key." << std::endl;
     return 1;
    }
   }
   else
    if(! key->LoadKey(path))
    {
     std::cerr << "Error: Unable to read the key." << std::endl;
     return 1;
   }
   l.setKey(key);
   break;

  default:
   std::cerr << "Error: Mode must be 1 or 2." << std::endl;
   return 1;
 }
#endif

 if(l.isValid())
  if(db->setLogin(l))
   std::cout << "The new User is saved." << std::endl;
  else
  {
   std::cerr << "Error: Problems with the DataBase." << std::endl;
   return 1;
  }
 else
 {
  std::cerr << "Error: The login information are incorrects." << std::endl;
  return 1;
 }
 return 0;
}

bool createtables()
{
 std::vector<std::string> tables;
 std::string path;
 bool res=true;
 int i;

 tables.push_back("LOGINS");
 tables.push_back("PASSWORDS");
#ifdef HAVE_CRYPTO
 tables.push_back("RSAKEYS");
#endif
 
 std::cout << "Enter directory where the definitions are: ";
  #ifndef WIN32
std::getline(std::cin, path);
 #else
 a_readline(std::cin, path);
 #endif
 if(path=="")
   {
   path="./Dbs";
   }

 for(i=0; i<(int)tables.size(); ++i)
 {
  std::cout << "Creating " << tables[i] << ": ";
  db->deleteTable(tables[i]);
  std::string tmp=getValidFilePath(path + "/"+ tables[i]);
  if(db->executeFileQuery(tmp))
   std::cout << "Success." << std::endl;
  else
  {
   std::cout << "Failure." << std::endl;
   res = false;
  }
 }

return res;
}

bool listusers()
{
 std::list<std::string> users;
 std::cout << "Getting users: ";
 if(!db->getAllUser(users))
 {
  std::cout << "Failed." << std::endl;
  return false;
 }
 std::cout << "Success." << std::endl;
 std::cout << "Number of users: " << users.size() << std::endl;
 std::list<std::string>::iterator last = users.end();
 for(std::list<std::string>::iterator i = users.begin(); i != last; ++i)
  std::cout << *i << std::endl;
 return true;
}

bool getUser()
{
 std::string login;
 LoginInfo l;
 std::cout << "Enter login: ";
 #ifndef WIN32
 std::getline(std::cin, login);
 #else
 a_readline(std::cin, login);
 #endif
 std::cout << "Getting user " << login << ": ";
 if(!db->getLogin(login, l))
 {
   std::cout << "Failed." << std::endl;
   return false;
  }
 if(!l.isValid())
 {
  std::cout << "Login Information for " << login << " is invalid." << std::endl;
  return false;
 }
 std::cout << "Success." << std::endl;
 std::cout << "Login       : " << l.getLogin() << std::endl;
 std::cout << "Id          : " << l.getID() << std::endl;
 std::cout << "ObjectID    : " << l.getObjectID() << std::endl;
#ifdef HAVE_CRYPTO
 switch(l.getLoginType())
 {
  case LoginInfo::pass:
    std::cout << "Method      : Password" << std::endl;
    break;
  case LoginInfo::RSA:
    std::cout << "Method      : RSA" << std::endl;
    break;
  default:
    std::cout << "Method      : Invalid" << std::endl;
    break;
 }
#endif
 std::cout << "LastDateTime: " << l.getLastDateTime() << std::endl;
 std::cout << "LastIP      : " << l.getLastIP() << std::endl;
 std::cout << "Failures    : " << l.getFailures() << std::endl;
 return true;
}

bool verifyuser()
{
 std::string login;
 LoginInfo l;
 std::cout << "Enter login: ";
 #ifndef WIN32
 std::getline(std::cin, login);
 #else
 a_readline(std::cin, login);
 #endif
 std::cout << "Getting user " << login << ": ";
 if(!db->getLogin(login, l))
 {
   std::cout << "Failed." << std::endl;
   return false;
  }
 if(!l.isValid())
 {
  std::cout << "Login Information for " << login << " is invalid." << std::endl;
  return false;
 }
 std::cout << "Success." << std::endl;

 switch(l.getLoginType())
 {
  case LoginInfo::pass:
    {
      std::string pass;
      std::cout << "Enter Password: ";
 #ifndef WIN32
      std::getline(std::cin, pass);
 #else
 a_readline(std::cin, pass);
 #endif
#ifdef HAVE_CRYPTO
      if(SHA1::verifyPW(pass, l.getHash()))
#else
      if(pass == l.getHash())
#endif
      {
        std::cout << "Login correct." << std::endl;
        return true;
      }
      else
      {
        std::cout << "Login incorrect." << std::endl;
        return false;
      }
    }
    break;
#ifdef HAVE_CRYPTO
  case LoginInfo::RSA:
    {
      RSAKey *keypriv;
      RSAKey *keypub;
      RAND r;
      std::vector<unsigned char> signature;
      std::vector<unsigned char> message;
      std::string path;
      std::cout << "Enter path to the key: ";
 #ifndef WIN32
      std::getline(std::cin, path);
 #else
      a_readline(std::cin, path);
 #endif
      keypriv = new RSAKey(path);
      if(! keypriv->IsValid())
      {
        std::cout << "Invalid Key." << std::endl;
        return false;
      }
      keypub = l.getKey();
      r.GetRandomBytes(message, 20);
      keypriv->SignMessage(message, signature);
      if(keypub->VerifySignature(message, signature))
      {
        std::cout << "Signature is correct." << std::endl;
        delete keypriv;
        return true;
      }
      else
      {
        std::cout << "Signature is incorrect." << std::endl;
        delete keypriv;
        return false;
      }
    }
    break;
#endif
  default:
    std::cout << "Login method invalid." << std::endl;
    break;
 }
 return true;
}

bool removeoid()
{
    std::cout << "Removing all Object ID..." << std::endl;
    if(! db->executeQuery("UPDATE LOGINS SET objectid=NULL"))
    {
	std::cout << "Failed." << std::endl;
	return false;
    }
    else
	std::cout << "Success." << std::endl;
    return true;
}

bool treat_command(char c)
{
 switch(c)
 {
  case '1':
   createtables();
   return true;
  case '2':
   newuser();
  return true;
  case '3':
   deleteuser();
   return true;
  case '4':
   getUser();
   return true;
  case '5':
   listusers();
   return true;
  case '6':
   verifyuser();
   return true;
  case '7':
   removeoid();
   return true;
  case 'q':
  case 'Q':
   return false;
 }
 return true;
}

int main_nogui()
{
 bool cont=true;
 unsigned int i,j;
 unsigned int cols;
 std::list<std::string>::iterator it, last;
 std::string line;
 std::string AM="";
 std::string quit="";
 std::string vide="";
 std::string star="";

 if(options.size() == 0)
  return 0;

 it = options.begin();
 cols = (*it).size() + 4;
 i = (cols - 17) / 2;
 for(j=0; j<i; ++j)
  AM += '*';
 AM += " Account Manager ";
 for(j=AM.size(); j<cols; ++j)
  AM += '*';
 quit="* q) Quit";
 for(j=quit.size(); j<cols-1; ++j)
  quit+=' ';
 quit+='*';
 vide = '*';
 for(j=vide.size(); j<cols-1; ++j)
  vide+=' ';
 vide+='*';
 for(j=0; j<cols; ++j)
  star+='*';

 do {
  std::cout << AM << std::endl;
  for(it = options.begin(), last = options.end(); it != last; ++it)
   std::cout << "* " << *it << " *" << std::endl;
  std::cout << vide << std::endl;
  std::cout << quit << std::endl;
  std::cout << star << std::endl;
  std::cout << std::endl;
  std::cout << "Your choice: " << std::endl;
 #ifndef WIN32
  if(std::getline(std::cin, line) == NULL)
 #else
  if(a_readline(std::cin, line) == NULL)
 #endif
   cont = false;
  else
  {
   for(i=0; i<line.size() && (line[i] == ' ') || (line[i] == '\t') ; ++i);
   if(i<line.size())
    cont = treat_command(line[i]);

   if(cont)
   {
    std::cout << "Press ENTER to continue." << std::endl;
 #ifndef WIN32
    if(std::getline(std::cin, line) == NULL)
 #else
    if(a_readline(std::cin, line) == NULL)
 #endif
     cont = false;
   }
  }
 } while(cont);
 return 0;
}

int main(int argc, char **argv)
{
 std::string user="";
 std::string pass="";
 std::string database="arianne";
 int i;
 char *name, *tmp;
 int res = 0;

 name = argv[0];
 tmp = name;
 while(*tmp)
 {
  if(*tmp == getValidFilePath("/")[0])
  {
   ++tmp;
   if(*tmp)
    name = tmp;
  }
  if(*tmp)
   ++tmp;
 }
 init();
 for(i=1; i<argc; ++i)
 {
  if(std::string(argv[i]) == std::string("-h"))
  {
   usage(name);
   return 0;
  }
  else if(std::string(argv[i]) == std::string("-u"))
  {
   ++i;
   if(i == argc)
   {
    usage(name);
    return 1;
   }
   user = argv[i];
  }
  else if(std::string(argv[i]) == std::string("-p"))
  {
   ++i;
   if(i == argc)
   {
    usage(name);
    return 1;
   }
   pass = argv[i];
  }
  else if(std::string(argv[i]) == std::string("-d"))
  {
   ++i;
   if(i == argc)
   {
    usage(name);
    return 1;
   }
   database = argv[i];
  }
  else
  {
   usage(name);
   return 1;
  }
 }
 try {
     db = new DataBase(1, database,"localhost",user, pass);
 }
 catch(std::exception &e)
 {
     std::cerr << "Error: " << __FILE__ << ": " << e.what() << std::endl;
     exit(1);
 }

 try {
     res = main_nogui();
 }
 catch(std::exception &e)
 {
     std::cerr << "Error: " << e.what() << std::endl;
 }
 delete db;
 db = 0;

 return res;
}

