/* $Id: analyse-dump.cpp,v 1.12 2002/02/24 14:27:27 guidoj Exp $ */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <cstdio>
#include <vector>

#include "Define.h"
#include "DataTypes/Serializer.h"
#include "Conversion/Conversion.h"

#include "0_Network/GenericMessage.h"
#include "1_Message/Message.h"
#include "1_Message/MessagesTypes.h"
using std::cout;
using std::cerr;
using std::endl;

int dump_server=0;

void dump_send_version(GenericMessage const& gm)
{
    if(dump_server)
    {
	int major, medium, min1, min2;
	Message_S2C_SEND_VERSION mes;
	Serializer s(gm.Data);
	s >> mes;
	min2 = mes.GetVersion() & 255;
	min1 = (mes.GetVersion() >> 8) & 255;
	medium = (mes.GetVersion() >> 16) & 255;
	major  = (mes.GetVersion() >> 24);
	cout << "Version: " << major << "." << medium << "." << min1 << "." << min2 << endl;
    }
    else
    {
	cout << "Error: Client should not send this message..." << endl;
    }
}

void dump_nlogin(GenericMessage const& gm)
{
    Message_S2C_NLOGIN mes;
    Serializer s(gm.Data);
    s >> mes;
    switch(mes.GetSubType())
    {
	case ARIANNE_SEND_LOGIN:
	    cout << "Subtype: ARIANNE_SEND_LOGIN" << endl;
	    break;
	case ARIANNE_METHOD_PASS:
            cout << "Subtype: ARIANNE_METHOD_PASS" << endl;
	    break;
#ifdef HAVE_CRYPTO
	case ARIANNE_METHOD_RSA:
            cout << "Subtype: ARIANNE_METHOD_RSA" << endl;
	    break;
#endif
	case ARIANNE_SEND_PASS:
            cout << "Subtype: ARIANNE_SEND_PASS" << endl;
	    break;
#ifdef HAVE_CRYPTO
	case ARIANNE_SEND_W2:
	    cout << "Subtype: ARIANNE_SEND_W2" << endl;
	    break;
	case ARIANNE_SEND_M:
            cout << "Subtype: ARIANNE_SEND_M" << endl;
	    break;
	case ARIANNE_SEND_S_M_:
            cout << "Subtype: ARIANNE_SEND_S_M_" << endl;
	    break;
#endif
	case ARIANNE_SEND_CONFIRM:
            cout << "Subtype: ARIANNE_SEND_CONFIRM" << endl;
	    break;
    }
    if(dump_server)
    {
	switch(mes.GetSubType())
	{
	    case ARIANNE_METHOD_PASS:
		break;

#ifdef HAVE_CRYPTO
	    case ARIANNE_METHOD_RSA:
		{
		    std::string hw1;
		    BinaryVectorToHexString(mes.GetData(), hw1);
		    cout << "H(W1) = 0x" << hw1 << endl;
		}
		break;

	    case ARIANNE_SEND_M:
		{
		    std::string m;
		    BinaryVectorToHexString(mes.GetData(), m);
		    cout << "M = 0x" << m << endl;
		}
		break;
#endif

	    case ARIANNE_SEND_CONFIRM:
		{
		    Uint32 pid;
		    NetworkVectorToInt(mes.GetData(), pid);
		    cout << "Login completed with pid: " << pid << "." << endl;
		}
		break;

	    default:
		cout << "Error: Server should not use this Subtype." << endl;
		break;
	}
    }
    else
    {
	switch(mes.GetSubType())
	{
	    case ARIANNE_SEND_LOGIN:
		{
		    std::string login;
		    VectorToString(mes.GetData(), login);
		    cout << "Login: " << login << endl;
		}
		break;

	    case ARIANNE_SEND_PASS:
		{
		    std::string pass;
		    VectorToString(mes.GetData(), pass);
		    cout << "Pass: " << pass << endl;
		}
		break;

#ifdef HAVE_CRYPTO
	    case ARIANNE_SEND_W2:
		{
		    std::string w2;
		    BinaryVectorToHexString(mes.GetData(), w2);
		    cout << "W2 = 0x" << w2 << endl;
		}
		break;

	    case ARIANNE_SEND_S_M_:
		{
		    std::string s_m_;
		    BinaryVectorToHexString(mes.GetData(), s_m_);
		    cout << "S(M) = 0x" << s_m_ << endl;
		}
		break;
#endif

	    default:
		cout << "Error: Client should not use this Subtype." << endl;
		break;
	}
    }
}

void dump_logout(GenericMessage const& gm)
{
    if(dump_server)
    {
	Message_S2C_LOGOUT mes;
	Serializer s(gm.Data);
	s >> mes;
	cout << "Bye String is: " << mes.GetByeString() << endl;
    }
    else
    {
	Message_C2S_LOGOUT mes;
	Serializer s(gm.Data);
	s >> mes;
	cout << "Statistics are: " << mes.GetStatistics() << endl;
    }
    
}

void dump_systinfo(GenericMessage const& gm)
{
    if(dump_server)
    {
	cout << "Error: Server should not send this Message." << endl;
    }
    else
    {
	Message_C2S_SYSTEM_INFO mes;
	Serializer s(gm.Data);
	s >> mes;
	cout << "Cpu: " << mes.GetCpu() << endl;
	cout << "OS:  " << mes.GetOs() << endl;
    }
}

void dump_perception(GenericMessage const& gm)
{
    if(dump_server)
    {
	int i;
	AListSerializable<Object> list;
	Object *ob;
	Perception perc;
	Message_S2C_PERCEPTION mes;
	Serializer s(gm.Data);
	s >> mes;
	cout << "uID: " << mes.GetuID() << endl;
	list = mes.GetObjectsModified();
	cout << "Size of ObjectsModified: " << list.Size() << endl;
	i=0;
	while(! list.isEmpty())
	{
	    ob = list.Remove(0);
	    cout << "Object n" << i << ": " << *ob << endl;
	    cout << "m_id: " << ob->m_id << endl;
	    cout << "m_type: " << ob->m_type << endl;
	    cout << "Position: x: " << ob->m_pos.m_x << " y: " << ob->m_pos.m_y << " z: " << ob->m_pos.m_z << endl;
	    cout << "Orientation: m_yaw: " << (Uint32) ob->m_orie.m_yaw << " m_roll: " << (Uint32) ob->m_orie.m_roll << " m_pitch: " << (Uint32) ob->m_orie.m_pitch << endl;
	    cout << "GFXResume: m_baseType: " << (Uint32) ob->m_gfx.m_baseType << " m_modificator: " << (Uint32) ob->m_gfx.m_modificator << " m_damageMod: " << (Uint32) ob->m_gfx.m_damageMod << endl;
	    cout << "Volume: Height: " << (Uint32) ob->m_vol.m_a << " Radius: " << (Uint32) ob->m_vol.m_b << endl;
	    ++i;
	    delete(ob);
	}
	list = mes.GetObjectsDeleted();
	cout << "Size of ObjectsDeleted: " << list.Size() << endl;
	i=0;
	while(! list.isEmpty())
	{
	    ob = list.Remove(0);
	    cout << "Object n" << i << ": " << endl;
	    cout << "m_id: " << ob->m_id << endl;
	    cout << "m_type: " << ob->m_type << endl;
	    cout << "Position: x: " << ob->m_pos.m_x << " y: " << ob->m_pos.m_y << " z: " << ob->m_pos.m_z << endl;
	    cout << "Orientation: m_yaw: " << (Uint32) ob->m_orie.m_yaw << " m_roll: " << (Uint32) ob->m_orie.m_roll << " m_pitch: " << (Uint32) ob->m_orie.m_pitch << endl;
	    cout << "GFXResume: m_baseType: " << (Uint32) ob->m_gfx.m_baseType << " m_modificator: " << (Uint32) ob->m_gfx.m_modificator << " m_damageMod: " << (Uint32) ob->m_gfx.m_damageMod << endl;
	    cout << "Volume: Height: " << (Uint32) ob->m_vol.m_a << " Radius: " << (Uint32) ob->m_vol.m_b << endl;
	    ++i;
	    delete(ob);
	}
    }
    else
    {
	cout << "Error: Client should not send this Message." << endl;
    }
}

int main(int argc, char **argv)
{
    std::vector<Uint8> buf;

    if(argc != 2)
    {
	cerr << "Usage: " << argv[0] << " file" << endl;
	exit(1);
    }

    int fd = open(argv[1], O_RDONLY);
    int nb;
    char tmpbuf[1024];
    int sizebuf=0;
    do
    {
	nb = read(fd, tmpbuf, 1024);
	if(nb > 0)
	{
	    buf.resize(sizebuf + nb);
	    std::copy(tmpbuf, tmpbuf+nb, buf.begin() + sizebuf);
	    sizebuf += nb;
	}
    } while(nb > 0);
    if(buf.size() >= ARIANNE_PROMPT.size())
    {
	std::string tmp;
	tmp.resize(ARIANNE_PROMPT.size());
	std::copy(buf.begin(), buf.begin() + ARIANNE_PROMPT.size(), tmp.begin());
	if(tmp == ARIANNE_PROMPT)
	{
	    cout << "Server Dump: Removing prompt." << endl;
	    buf.erase(buf.begin(), buf.begin() + ARIANNE_PROMPT.size());
	    dump_server = 1;
	}
    }
    while(buf.size()>=6)
    {
	GenericMessage gm;
	Serializer s;
	Uint32 size=0;
	size = buf[2]<<24;
	size +=buf[3]<<16;
	size +=buf[4]<<8;
	size +=buf[5];

	if(size > buf.size() - 6)
	{
	    cerr << "Oups the dump is incorrect..." << endl;
	    exit(1);
	}
	std::vector<Uint8>::iterator i;
	for(i=buf.begin(); i!= buf.begin()+size+6; ++i)
	    s.push(*i);
	buf.erase(buf.begin(), buf.begin()+size+6);
	s >> gm;
	cout << "----------------------------" << endl;
	switch(gm.Type)
	{
	    case ARIANNE_SEND_VERSION:
		cout << "Message type: ARIANNE_SEND_VERSION" << endl;
		cout << "Message Flag: " << Uint32(gm.Flags) << endl;
		dump_send_version(gm);
		break;

	    case ARIANNE_NLOGIN:
		cout << "Message type: ARIANNE_NLOGIN" << endl;
		cout << "Message Flag: " << Uint32(gm.Flags) << endl;
		dump_nlogin(gm);
		break;

	    case ARIANNE_LOGOUT:
		cout << "Message type: ARIANNE_LOGOUT" << endl;
		cout << "Message Flag: " << Uint32(gm.Flags) << endl;
		dump_logout(gm);
		break;

	    case ARIANNE_SYSTEM_INFO:
		cout << "Message type: ARIANNE_SYSTEM_INFO" << endl;
		cout << "Message Flag: " << Uint32(gm.Flags) << endl;
		dump_systinfo(gm);
		break;

	    case ARIANNE_ACTION:
		cout << "Message type: ARIANNE_ACTION" << endl;
		cout << "Message Flag: " << Uint32(gm.Flags) << endl;
		break;

	    case ARIANNE_PERCEPTION:
		cout << "Message type: ARIANNE_PERCEPTION" << endl;
		cout << "Message Flag: " << Uint32(gm.Flags) << endl;
		dump_perception(gm);
		break;

	    case ARIANNE_POISON:
		cout << "Message type: ARIANNE_POISON" << endl;
		cout << "Message Flag: " << Uint32(gm.Flags) << endl;
		break;

	    default:
		cout << "Message type Unknown: " << gm.Type << endl;
		cout << "Message Flag: " << Uint32(gm.Flags) << endl;
		break;
	}
	cout << "----------------------------" << endl;
    }
}
